Package: ViSEAGO
Version: 1.24.0
Title: ViSEAGO: a Bioconductor package for clustering biological
        functions using Gene Ontology and semantic similarity
Author: Aurelien Brionne [aut, cre],
        Amelie Juanchich [aut],
        Christelle hennequet-antier [aut]
Maintainer: Aurelien Brionne <aurelien.brionne@inrae.fr>
Imports: data.table, AnnotationDbi, dendextend, dynamicTreeCut,
        GOSemSim, GO.db, heatmaply, topGO, AnnotationForge, DT,
        DiagrammeR, R.utils, RColorBrewer, UpSetR, biomaRt, fgsea,
        ggplot2, htmlwidgets, igraph, methods, plotly, scales,
        ComplexHeatmap, circlize
Depends: R (>= 3.6)
Suggests: htmltools, org.Mm.eg.db, limma, Rgraphviz, BiocStyle, knitr,
        rmarkdown, corrplot, remotes, BiocManager, stats, utils,
        grDevices, processx
Encoding: UTF-8
Description: The main objective of ViSEAGO package is to carry out a data mining of biological functions and establish links between genes involved in the study. We developed ViSEAGO in R to facilitate functional Gene Ontology (GO) analysis of complex experimental design with multiple comparisons of interest. It allows to study large-scale datasets together and visualize GO profiles to capture biological knowledge. The acronym stands for three major concepts of the analysis: Visualization, Semantic similarity and Enrichment Analysis of Gene Ontology. It provides access to the last current GO annotations, which are retrieved from one of NCBI EntrezGene, Ensembl or Uniprot databases for several species. Using available R packages and novel developments, ViSEAGO extends classical functional GO analysis to focus on functional coherence by aggregating closely related biological themes while studying multiple datasets at once. It provides both a synthetic and detailed view using interactive functionalities respecting the GO graph structure and ensuring functional coherence supplied by semantic similarity. ViSEAGO has been successfully applied on several datasets from different species with a variety of biological questions. Results can be easily shared between bioinformaticians and biologists, enhancing reporting capabilities while maintaining reproducibility.
VignetteBuilder: knitr
License: GPL-3 bioconductor.org
URL:
        https://www.bioconductor.org/packages/release/bioc/html/ViSEAGO.html,
        https://forgemia.inra.fr/UMR-BOA/ViSEAGO
BugReports: https://forgemia.inra.fr/UMR-BOA/ViSEAGO/issues
biocViews: Software, Annotation, GO, GeneSetEnrichment,
        MultipleComparison, Clustering, Visualization
RoxygenNote: 7.3.2
Collate: 'genomic_ressource.R' 'Bioconductor2GO.R' 'Ensembl2GO.R'
        'EntrezGene2GO.R' 'EntrezGene_orthologs.R' 'enrich_GO_terms.R'
        'GO_SS.R' 'GO_clusters.R' 'GOclusters_heatmap.R' 'GOcount.R'
        'GOterms_heatmap.R' 'MDSplot.R' 'Uniprot2GO.R' 'Upset.R'
        'ViSEAGO.R' 'annotate.R' 'available_organisms.R' 'gene2GO.R'
        'build_GO_SS.R' 'clusters_cor.R' 'compare_clusters.R'
        'compute_SS_distances.R' 'create_topGOdata.R' 'custom2GO.R'
        'datasets.R' 'fgsea.R' 'merge_enrich_terms.R' 'overlapper.R'
        'pkgdiagram.R' 'runfgsea.R' 'show_heatmap.R' 'show_table.R'
        'taxonomy.R'
git_url: https://git.bioconductor.org/packages/ViSEAGO
git_branch: RELEASE_3_22
git_last_commit: 9020dee
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Packaged: 2025-10-30 07:08:33 UTC; biocbuild

# Generated by roxygen2: do not edit by hand

export(Bioconductor2GO)
export(Custom2GO)
export(Ensembl2GO)
export(EntrezGene2GO)
export(EntrezGene_orthologs)
export(Uniprot2GO)
export(overLapper)
export(pkgdiagram)
export(taxonomy)
exportMethods(GOclusters_heatmap)
exportMethods(GOcount)
exportMethods(GOterms_heatmap)
exportMethods(MDSplot)
exportMethods(Upset)
exportMethods(annotate)
exportMethods(available_organisms)
exportMethods(build_GO_SS)
exportMethods(clusters_cor)
exportMethods(compare_clusters)
exportMethods(compute_SS_distances)
exportMethods(create_topGOdata)
exportMethods(merge_enrich_terms)
exportMethods(runfgsea)
exportMethods(show_heatmap)
exportMethods(show_table)
import(AnnotationDbi)
import(GO.db)
import(GOSemSim)
import(biomaRt)
import(data.table)
import(dendextend)
import(fgsea)
import(topGO)
importFrom(AnnotationDbi,as.list)
importFrom(AnnotationDbi,keys)
importFrom(AnnotationDbi,select)
importFrom(AnnotationDbi,species)
importFrom(AnnotationForge,loadAnnDbPkgIndex)
importFrom(ComplexHeatmap,Heatmap)
importFrom(ComplexHeatmap,max_text_width)
importFrom(ComplexHeatmap,rowAnnotation)
importFrom(DT,datatable)
importFrom(DiagrammeR,grViz)
importFrom(GO.db,GO.db)
importFrom(GO.db,GOBPANCESTOR)
importFrom(GO.db,GOBPOFFSPRING)
importFrom(GO.db,GOCCANCESTOR)
importFrom(GO.db,GOCCOFFSPRING)
importFrom(GO.db,GOMFANCESTOR)
importFrom(GO.db,GOMFOFFSPRING)
importFrom(R.utils,gunzip)
importFrom(RColorBrewer,brewer.pal)
importFrom(UpSetR,upset)
importFrom(biomaRt,getBM)
importFrom(biomaRt,listDatasets)
importFrom(biomaRt,listEnsembl)
importFrom(biomaRt,listEnsemblGenomes)
importFrom(biomaRt,listFilters)
importFrom(biomaRt,useDataset)
importFrom(biomaRt,useEnsembl)
importFrom(biomaRt,useEnsemblGenomes)
importFrom(circlize,colorRamp2)
importFrom(data.table,":=")
importFrom(data.table,.N)
importFrom(data.table,.SD)
importFrom(data.table,data.table)
importFrom(data.table,fread)
importFrom(data.table,fwrite)
importFrom(data.table,like)
importFrom(data.table,melt)
importFrom(data.table,melt.data.table)
importFrom(data.table,rbindlist)
importFrom(data.table,setorderv)
importFrom(dendextend,branches_attr_by_clusters)
importFrom(dendextend,get_leaves_attr)
importFrom(dendextend,get_nodes_attr)
importFrom(dendextend,set)
importFrom(dynamicTreeCut,cutreeDynamic)
importFrom(ggplot2,scale_fill_gradient)
importFrom(ggplot2,scale_fill_gradient2)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,dev.off)
importFrom(grDevices,png)
importFrom(graphics,text)
importFrom(heatmaply,heatmaply)
importFrom(htmlwidgets,JS)
importFrom(igraph,compare)
importFrom(methods,"slot<-")
importFrom(methods,is)
importFrom(methods,new)
importFrom(methods,slot)
importFrom(plotly,add_heatmap)
importFrom(plotly,add_markers)
importFrom(plotly,export)
importFrom(plotly,layout)
importFrom(plotly,orca)
importFrom(plotly,plot_ly)
importFrom(scales,rescale)
importFrom(stats,as.dendrogram)
importFrom(stats,as.dist)
importFrom(stats,cmdscale)
importFrom(stats,cor)
importFrom(stats,cutree)
importFrom(stats,dist)
importFrom(stats,end)
importFrom(stats,hclust)
importFrom(stats,na.omit)
importFrom(stats,order.dendrogram)
importFrom(stats,start)
importFrom(topGO,inverseList)
importFrom(utils,combn)
importFrom(utils,download.file)
importFrom(utils,installed.packages)
importFrom(utils,stack)
importFrom(utils,tail)
importFrom(utils,unstack)

<img src="./inst/extdata/univ.png" align="right"/>
<img src="./inst/extdata/boa.png" align="right"/>
<img src="./inst/extdata/inra.png" align="right"/>

# ViSEAGO: a Bioconductor package for clustering biological functions using Gene Ontology and semantic similarity.

The main objective of ViSEAGO package is to carry out a data mining of biological functions and establish links between genes involved in the study. We developed ViSEAGO in R to facilitate functional Gene Ontology (GO) analysis of complex experimental design with multiple comparisons of interest. It allows to study large-scale datasets together and visualize GO profiles to capture biological knowledge. The acronym stands for three major concepts of the analysis: **Vi**sualization, **S**emantic similarity and **E**nrichment **A**nalysis of **G**ene **O**ntology. It provides access to the last current GO annotations, which are retrieved from one of NCBI EntrezGene, Ensembl or Uniprot databases for several species. Using available R packages and novel developments, ViSEAGO extends classical functional GO analysis to focus on functional coherence by aggregating closely related biological themes while studying multiple datasets at once. It provides both a synthetic and detailed view using interactive functionalities respecting the GO graph structure and ensuring functional coherence supplied by semantic similarity. ViSEAGO has been successfully applied on several datasets from different species with a variety of biological questions. Results can be easily shared between bioinformaticians and biologists, enhancing reporting capabilities while maintaining reproducibility.

## Installation

```r
# install package from Bioconductor
BiocManager::install("ViSEAGO")

## install package from gitLab
remotes::install_gitlab(
    "aurelien.brionne/ViSEAGO",
    host = "forgemia.inra.fr",
    build_opts = c("--no-resave-data", "--no-manual")
)

## install package from gitLab alternative
    # clone package (from prompt)
    git clone https://forgemia.inra.fr/umr-boa/viseago.git

    # build package (from R console) 
    devtools::build("ViSEAGO")

    # install package (from R console)
    install.packages("ViSEAGO_1.7.3.tar.gz", repos = NULL, type = "source")
```

## Citation

```r
citation("ViSEAGO")
```

## Overview

![](./inst/extdata/figure2_BMCBioData.png)

## ----setup,include=FALSE------------------------------------------------------
# load ViSEAGO and mouse db package
library(ViSEAGO)

# knitr document options
knitr::opts_chunk$set(
    eval=FALSE,echo=TRUE,fig.pos = 'H',
    fig.width=6,message=FALSE,comment=NA,warning=FALSE
)

## ----geneList_input,eval=TRUE-------------------------------------------------
# load gene identifiants and padj test results from Differential Analysis complete tables
PregnantvsLactate<-data.table::fread(
    system.file(
        "extdata/data/input",
        "pregnantvslactate.complete.txt",
        package = "ViSEAGO"
    ),
    select = c("Id","padj")
)

VirginvsLactate<-data.table::fread(
     system.file(
        "extdata/data/input",
        "virginvslactate.complete.txt",
        package = "ViSEAGO"
    ),
    select = c("Id","padj")
)

VirginvsPregnant<-data.table::fread(
    system.file(
        "extdata/data/input",
        "virginvspregnant.complete.txt",
        package = "ViSEAGO"
    ),
    select = c("Id","padj")
)

# rank Id based on statistical value (BH padj in this example)
data.table::setorder(PregnantvsLactate,padj)
data.table::setorder(VirginvsLactate,padj)
data.table::setorder(VirginvsPregnant,padj)

## ----geneList_input-head,echo=FALSE,eval=TRUE---------------------------------
# show the ten first lines of genes_DE (same as genes_ref)
PregnantvsLactate

## ----Genomic-ressources-------------------------------------------------------
# # connect to Bioconductor
# Bioconductor<-ViSEAGO::Bioconductor2GO()
# 
# # load GO annotations from Bioconductor
# myGENE2GO<-ViSEAGO::annotate(
#     "org.Mm.eg.db",
#     Bioconductor
# )

## ----Genomic-ressources_show--------------------------------------------------
# # display summary
# myGENE2GO

## ----Genomic-ressources_display,echo=FALSE,eval=TRUE--------------------------
cat(
"- object class: gene2GO
- database: Bioconductor
- stamp/version: 2019-Jul10
- organism id: org.Mm.eg.db

GO annotations:
- Molecular Function (MF): 22707 annotated genes with 91986 terms (4121 unique terms)
- Biological Process (BP): 23210 annotated genes with 164825 terms (12224 unique terms)
- Cellular Component (CC): 23436 annotated genes with 107852 terms (1723 unique terms)"
)

## ----Enrichment_data----------------------------------------------------------
# # perform fgseaMultilevel tests
# BP_PregnantvsLactate<-ViSEAGO::runfgsea(
#     geneSel=PregnantvsLactate,
#     ont="BP",
#     gene2GO=myGENE2GO,
#     method ="fgseaMultilevel",
#     params = list(
#         scoreType = "pos",
#          minSize=5
#     )
# )
# 
# BP_VirginvsLactate<-ViSEAGO::runfgsea(
#     geneSel=VirginvsLactate,
#     gene2GO=myGENE2GO,
#     ont="BP",
#     method ="fgseaMultilevel",
#     params = list(
#         scoreType = "pos",
#          minSize=5
#     )
# )
# 
# BP_VirginvsPregnant<-ViSEAGO::runfgsea(
#     geneSel=VirginvsPregnant,
#     gene2GO=myGENE2GO,
#     ont="BP",
#     method ="fgseaMultilevel",
#     params = list(
#         scoreType = "pos",
#          minSize=5
#     )
# )

## ----Enrichment_merge---------------------------------------------------------
# # merge fgsea results
# BP_sResults<-ViSEAGO::merge_enrich_terms(
#     cutoff=0.01,
#     Input=list(
#         PregnantvsLactate="BP_PregnantvsLactate",
#         VirginvsLactate="BP_VirginvsLactate",
#         VirginvsPregnant="BP_VirginvsPregnant"
#     )
# )

## ----Enrichment_merge_show----------------------------------------------------
# # display a summary
# BP_sResults

## ----Enrichment_merge_display,echo=FALSE,eval=TRUE----------------------------
cat(
"- object class: enrich_GO_terms
- ontology: BP
- method: fgsea
- summary:
 PregnantvsLactate
    method : fgseaMultilevel
    sampleSize : 101
    minSize : 5
    maxSize : Inf
    eps : 0
    scoreType : pos
    nproc : 0
    gseaParam : 1
    BPPARAM : fgseaMultilevel
    absEps : 101
 VirginvsLactate
    method : fgseaMultilevel
    sampleSize : 101
    minSize : 5
    maxSize : Inf
    eps : 0
    scoreType : pos
    nproc : 0
    gseaParam : 1
    BPPARAM : fgseaMultilevel
    absEps : 101
 VirginvsPregnant
    method : fgseaMultilevel
    sampleSize : 101
    minSize : 5
    maxSize : Inf
    eps : 0
    scoreType : pos
    nproc : 0
    gseaParam : 1
    BPPARAM : fgseaMultilevel
    absEps : 101- enrichment pvalue cutoff:
        PregnantvsLactate : 0.01
        VirginvsLactate : 0.01
        VirginvsPregnant : 0.01
- enrich GOs (in at least one list): 184 GO terms of 3 conditions.
        PregnantvsLactate : 67 terms
        VirginvsLactate : 58 terms
        VirginvsPregnant : 64 terms"
)

#' @title Store organisms GO annotations from custom database file.
#' @description  Store the available species and current GO annotations from a custom table file
#' @import data.table
#' @import AnnotationDbi
#' @import GO.db
#' @family genomic_ressource
#' @param file custom GO annotation file
#' @details This function load a custom GO annotation database table that must contain columns:
#' \describe{
#'      \item{taxid}{custom taxonomic identifiants}
#'      \item{gene_id}{custom gene identifiants}
#'      \item{gene_symbol}{custom gene symbols}
#'      \item{GOID}{Known GO identifiants (see \code{select(GO.db,columns=columns(GO.db),keys=keys(GO.db))}}
#'      \item{evidence}{Known GO \href{http://geneontology.org/page/guide-go-evidence-codes}{evidence codes}}
#' }
#' @return a  \code{\link{genomic_ressource-class}} object required by \code{\link{annotate}}.
#' @references
#' Matt Dowle and Arun Srinivasan (2017). data.table: Extension of `data.frame`. R package version 1.10.4. https://CRAN.R-project.org/package=data.table.
#' @include genomic_ressource.R
#' @examples
#' \dontrun{
#' # Download custom GO annotations
#' Custom<-ViSEAGO::Custom2GO(
#'     system.file(
#'         "extdata/customfile.txt",
#'         package = "ViSEAGO"
#'     )
#' )
#' }
#' @export
Custom2GO=function(file){

    # read the file (linux and windows)
    gene2go<-fread(file)

    # check columns name
    if(!all(c("taxid","gene_id","gene_symbol","GOID","evidence")%in%names(gene2go))){
        stop('custom annotation file columns names required: "taxid","gene_id","gene_symbol","GOID","evidence"')
    }

    # convert columns in character
    gene2go[,
        `:=`(
            taxid=as.character(gene2go$taxid),
            gene_id=as.character(gene2go$gene_id)
        )
    ]

    # check GOID validity
    if(!all(gene2go$GOID%in%keys(GO.db))){
        #stop('GOID must be a valid identifiant. see select(GO.db,columns=columns(GO.db),keys=keys(GO.db)')
        warning('Invalid GOIDs have been removed. see select(GO.db,columns=columns(GO.db),keys=keys(GO.db)')

        # load GO database
        GO<-select(GO.db,columns=columns(GO.db),keys=keys(GO.db))

        # subset valid GOID
        gene2go<-gene2go[GOID%in%GO$GOID]
    }

    # extract Go category
    GO<-select(GO.db,columns=c("GOID","ONTOLOGY"),keys=unique(gene2go$GOID))

    # renanme ontology column
    names(GO)[2]<-"category"

    # merge with Gene2GO
    gene2go<-merge(gene2go,GO,by="GOID",all.x=TRUE,sort=FALSE)

    ###################
    # return data genomic_ressource class object
    new(
        "genomic_ressource",
        db="Custom",
        stamp=file,
        data=gene2go,
        organisms=data.table(taxid=unique(gene2go$taxid))
    )
}

#' @title Heatmap to compare partitions
#' @description Build an interactive heatmap of the common GO terms frequency between several partitions.
#' @importFrom data.table data.table
#' @importFrom plotly plot_ly add_heatmap layout
#' @importFrom utils stack unstack tail
#' @family GO_clusters
#' @param clusters  a \code{list} of named \code{\link{GO_clusters-class}} objects,
#' from \code{\link{GOterms_heatmap}} or \code{\link{GOclusters_heatmap}} methods.
#' @details
#' Build an interactive heatmap of common GO terms frequency between partitions from several
#' \code{\link{GO_clusters-class}} objects.
#' @return an interactive javascript heatmap.
#' @references
#'  Carson Sievert, Chris Parmer, Toby Hocking, Scott Chamberlain, Karthik Ram, Marianne Corvellec and Pedro Despouy (2017).
#'  plotly: Create Interactive Web Graphics via 'plotly.js'. R package version 4.6.0. https://CRAN.R-project.org/package=plotly
#' @include GO_clusters.R
#' @examples
#' # load example object
#' data(
#'     myGOs,
#'     package="ViSEAGO"
#' )
#'
#' \dontrun{
#' # compute Semantic Similarity (SS)
#' myGOs<-ViSEAGO::compute_SS_distances(
#'     myGOs,
#'     distance=c("Resnik","Lin","Rel","Jiang","Wang")
#' )
#'
#' # Resnik distance GO terms heatmap
#' Resnik_clusters_wardD2<-ViSEAGO::GOterms_heatmap(
#'     myGOs,
#'     showIC=TRUE,
#'     showGOlabels=TRUE,
#'     GO.tree=list(
#'         tree=list(
#'             distance="Resnik",
#'             aggreg.method="ward.D2"
#'         ),
#'         cut=list(
#'             dynamic=list(
#'                 deepSplit=2,
#'                 minClusterSize =2
#'             )
#'         )
#'     ),
#'     samples.tree=NULL
#' )
#'
#' # Lin distance GO terms heatmap
#' Lin_clusters_wardD2<-ViSEAGO::GOterms_heatmap(
#'     myGOs,
#'     showIC=TRUE,
#'     showGOlabels=TRUE,
#'     GO.tree=list(
#'         tree=list(
#'             distance="Lin",
#'             aggreg.method="ward.D2"
#'         ),
#'         cut=list(
#'             dynamic=list(
#'                 deepSplit=2,
#'                 minClusterSize =2
#'             )
#'         )
#'     ),
#'     samples.tree=NULL
#' )
#'
#' # Resnik distance GO terms heatmap
#' Rel_clusters_wardD2<-ViSEAGO::GOterms_heatmap(
#'     myGOs,
#'     showIC=TRUE,
#'     showGOlabels=TRUE,
#'     GO.tree=list(
#'         tree=list(
#'             distance="Rel",
#'             aggreg.method="ward.D2"
#'         ),
#'         cut=list(
#'             dynamic=list(
#'                 deepSplit=2,
#'                 minClusterSize =2
#'             )
#'         )
#'     ),
#'     samples.tree=NULL
#' )
#'
#' # Resnik distance GO terms heatmap
#' Jiang_clusters_wardD2<-ViSEAGO::GOterms_heatmap(
#'     myGOs,
#'     showIC=TRUE,
#'     showGOlabels=TRUE,
#'     GO.tree=list(
#'         tree=list(
#'             distance="Jiang",
#'             aggreg.method="ward.D2"
#'         ),
#'         cut=list(
#'             dynamic=list(
#'                 deepSplit=2,
#'                 minClusterSize =2
#'             )
#'         )
#'     ),
#'     samples.tree=NULL
#' )
#'
#' # Resnik distance GO terms heatmap
#' Wang_clusters_wardD2<-ViSEAGO::GOterms_heatmap(
#'     myGOs,
#'     showIC=TRUE,
#'     showGOlabels=TRUE,
#'     GO.tree=list(
#'         tree=list(
#'             distance="Wang",
#'             aggreg.method="ward.D2"
#'         ),
#'         cut=list(
#'             dynamic=list(
#'                 deepSplit=2,
#'                 minClusterSize =2
#'             )
#'         )
#'     ),
#'     samples.tree=NULL
#' )
#' }
#'
#' # clusters to compare
#' clusters<-list(
#'     Resnik="Resnik_clusters_wardD2",
#'     Lin="Lin_clusters_wardD2",
#'     Rel="Rel_clusters_wardD2",
#'     Jiang="Jiang_clusters_wardD2",
#'     Wang="Wang_clusters_wardD2"
#' )
#'
#' \dontrun{
#' # clusters content comparisons
#' clusters_comp<-ViSEAGO::compare_clusters(clusters)
#' }
#' @name compare_clusters
#' @rdname compare_clusters-methods
#' @exportMethod compare_clusters
setGeneric(
    name="compare_clusters",
    def=function(clusters)
        {standardGeneric("compare_clusters")
    }
)

#' @rdname compare_clusters-methods
#' @aliases compare_clusters
setMethod(
    "compare_clusters",
    signature ="list",
    definition=function(clusters) {

        ## convert GO.ID GO.clusters column in list

        # extract clusters and GO.ID
        Clusters<-unlist(
            recursive=FALSE,
            lapply(clusters,function(x){

                # load the target object
                object=get(x[[1]])

                # check class
                if(!is(object,"GO_clusters")){
                    stop("object must come from ViSEAGO::GOterms_heatmap()")
                }

                # extract GO.ID and correspond cluster assignation to a list
                object=unstack(
                    slot(slot(object,"enrich_GOs"),"data")[,c("GO.ID","GO.cluster"),with=FALSE]
                )

                # names of list elements
                list.names<-as.numeric(names(object))

                # add 0 if needed
                list.names<-sprintf(
                    paste(
                        "%0",
                        max(nchar(list.names)),
                        "d",
                        sep=""
                    ),
                    list.names
                )

                # rename list elements
                names(object)<-paste(names(x),list.names,sep="cl")

                # return the object
                object
            })
        )

        ## count common terms

        # return a matrix cross-product from all common GO.ID count by Cluster-objects
        clusters.length<-lengths(Clusters)

        # return a matrix cross-product from all common GO.ID count by Cluster-objects
        Data<-crossprod(
            table(
                stack(Clusters)
            )
        )

        # assign 0 to upper part of the matrix
        Data[upper.tri(Data)]<-0

        # reverse matrix row
        Data<-Data[nrow(Data):1,]

        ## rectangle shape definition

        # methods
        methods=unique(
            gsub("\\..+$","",colnames(Data))
        )

        # rectangle shape definition
        coord<-unlist(
            recursive=FALSE,
            lapply(seq_along(methods),function(x){

                # x coordinates
                col=grep(paste(methods[x],"\\.",sep=""),colnames(Data))

                # y coordinates
                lapply(x:length(methods),function(y){

                    # y coordinates
                    row=grep(paste(methods[y],".",sep=""),row.names(Data))

                    # y coordinates
                    val=c(col[1]-1.5,tail(col,n=1)-0.5,row[1]-1.5,tail(row,n=1)-0.5)

                    # val names
                    names(val)<-c("x0","x1","y0","y1")

                    # return val
                    val
                })
            })
        )

        # initiate a rect shape object
        rect <- list(
            type = "rect",
            fillcolor = "",
            line = list(
                color = "black",
                width=1,
                dash="dash"
            ),
            opacity = 0.3,
            xref = "x",
            yref = "y"
        )

        # all rectangles
        rects <-lapply(coord,function(x){

            # rect dimensions
            rect[["x0"]] <- x["x0"]
            rect[["x1"]] <- x["x1"]
            rect[["y0"]] <- x["y0"]
            rect[["y1"]] <- x["y1"]

            # rect rect
            rect
        })

        ## build plot matrix

        # count by line
        rowsum=apply(Data,1,sum)

        # count by column
        colsum=apply(Data,2,sum)

        # keep Data without all 0
        count<-matrix(
            paste('<br>GO.common: ',Data),
            ncol=ncol(Data),
            byrow=FALSE
        )

        # row %
        row_prc<-round(
            (Data/clusters.length[row.names(Data)])*100,
            digits=1
        )

        # col % hover text
        row_prc_hover.text=matrix(
            paste('<br>y%:',row_prc),
            ncol=ncol(Data)
        )

        # replace 0 by NA
        row_prc[row_prc==0]<-NA

        # col %
        col_prc<-t(
            round(
                (t(Data)/clusters.length[colnames(Data)])*100,
                digits=1
            )
        )

        # col % hover text
        col_prc_hover.text=matrix(
            paste('<br>x%:',col_prc),
            ncol=ncol(Data)
        )

        # replace 0 by NA
        col_prc[col_prc==0]<-NA

        # column GOcount
        x=matrix(
            paste(
                'x:',
                rep(
                    colnames(col_prc),
                    each=ncol(col_prc)
                ),
                '<br>x GO sum:',
                rep(
                    clusters.length[colnames(col_prc)],
                    each=ncol(col_prc)
                )
            ),
            ncol=ncol(col_prc)
        )

        # row GOcount
        y=matrix(
            paste(
                '<br>y:',
                rep(
                    row.names(col_prc),
                    each=ncol(col_prc)
                ),
                '<br>y GO sum:',
                rep(
                    clusters.length[row.names(col_prc)],
                    each=ncol(col_prc)
                )
            ),
            ncol=ncol(col_prc),
            byrow=TRUE
        )

        # create hover text matrix
        hover.text<-matrix(
            paste(
                x,
                y,
                count,
                col_prc_hover.text,
                row_prc_hover.text
            ),
            ncol=ncol(count),
            byrow=FALSE
        )

        # remove hover text with no GO in common
        hover.text[count=="<br>GO.common:  0"]<-""

        ## interactive heatmap

        # initialyse p
        p<-plot_ly()

        # Draw heatmap with y%
        p<-add_heatmap(
            p,
            x=colnames(col_prc),
            y=row.names(col_prc),
            z=col_prc,
            colorscale = "Greys",
            name="x identity%",
            hoverinfo='text',
            text=hover.text,
            visible=TRUE
        )

        # Draw heatmap with y%
        p<-add_heatmap(
            p,
            x=colnames(row_prc),
            y=row.names(row_prc),
            z=row_prc,
            colorscale = "Greys",
            name="y identity%",
            hoverinfo='text',
            text=hover.text,
            visible=FALSE
        )

        # add custom layout with dropdown menu
        layout(

            # graph object
            p,

            # add title
            title="clusters comparisons",

            # add axis legends
            xaxis=list(
                title="x",
                tickfont=list(size=10)
            ),
            yaxis=list(
                title="y",
                tickfont=list(size=10)
            ),

            # custom margin
            margin=list(
                t=30,
                b=100,
                l=100,
                r=0)
            ,

            # add scrolling menu for availables measures
            shapes = rects,

            # add scrolling menu for availables measures
            updatemenus = list(

                # measure dropdown menu
                list(

                    # button position
                    x= 0.1,
                    y= 1.1,

                    buttons = list(

                        # x output button
                        list(
                            method ="restyle",
                            args = list("visible",list(TRUE,FALSE)),
                            label="x identity%"
                        ),

                        # y output button
                        list(
                            method ="restyle",
                            args = list("visible",list(FALSE,TRUE)),
                            label="y indentity%"
                        )
                    )
                )
            )
        )
    }
)

#' @title Enriched GO terms intersections plot.
#' @description This method allows to visualize GO terms intersections between results of enrichment tests.
#' @import data.table
#' @importFrom UpSetR upset
#' @family enrich_GO_terms
#' @family visualization
#' @param object an \code{\link{enrich_GO_terms-class}} or \code{\link{GO_clusters-class}} objects.
#' @param file output file name (default to "./upset.xls")
#' @details
#' This function displays and print the intersections of enriched GO terms (p<0.01) between all results provided by \code{\link{enrich_GO_terms-class}}
#' or \code{\link{GO_clusters-class}} objects. The intersections are shown in an upset plot and printed in a table.
#' @return print table and upset.
#' @include enrich_GO_terms.R GO_clusters.R
#' @examples
#' # load example object
#' data(
#'     myGOs,
#'     package="ViSEAGO"
#' )
#'
#' # print upset
#' ViSEAGO::Upset(myGOs)
#' @name Upset
#' @rdname Upset-methods
#' @exportMethod Upset
setGeneric(name="Upset",def=function(object,file="./upset.xls") {standardGeneric("Upset")})

#' @rdname Upset-methods
#' @aliases Upset
setMethod(
    "Upset",
    signature="ANY",
    definition=function(object,file){

        # check the class
        if(!is(object,"enrich_GO_terms") & !is(object,"GO_SS") & !is(object,"GO_clusters")){
            stop("object must be enrich_GO_terms, GO_SS, or GO_clusters class objects")
        }

        # extract data.table from enrich_GO_terms or GO_clusters class object.
        if(is(object,"enrich_GO_terms")){

            # extract data.table from enrich_GO_terms class object
            Data<-slot(object,"data")

        }else{

            # extract data.table from GO_clusters class object
            Data<-slot(slot(object,"enrich_GOs"),"data")
        }

        # keep only GOterms and pvalues by condition
        Data<-Data[,grep("GO\\.ID|\\.pvalue",names(Data)),with=FALSE]

        # remove pvalues in columns names
        names(Data)<-gsub("\\.pvalue","",names(Data))

        # build binary matrix for Upset graph
        Data<-data.table(
            GO.ID=Data[,"GO.ID",with=FALSE],
            Data[,lapply(.SD,function(x){val=x<0.01;x[val]<-1;x[!val]<-0;x}),.SDcols=2:ncol(Data)]
        )

        # draw upset at screen
        print(
            upset(
                Data,
                sets=rev(names(Data)[-1]),
                keep.order = TRUE,
                text.scale=2
            )
        )

        # print image to file
        png(sub("\\.[^\\.]+$",".png",file))
            print(
                upset(
                    Data,
                    sets=rev(names(Data)[-1]),
                    keep.order = TRUE,
                    text.scale=2
                )
            )
        dev.off()
        
        # build list vectors of significant GO.ID by conditions
        setlist<-lapply(2:ncol(Data),function(x){

            # extract significant GO terms
            Data$GO.ID[Data[,x,with=FALSE]==1]
        })
        names(setlist)<-names(Data)[-1]

        # use ViSEAGO internal intersetions function
        OLlist=overLapper(setlist)

        # return or print
        if(is.null(file)){

            # return
            OLlist

        }else{

            # build matrix
            OLexport <-as.matrix(
                unlist(vapply(OLlist, paste, collapse=";",""))
            )

            # convert in data.table
                OLexport<-data.table(
                combs=row.names(OLexport),
                length=lengths(OLlist),
                GOterms=OLexport[,1]
            )

            # write the file
            fwrite(
                OLexport,
                file=file,
                quote=FALSE,
                sep="\t"
            )
        }
    }
)

#' @title perform multilevel preranked gene set enrichment analysis.
#' @description This method perform fast gene set enrichment analysis (GSEA) using \pkg{fgsea} package.
#' @import data.table
#' @import topGO
#' @import fgsea
#' @family GO_terms
#' @param geneSel a 2 columns \code{data.table} with \bold{preranked} gene identifiants (in first column) based on the statistical values (second column).
#' @param gene2GO a \code{\link{gene2GO-class}} object created by \code{\link{annotate}} method.
#' @param ont the ontology used is "MF" (Molecuar Function), "BP" (Biological Process), or "CC" (Cellular Component).
#' @param method fgsea method to use with \code{\link[fgsea]{fgseaSimple}} or \code{\link[fgsea]{fgseaMultilevel}}.
#' @param params a \code{list} with \code{\link[fgsea]{fgseaSimple}} or \code{\link[fgsea]{fgseaMultilevel}} parameters.
#' @details This method is a convenient wrapper using a given ontology category (\code{ont} argument)
#' in order to perform gene set enrichment analysis using \code{\link[fgsea]{fgseaSimple}} or
#' \code{\link[fgsea]{fgseaMultilevel}} algorithm from  \pkg{fgsea} package.
#' 
#' The complete GO annotation is required (\code{gene2GO} argument), and also a 2 columns \code{data.table} with \bold{preranked}
#' gene identifiants (in first column) based on statistical values (second column).
#' 
#' Defaults fgseaSimple parameters were used for perform test with \code{nperm} set to 10,000.\cr
#' Defaults fgseaMultilevel parameters were used for perform test 
#' except the \code{eps} arg that was set to 0 for better pvalues estimation.\cr
#' A gene frequency (\%) of leadingEdge/size is added to output \code{data.table}.
#' @return a \code{\link{fgsea-class}} object.
#' @references
#' Korotkevich G, Sukhov V, Sergushichev A (2019). "Fast gene set enrichment analysis." bioRxiv. doi: 10.1101/060012, http://biorxiv.org/content/early/2016/06/20/060012.
#' @include fgsea.R
#' @examples
#' # gene list
#' PregnantvsLactate<-data.table::fread(
#'     system.file(
#'         "extdata/data/input",
#'         "pregnantvslactate.complete.txt",
#'         package = "ViSEAGO"
#'     ),
#'     select = c("Id","padj")
#' )
#'
#' # rank Id based on statistical value (padj here)
#' PregnantvsLactate<-data.table::setorder(PregnantvsLactate,padj)
#'
#' \dontrun{
#' # connect to Bioconductor
#' Bioconductor<-ViSEAGO::Bioconductor2GO()
#'
# 'load GO annotations from Bioconductor
#' myGENE2GO<-ViSEAGO::annotate(
#'    "org.Mm.eg.db",
#'    Bioconductor
#' )
#' 
#' # run fgseaMultilevel
#' pregnantvslactate<-ViSEAGO::runfgsea(
#'     geneSel=PregnantvsLactate,
#'     gene2GO=myGENE2GO,
#'     ont="BP",
#'     method="fgseaMultilevel",
#'     params=list(
#'         minSize=5,
#'         scoreType="pos"
#'     )
#' )
#' }
#' @name runfgsea
#' @rdname runfgsea-methods
#' @exportMethod runfgsea
setGeneric(
    name="runfgsea",
    def=function(geneSel,gene2GO,ont,method=c("fgseaSimple","fgseaMultilevel"),params=list(nperm=10000,sampleSize=101,minSize=1,maxSize=Inf,eps = 0,scoreType=c("std", "pos", "neg"),nproc = 0,gseaParam = 1,BPPARAM = NULL,absEps = NULL)){
        standardGeneric("runfgsea")
    }
)

#' @rdname runfgsea-methods
#' @aliases runfgsea
setMethod(
    "runfgsea",
    signature(
        gene2GO="gene2GO",
        ont="character"
    ),
    definition=function(geneSel,gene2GO,ont,method,params){

        # fgseaSimple  default parameters
        if(method=="fgseaSimple"){
            defaults_params=list(
                nperm=10000,
                minSize=1,
                maxSize=Inf,
                scoreType=c("std", "pos", "neg"),
                nproc = 0,
                gseaParam = 1,
                BPPARAM = NULL
            )
        }

        # fgseaMultilevel default parameters
        if(method=="fgseaMultilevel"){
            defaults_params=list(
                sampleSize=101,
                minSize=1,
                maxSize=Inf,
                eps = 0,
                scoreType=c("std", "pos", "neg"),
                nproc = 0,
                gseaParam = 1,
                BPPARAM = NULL,
                absEps = NULL
            )
        }

        # params args
        params_args<-names(params)[names(params)%in%names(defaults_params)]
        
        # modify default parameters
        defaults_params[params_args]<-params[params_args]

        # rename
        params<-defaults_params

        # reverse list
        Pathways<-inverseList(slot(gene2GO,ont))

        # assign header
        names(geneSel)<-c("Id","value")

        # build ranks
        Ranks<-geneSel$value

        # assign Ids to rank
        names(Ranks)<-geneSel$Id

        # perform simple fgsea
        if(method=="fgseaSimple"){
            data<-fgseaSimple(
                Pathways,
                stats=Ranks,
                nperm=params$nperm,
                minSize=params$minSize,
                maxSize=params$maxSize,
                scoreType=params$scoreType,
                nproc=params$nproc,
                gseaParam=params$gseaParam,
                BPPARAM=params$BPPARAM
            )
        }

        # perform multilevel fgsea
        if(method=="fgseaMultilevel"){
            data<-fgseaMultilevel(
                Pathways,
                Ranks,
                sampleSize=params$sampleSize,
                minSize=params$minSize,
                maxSize=params$maxSize,
                eps=params$eps,
                scoreType=params$scoreType,
                nproc=params$nproc,
                gseaParam=params$gseaParam,
                BPPARAM=params$BPPARAM,
                absEps=params$absEps
            )
        }

        # add leadingEdge gene frequency
        data<-data[,
            `genes_frequency`:=paste(
                round(
                    length(unlist(leadingEdge))/size*100,
                    digits=3
                ),
                "% (",
                length(unlist(leadingEdge)),
                "/",
                size,
                ")",
                sep=""
            )
            ,by=pathway
        ]

        # create fgsea output
        new(
            "fgsea",
            description=paste(
                slot(gene2GO,"db"),
                slot(gene2GO,"organism"),
                slot(gene2GO,"stamp")
            ),
            ontology =ont,
            method=method,
            params=params,
            input=list(geneSel),
            data=list(data)
        )
})

#' @title Build a clustering heatmap on GO terms.
#' @description This method computes a clustering heatmap based on GO terms semantic similarity.
#' @import data.table
#' @importFrom ggplot2 scale_fill_gradient2
#' @importFrom plotly layout
#' @importFrom heatmaply heatmaply
#' @importFrom ComplexHeatmap Heatmap rowAnnotation
#' @importFrom circlize colorRamp2
#' @importFrom dendextend branches_attr_by_clusters get_nodes_attr set
#' @importFrom dynamicTreeCut cutreeDynamic
#' @importFrom RColorBrewer brewer.pal
#' @importFrom scales rescale
#' @importFrom methods slot slot<-
#' @importFrom stats as.dist cor dist cutree na.omit
#' @importFrom grDevices colorRampPalette
#' @family GO_terms
#' @family semantic_similiarity
#' @family visualization
#' @param myGOs a \code{\link{GO_SS-class}} object from \code{\link{compute_SS_distances}}.
#' @param showIC \code{logical} (default to TRUE) to display the GO terms Information Content (IC) side bar.
#' @param showGOlabels \code{logical} (default to TRUE) to display the GO terms ticks on y axis.
#' @param heatmap_colors pvalues color range with white to Sangria collors by default (c("#ffffff","#99000D")).
#' @param GO.tree a named \code{list} of parameters to build and cut the GO terms \code{dendrogram}.
#'  \describe{
#'      \item{tree (a named \code{list} with:)}{
#'          \describe{
#'              \item{distance ("Wang" by default)}{distance computed from the semantic similarity which could be
#'               IC-based ("Resnik", "Rel", "Lin", or "Jiang") or graph-based ("Wang").
#'              }
#'               \item{aggreg.method ("ward.D2" by default)}{aggregation method criteria from \code{\link[stats]{hclust}} ("ward.D",
#'               "ward.D2", "single", "complete", "average", "mcquitty", "median", or "centroid") to build a \code{dendrogram}.}
#'              \item{rotate}{sort the branches of the tree based on a vector - eithor of labels order or the labels in their new order}
#'          }
#'      }
#'      \item{cut (a named \code{list} with:)}{
#'          \describe{
#'              \item{static (default to NULL)}{a \code{numeric} value that is the height (between 0 and 1),
#'              or the number of clusters (value > 1) to cut the \code{dendrogram}.
#'              }
#'               \item{dynamic (a named \code{list} which only contains \code{\link[dynamicTreeCut]{cutreeDynamic}}
#'               options values below)
#'               }{
#'                 \describe{
#'                     \item{pamStage (default to TRUE)}{second (PAM-like) stage will be performed.}
#'                     \item{pamRespectsDendro (default to TRUE)}{PAM stage will respect the dendrogram in the sense that objects
#'                      and small clusters will only be assigned to clusters that belong to the same branch that the objects or small
#'                      clusters being assigned belong to.}
#'                     \item{deepSplit (default to 2)}{provides a rough control over sensitivity for cluster splitting (range 0 to 4).
#'                       The higher the value (or if TRUE), the more and smaller clusters will be produced.}
#'                     \item{minClusterSize (default to 2)}{minimum cluster size.}
#'                 }
#'              }
#'          }
#'      }
#' }
#' @param samples.tree a named \code{list} of parameters to build and cut the samples \code{dendrogram} (default to NULL).
#'  \describe{
#'      \item{tree (a named \code{list} with:)}{
#'          \describe{
#'              \item{distance ("pearson" by default)}{distance computed that could be correlation ("abs.pearson","pearson", "kendall", or "spearman"),
#'              or \code{dist} method (euclidean", "maximum", "manhattan", "canberra", "binary", or "minkowski).}
#'               \item{aggreg.method ("average" by default)}{same options than for \code{GO.tree} argument}
#'          }
#'      }
#'      \item{cut}{same options than for \code{GO.tree} argument.}
#' }
#' @details This method computes a clustering heatmap based on GO terms semantic similarity (computed with \code{\link{compute_SS_distances}}).\cr
#' The dendrogram produced could be cutted in static or dynamic mode.\cr
#'  \enumerate{
#'   \item build dendrograms on  GO terms and optionally on samples.
#'   \item cut in static or dynamic mode and color the dendrogram branchs.
#'   \item build an interactive clustering heatmap based on \code{\link[heatmaply]{heatmaply}}.
#'  }
#' @return a \code{\link{GO_clusters-class}} object.
#' @references
#' Matt Dowle and Arun Srinivasan (2017). data.table: Extension of `data.frame`. R package version 1.10.4. https://CRAN.R-project.org/package=data.table.
#'
#' Tal Galili (2015). dendextend: an R package for visualizing, adjusting, and comparing trees of hierarchical clustering.
#' Bioinformatics. DOI:10.1093/bioinformatics/btv428.
#'
#' Tal Galili (2017). heatmaply: Interactive Cluster Heat Maps Using 'plotly'.
#' R package version 0.9.1. https://CRAN.R-project.org/package=heatmaply.
#'
#' Peter Langfelder, Bin Zhang and with contributions from Steve Horvath (2016). dynamicTreeCut: Methods for Detection of Clusters
#' in Hierarchical Clustering Dendrograms. R package version 1.63-1. https://CRAN.R-project.org/package=dynamicTreeCut.
#'
#' Erich Neuwirth (2014). RColorBrewer: ColorBrewer Palettes. R package version 1.1-2. https://CRAN.R-project.org/package=RColorBrewer.
#'
#' Carson Sievert, Chris Parmer, Toby Hocking, Scott Chamberlain, Karthik Ram, Marianne Corvellec and Pedro Despouy (2017).
#' plotly: Create Interactive Web Graphics via 'plotly.js'. R package version 4.6.0. https://CRAN.R-project.org/package=plotly.
#'
#' Hadley Wickham (2016). scales: Scale Functions for Visualization. R package version 0.4.1. https://CRAN.R-project.org/package=scales.
#'
#' H. Wickham. ggplot2: Elegant Graphics for Data Analysis. Springer-Verlag New York, 2009.
#' @include GO_clusters.R
#' @examples
#' # load data example
#' utils::data(
#'     myGOs,
#'     package="ViSEAGO"
#' )
#' \dontrun{
#' # compute GO terms Semantic Similarity distances
#' myGOs<-ViSEAGO::compute_SS_distances(
#'    myGOs,
#'    distance="Wang"
#' )
#'
#' # GOtermsHeatmap with default parameters
#' Wang_clusters_wardD2<-ViSEAGO::GOterms_heatmap(
#'     myGOs,
#'     showIC=TRUE,
#'     showGOlabels=TRUE,
#'     GO.tree=list(
#'         tree=list(
#'             distance="Wang",
#'             aggreg.method="ward.D2",
#'             rotate=NULL
#'         ),
#'         cut=list(
#'             dynamic=list(
#'                 pamStage=TRUE,
#'                 pamRespectsDendro=TRUE,
#'                 deepSplit=2,
#'                 minClusterSize =2
#'             )
#'         )
#'     ),
#'     samples.tree=NULL
#' )
#' }
#' @name GOterms_heatmap
#' @rdname GOterms_heatmap-methods
#' @exportMethod GOterms_heatmap
setGeneric(
    name="GOterms_heatmap",
    def=function(
        myGOs,
        showIC=TRUE,
        showGOlabels=TRUE,
        heatmap_colors=c("#ffffff","#99000D"),
        GO.tree=list(
            tree=list(
                distance="Wang",
                aggreg.method="ward.D2",
                rotate=NULL
            ),
            cut=list(
                dynamic=list(
                    pamStage=TRUE,
                    pamRespectsDendro=TRUE,
                    deepSplit=2,
                    minClusterSize =2
                )
            )
        ),
        samples.tree=NULL
    ){
        standardGeneric("GOterms_heatmap")
    }
)

#' @rdname GOterms_heatmap-methods
#' @aliases GOterms_heatmap
setMethod(
    "GOterms_heatmap",
    signature(
        myGOs="GO_SS"
    ),
    definition=function(myGOs,showIC,showGOlabels,heatmap_colors,GO.tree,samples.tree){

        ## check entry
        if(length(slot(myGOs,"terms_dist"))==0){
            stop("Please compute Semantic Similarity distance with ViSEAGO::compute_SS_distances()")
        }

        ## creates trees

        # add default params if not available in the tree list
        Tree.params<-function(...){

            # keep the name
            name<-as.character(...)

            # Tree
            Tree<-list(get(name))[[1]]

            if(!is.null(Tree)){

                # extract list names
                tree<-names(Tree$tree)

                # default distance if not available
                if("distance"%in%tree){
                    distance<-Tree$tree$distance
                }else{
                    if(name=="GO.tree"){
                        distance="Wang"
                    }else{
                        distance="pearson"
                    }
                }

                # default distance if not available
                if("aggreg.method"%in%tree){
                    aggreg.method<-Tree$tree$aggreg.method

                }else{
                    if(name=="GO.tree"){
                        aggreg.method="ward.D2"
                    }else{
                        aggreg.method="average"
                    }
                }

                # default order
                if("rotate"%in%tree){
                    rotate<-Tree$tree$rotate
                }else{
                    rotate=NULL
                }

                # extract list names
                if(!is.null(Tree$cut)){

                    # default static cut tree method if not available
                    tree=names(Tree$cut)

                    # default static cut tree method if not available
                    if("static"%in%tree){
                        static<-Tree$cut$static
                    }else{
                        static=NULL
                    }

                    if(is.null(static)){

                        # extract list names
                        tree<-names(Tree$cut$dynamic)

                        # default aggreg.method if not available
                        if("pamStage"%in%tree){
                            pamStage<-Tree$cut$dynamic$pamStage
                        }else{
                            pamStage=TRUE
                        }

                    # default aggreg.method if not available
                    if("pamRespectsDendro"%in%tree){
                        pamRespectsDendro<-Tree$cut$dynamic$pamRespectsDendro
                    }else{
                        pamRespectsDendro=TRUE
                    }

                        # default aggreg.method if not available
                        if("deepSplit"%in%tree){
                            deepSplit<-Tree$cut$dynamic$deepSplit
                        }else{
                            deepSplit=2
                        }

                        # default aggreg.method if not available
                        if("minClusterSize"%in%tree){
                            minClusterSize<-Tree$cut$dynamic$minClusterSize
                        }else{
                             minClusterSize =2
                        }

                        # build dynamic
                        dynamic=list(
                            pamStage=pamStage,
                                pamRespectsDendro=pamRespectsDendro,
                            deepSplit=deepSplit,
                            minClusterSize =minClusterSize
                        )
                    }else{

                        # build dynamic
                        dynamic=NULL
                    }

                    # build cut list
                    cut=list(
                        static=static,
                        dynamic=dynamic
                    )
                }else{

                    # build empty cut list
                    cut=NULL
                }

                # Tree with default params if nessecary
                list(
                    tree=list(
                        distance=distance,
                        aggreg.method=aggreg.method,
                        rotate=rotate
                    ),
                    cut=cut
                )
            }else{
                # return empty
                NULL
            }
        }

        # row.tree with default if NA
        row.tree<-Tree.params("GO.tree")

        # col.tree with default if NA
        col.tree<-Tree.params("samples.tree")

        # check dendrogram entry
        if(is.null(row.tree$tree$distance)){
            stop(
                paste(
                    "please enter a myGOs object computed SS distance name (",
                    paste(
                        names(
                            slot(myGOs,"terms_dist")
                        ),
                        collapse=", "
                    ),
                    ") in the GO.tree distance argument",
                    sep=""
                )
            )
        }

        ## load Data

        # data input
        sResults<-slot(slot(myGOs,"enrich_GOs"),"data")

        # extract pvalues columns
        mat<-as.matrix(
            sResults[,grep("-log10.pvalue",names(sResults)),with=FALSE]
        )

        # remove -log10.pvalue in colnames
        colnames(mat)<-gsub("\\.-log10.pvalue","",colnames(mat))

        # duplicate sResults in Data
        Data<-sResults

        # add modfied terms definition
        row.names(mat)<-paste("<br>GO.ID:",Data$`GO.ID`,"<br>GO.name:",Data$term)

        # if Inf in -log10pvalue, set 9
        for(i in  seq_len(ncol(mat))){mat[mat[,i]=="Inf",i]<-9}

        # transpose for GO in rows and samples in columns  in the levelplot
        x<-t(mat)

        # extract genes frequency columns
        genes_frequency<-as.matrix(
            sResults[,grep("genes_frequency",names(sResults)),with=FALSE]
        )

        # add row_names
        row.names(genes_frequency)<-sResults[,`GO.ID`]

        # IC
        IC<-round(
            slot(myGOs,"IC")[sResults$`GO.ID`],
            digits=2
        )

        ## distance correlation

        # distance function
        dist.fun<-function(...){

            # text tree
            Tree<-as.character(...)

            # row or col tree
            if(length(grep("col",Tree))==1){col=TRUE}else{col=FALSE}

            # get tree
            Tree<-get(Tree)

            # if tree on column
            if(!is.null(Tree)){

                # column tree with correlation distance
                if(is.character(Tree$tree$distance) || Tree$tree$distance%in%c("abs.pearson","pearson", "kendall","spearman")){

                 # if tree on column
                    if(col==TRUE){x<-t(x)}

                    # for absolute pearson correlation
                    if(Tree$tree$distance=="abs.pearson"){

                        # col.tree with absolute pearson correlation
                        as.dist(1-abs(cor(x,method="pearson")))

                    }else{

                        # other correlation distances
                        as.dist(1-cor(x,method=Tree$tree$distance))
                    }

                }else{

                    # if row on column
                    if(col==FALSE){x<-t(x)}

                    # calculate distance
                    dist(x,method=Tree$tree$distance)
                }
            }
        }

        # columns distance
        col.dist<-dist.fun("col.tree")

        # row distance (precalculated distance based on similarity of GO terms)
        row.dist<-slot(myGOs,"terms_dist")[[row.tree$tree$distance]]

        ## aggregation

        # aggregation function
        aggreg.fun<-function(...){

            # text tree
            Tree<-as.character(...)

            # add calculated distance
            dist<-get(
                paste(
                    substring(Tree,1,3),
                    "dist",
                    sep="."
                )
            )

            # if distance matrix not empty
            if(!is.null(dist)){

                # get tree
                Tree<-get(Tree)

                # perform hclust
                hc<-hclust(dist, method =Tree$tree$aggreg.method)

                # rotate
                if(!is.null(Tree$tree$rotate)){
                    hc<-dendextend::rotate(hc,Tree$tree$rotate)
                }

                # return roated hc
                return(hc)
            }
        }

        # columns distance
        col_tree<-aggreg.fun("col.tree")

        # row distance
        row_tree<-aggreg.fun("row.tree")

        ## create dendrogram(s)

        # visible binding for global variable
        # Bioconductor submission "checking R code possible problems" step
        col.ord<-NULL
        row.ord<-NULL

        # create row and column dendrograms
        for(i in c("row_tree","col_tree")){

            # create dendrogram
            if(!is.null(get(i))){

                # dendrogram name
                dd<-paste("dd",substring(i,1,3),sep=".")

                # dendrogram convert
                dend<-as.dendrogram(get(i))

                # create dendrogram
                assign(dd,dend)

                # create ordering vector
                assign(
                    paste(
                        substring(i,1,3),
                        "ord",
                        sep="."
                    ),
                    order.dendrogram(
                        get(dd)
                    )
                )

            }else{

                # ordering vector ni null according column and row
                if(i=="col_tree"){Dat<-seq_len(nrow(x))}else{Dat<-seq_len(ncol(x))}

                # create ordering vector
                assign(
                    paste(
                        substring(i,1,3),
                        "ord",
                        sep="."
                    ),
                    Dat
                )
            }
        }

        ## cut trees

        # cut tree
        cut.tree<-function(...){

            # text tree
            Tree<-as.character(...)

            # text tree
            if(!is.null(get(Tree))){

                # add calculated distance
                dendro=get(sub("\\.","_",Tree))

                # add calculated distance
                dist<-get(
                    paste(
                        substring(Tree,1,3),
                        "dist",
                        sep="."
                    )
                )

                # add order
                ord<-get(
                    paste(
                        substring(Tree,1,3),
                        "ord",
                        sep="."
                    )
                )

                # get tree
                Tree<-get(Tree)

                # cut or not
                if(!is.null(Tree$cut)){

                    # cut dynamic
                    if(is.null(Tree$cut$static)){

                        # cut dynamic
                        gp<-cutreeDynamic(
                            dendro =dendro,
                            method="hybrid",
                            verbose=FALSE,
                            distM =as.matrix(dist),
                            pamStage=Tree$cut$dynamic$pamStage,
                            pamRespectsDendro=Tree$cut$dynamic$pamRespectsDendro,
                            deepSplit=Tree$cut$dynamic$deepSplit,
                            minClusterSize =Tree$cut$dynamic$minClusterSize
                        )

                        # groups
                        clust=unique(gp[ord])

                        # new clusters names table
                        clust=data.table(
                            ini=clust,
                            new=seq_along(clust)
                        )

                        # convert gp to data.table
                        gp<-data.table(ini=gp)

                        # merge gp with new clusters names
                        gp<-merge(
                            gp,
                            clust,
                            by="ini",
                            all.x=TRUE,
                            sort=FALSE
                        )

                        # extract only new clusters names in the initial order
                        gp<-gp$new

                        # add go names
                        names(gp)<-attr(dist,"Labels")

                        # return the object
                        gp

                    }else{

                        # static cut tree by group numbers
                        if(Tree$cut$static>1){

                            # cut row tree
                            cutree(dendro, k =Tree$cut$static,h = NULL)[ord]

                        }else{

                            # cut row tree
                            cutree(dendro,k=NULL,h=Tree$cut$static)[ord]
                        }
                    }
                }else{

                    # text tree
                    Tree<-as.character(...)

                    # unique group
                    gp<-rep(1,length(get(sub("tree","ord",Tree))))

                    # give col names attributes
                    if(Tree=="col.tree"){attr(gp,"names")<-row.names(x)}

                    # give row names attributes
                    if(Tree=="row.tree"){attr(gp,"names")<-colnames(x)}

                    # return gp
                    gp
                }

            }else{

                # unique group
                gp<-rep(1,length(get(sub("tree","ord",Tree))))

                # give col names attributes
                if(Tree=="col.tree"){attr(gp,"names")<-row.names(x)}

                # give row names attributes
                if(Tree=="row.tree"){attr(gp,"names")<-colnames(x)}

                # return gp
                gp
            }
        }

        # cut row tree
        row.gp<-cut.tree("row.tree")

        # cut column tree
        col.gp<-cut.tree("col.tree")

        # add cluster to row.names
        row.names(mat)<-paste("<br>cluster:",row.gp,row.names(mat))

        ## color row dendrogram according clusters

        # color labels of cutting tree
        palette=c(
            "darksalmon","cyan","orchid","midnightblue","hotpink",
            "sienna","aquamarine","royalblue","forestgreen","salmon","maroon",
            "green","deepskyblue","olivedrab","springgreen","limegreen","magenta",
            "peru","tan","steelblue","orange","burlywood","skyblue","red",
            "darkslateblue","plum","goldenrod","cadetblue","indianred","pink",
            "slateblue","turquoise","darkorchid","cornflowerblue","darkolivegreen",
            "orangered","powderblue","rosybrown","darkgoldenrod","seagreen",
            "darkturquoise","thistle","dodgerblue","sandybrown","purple","tomato",
            "deeppink", "darkmagenta","darkred","darkkhaki","coral","lawngreen",
             "darkorange","black","chocolate","firebrick","darkseagreen","blue",
            "navy","darkgreen"
        )

        # color branches according clusters
        dd.row<-branches_attr_by_clusters(
            dd.row,
            row.gp[row.ord],
            values = palette
        )

        # color labels of cutting tree
        colors=na.omit(
            unique(
                unlist(
                    get_nodes_attr(dd.row,"edgePar")
                )
            )
        )

        # colors table
        colors=data.table(
            gp=unique(row.gp[row.ord]),
            color=colors
        )

        # merge cluster term assignation and corresponding color
        colors<-merge(
            data.table(gp=row.gp[row.ord]),
            colors,
            by="gp",
            all.x=TRUE,
            sort=FALSE
        )

        # assign text color
        dd.row<-set(
            dd.row,
            "labels_col",
            colors$color
        )

        # create dendrogram
        dd.row<-set(
            dd.row,
            "labels_cex",
            1
        )

        ## column dendrogram according clusters

        if(!is.null(col.dist)){

            # color branches according clusters
            dd.col<-branches_attr_by_clusters(
                dd.col,
                col.gp[col.ord],
                values =palette
            )

            # color labels of cutting tree
            colors=na.omit(
                unique(
                    unlist(
                        get_nodes_attr(dd.col,"edgePar")
                    )
                )
            )

            # colors table
            colors=data.table(
                gp=unique(col.gp[col.ord]),
                color=colors
            )

            # merge cluster term assignation and corresponding color
            colors<-merge(
                data.table(gp=col.gp[col.ord]),
                colors,
                by="gp",
                all.x=TRUE,
                sort=FALSE
            )

            # assign text color
            dd.col<-set(
                dd.col,
                "labels_col",
                colors$color
            )

            # create dendrogram
            dd.col<-dendextend::set(
                dd.col,
                "labels_cex",
                1
            )
        }

        ## draw heatmap

        # If gene background not the same
        if(slot(slot(myGOs,"enrich_GOs"),"same_genes_background")==FALSE){

            # show warning
            warning(
                call. =FALSE,
                "Not equal genes background in all conditions:\n--> pvalues converted to significant (1) or not (0) by condition in the cluster-heatmap"
            )

            # reduce to significant or not (p<0.01, -log10(p)>2)
            mat<-ifelse(mat < 2, 0, 1)

            # replace NA per 0
            mat[is.na(mat)]<-0
        }

        if(showIC==TRUE){

            # draw heatmapply
            hm<-heatmaply::heatmaply(

                # the initial matrix
                x=mat,

                # row labels
                labRow=row.names(mat),

                # columns labels
                labCol=colnames(mat),

                # the row dendrogram
                Rowv=dd.row,

                # the ordered matrix according dendrograms for columns
                Colv=if(!is.null(col.dist)){dd.col}else{FALSE},

                # the IC information,
                row_side_colors=data.table(IC=IC),

                # the IC information
                row_side_palette =colorRampPalette(c("#FFFFFF","#49006A")),

                # the color palette
                scale_fill_gradient_fun=if(slot(slot(myGOs,"enrich_GOs"),"same_genes_background")){

                    # if same gene background
                    scale_fill_gradient2(
                        name="-log10pvalue",
                        low= heatmap_colors[1],
                        mid= heatmap_colors[1],
                        high =heatmap_colors[2],
                        midpoint = 1.3
                    )
                }else{

                    # if not same gene background
                    scale_fill_gradient(
                        low=heatmap_colors[1],
                        high=heatmap_colors[2],
                        name="significance"
                    )
                },

                # the width of dendrogramm
                branches_lwd = 0.1,

                # color bar length
                colorbar_len=0.05
            )
        }else{

            # draw heatmapply
            hm<-heatmaply::heatmaply(

                # the initial matrix
                x=mat,

                # row labels
                labRow=row.names(mat),

                # columns labels
                labCol=colnames(mat),

                # the row dendrogram
                Rowv=dd.row,

                # the ordered matrix according dendrograms for columns
                Colv=if(!is.null(col.dist)){dd.col}else{FALSE},

                # the color palette
                scale_fill_gradient_fun=if(slot(slot(myGOs,"enrich_GOs"),"same_genes_background")){

                    # if same gene background
                    scale_fill_gradient2(
                        name="-log10pvalue",
                        low= heatmap_colors[1],
                        mid= heatmap_colors[1],
                        high =heatmap_colors[2],
                        midpoint = 1.3
                    )
                }else{

                    # if not same gene background
                    scale_fill_gradient(
                        low=heatmap_colors[1],
                        high=heatmap_colors[2],
                        name="significance"
                    )
                },
                
                # the width of dendrogramm
                branches_lwd = 0.1,
                
                # color bar length
                colorbar_len=0.05
            )
        }

        # with column dendrogram and showIC
        col=vapply(hm$x$data,function(x){length(x$text)},0)
        col<-which(col==(nrow(mat)*ncol(mat)))[1]

        # for each column
        text<-hm$x$data[[col]]$text

        # ordering gene_frequency table according text
        genes_frequency<-data.table(genes_frequency[rev(row.ord),col.ord])

        # for each column
        for (i in seq_len(ncol(text))){
            text[,i]<-gsub("(<br>|^)row: ","",text[,i])
            text[,i]<-gsub("<br>value:","<br>-log10 pvalue:",text[,i])
            text[,i]<-paste(text[,i],"<br>gene frequency: ",unlist(genes_frequency[,i,with=FALSE]),sep="")
        }

        # add text
        hm$x$data[[col]]$text<-text

        # modify row_side_color hover text
        if(showIC==TRUE){

            # if column dendrogram
            start=col+2

            # modify row_side_color hover text
            for(i in start:(start+nrow(mat)-1)){

                # IC color font correction for maxima value rounded to 0 digits
                if(round(as.numeric(hm$x$data[[i]]$name),digits=0)==round(max(IC),digits=0)){

                    # correct maxima value color purple "#49006A" in rgba
                    hm$x$data[[i]]$fillcolor<-"rgba(73,0,106,1)"
                }

                # store the text
                text<-hm$x$data[[i]]$text

                # extract the corresponding row names value
                text<-row.names(mat)[as.numeric(gsub("^.+row:[[:space:]]|<br />value.+$","",text))]

                # return values
                hm$x$data[[i]]$text<-paste(
                    "value:",
                    round(IC[gsub("^.+GO.ID: | <br>GO.name.+$","",text)],digits=2),
                    "<br>column: IC<br>row:",
                    text
                )
            }
        }

        # custom row text
        row.text=gsub(
            "^.+GO.name: ",
            "",
            rev(row.names(mat)[row.ord])
        )

        # cut very long definition
        row.text[nchar(row.text)>50]<-paste(substring(
        row.text[nchar(row.text)>50],1,50),"...",sep="")

        # modify layout
        hm<-layout(
            hm,

            # add title
            title=paste(
                row.tree$tree$distance,
                "GOterms distance clustering heatmap plot"
            ),

            # title size
            font=list(size=14),

            # set margin
            margin=list(l=300,r=0,b=150,t=100)
        )

        # modify layout
        if(!is.null(col.dist)){

            # modify layout
            hm<-layout(
                hm,

                # domain
                yaxis=list(domain=c(0.95,1)),

                # y axis
                yaxis2=list(
                    domain=c(0,0.95),
                    family="Times New Roman",
                    tickmode="array",
                    tickvals=seq_len(nrow(mat)),
                    ticktext=row.text,
                    tickfont=list(size=10),
                    showticklabels=showGOlabels
                )
            )
        }else{

            # modify layout
            hm<-layout(
                hm,

                 # y axis
                yaxis=list(
                    family="Times New Roman",
                    tickmode="array",
                    tickvals=if(showGOlabels==TRUE){seq_len(nrow(mat))}else{NULL},
                    ticktext=row.text,
                    tickfont=list(size=10),
                    showticklabels=showGOlabels
                )
            )
        }

        # modify layout
        if(showIC==TRUE){

            hm<-layout(hm,

                # x axis
                xaxis=list(
                    domain=c(0.025,0.50),
                    family="Times New Roman",
                    tickfont=list(size=10)
                ),
                xaxis2=list(
                    domain=c(0.50,0.55)
                ),
                xaxis3=list(
                    domain=c(0.55,1)
                )
            )
        }else{

            # modify layout
            hm<-layout(
                hm,

                # x axis
                xaxis=list(
                    domain=c(0.025,0.55),
                    family="Times New Roman",
                    tickfont=list(size=10)
                ),
                xaxis2=list(
                    domain=c(0.55,1)
                )
            )
        }

        # remove row side colors text mention
        if(length(hm$x$layout$annotations)>0){
            hm$x$layout$annotations[[1]]$text<-""
        }

        # custom row text
        row.text=gsub(
            "^.+GO.name: ",
            "",
            row.names(mat)
        )
        
        # cut very long definition
        row.text[nchar(row.text)>50]<-paste(
            substring(
                row.text[nchar(row.text)>50],
                1,
                50
            ),
            "...",
            sep=""
        )
    
        # copy
        row.names(mat)<-row.text

        # static heatmap
        if(showIC){
            hs<-Heatmap(
                mat,
                raster_device="png",
                use_raster = TRUE,
                cluster_rows=dd.row,
                cluster_columns = if(!is.null(col.dist)){dd.col}else{FALSE},
                show_row_names = showGOlabels,
                show_column_names = TRUE,
                col=if(slot(slot(myGOs,"enrich_GOs"),"same_genes_background")){
                    colorRamp2(c(0,1.3,max(mat,na.rm=TRUE)), c(heatmap_colors[1],heatmap_colors[1],heatmap_colors[2]))
                }else{
                    structure(c(heatmap_colors[1],heatmap_colors[2]),names = c("0","1"))
                },
                row_names_side = "left",
                row_dend_side = "right",
                row_dend_width = grid::unit(4, "cm"),
                row_names_max_width = max_text_width(
                    rownames(mat), 
                ),
                name=if(slot(slot(myGOs,"enrich_GOs"),"same_genes_background")){"-Log10 pvalue"}else{"significance"},
                right_annotation=rowAnnotation(
                    IC=IC,
                    col=list(
                        IC=colorRamp2(c(0,max(IC[is.finite(IC)])), c("#FFFFFF","#49006A"))
                    ),
                    annotation_name_gp=grid::gpar(fontsize = 10)
                )
            )
        }else{
            hs<-Heatmap(
                mat,
                raster_device="png",
                use_raster = TRUE,
                cluster_rows=dd.row,
                cluster_columns = if(!is.null(col.dist)){dd.col}else{FALSE},
                show_row_names = showGOlabels,
                show_column_names = TRUE,
                col=if(slot(slot(myGOs,"enrich_GOs"),"same_genes_background")){
                    colorRamp2(c(0,1.3,max(mat,na.rm=TRUE)), c(heatmap_colors[1],heatmap_colors[1],heatmap_colors[2]))
                }else{
                    structure(c(heatmap_colors[1],heatmap_colors[2]),names = c("0","1"))
                },
                row_names_side = "left",
                row_dend_side = "right",
                row_dend_width = grid::unit(4, "cm"),
                row_names_max_width = max_text_width(
                    rownames(mat), 
                ),
                name=if(slot(slot(myGOs,"enrich_GOs"),"same_genes_background")){"-Log10 pvalue"}else{"significance"},
            )
        }

        # hm to list
        hm<-list(hm,hs)

        # give names to hm list
        names(hm)<-c("GOterms","GOterms_static")

        # create an empty column dendrogram if needed
        if(!"dd.col"%in%ls()){dd.col=NULL}

        ## export results

        # bind data
        sResults<-data.table(
            GO.cluster=as.character(row.gp),
            IC=IC,
            sResults
        )

        # ordering results
        sResults<- sResults[row.ord]

        # replace data by sResults in myGOs
        slot(slot(myGOs,"enrich_GOs"),"data")<-sResults

        # remove not used elements in row.tree
        if(!is.null(row.tree$cut$static)){

            # keep only static parameters
            row.tree$cut$dynamic<-NULL

            # remove not used elements in row.tree
            if(row.tree$cut$static<=1){

                # add h to static
                names(row.tree$cut)<-paste(names(row.tree$cut),"(h)")

            }else{

                # add k to static
                names(row.tree$cut)<-paste(names(row.tree$cut),"(k)")
            }
        }

        # return heatmap
        new(
            "GO_clusters",
            ont=slot(myGOs,"ont"),
            enrich_GOs=slot(myGOs,"enrich_GOs"),
            IC=slot(myGOs,"IC"),
            terms_dist=slot(myGOs,"terms_dist")[row.tree$tree$distance],
            hcl_params=list(
                heatmap=list(
                    showIC=showIC,
                    showGOlabels=showGOlabels,
                    heatmap_colors=heatmap_colors
                ),
                GO.tree=row.tree,
                sample.tree=col.tree
            ),
            dendrograms=list(samples=dd.col,GO=dd.row),
            samples.gp=col.gp,
            heatmap=hm
        )
    }
)

#' @title Merge enriched GO terms.
#' @description combine results from GO enrichment tests (obtained with \pkg{topGO} package) or from \pkg{fgsea} (obtained with \code{\link{runfgsea}} method),
#' for a given ontology (MF, BP, or CC).
#' @import data.table
#' @import biomaRt
#' @import topGO
#' @import AnnotationDbi
#' @importFrom GO.db GO.db
#' @family GO_terms
#' @param Input a list containing named elements. Each element must contain the name of:
#' \itemize{
#'     \item{\pkg{topGO}: \code{\link[topGO]{topGOdata-class}} object created by \code{\link{create_topGOdata}} method and the associated \code{\link[topGO]{topGOresult-class}} object.}
#'     \item{\pkg{fgsea}: \code{\link{fgsea-class}} object created by \code{\link{runfgsea}} method.}
#' }
#' @param cutoff default pvalue cutoff (default to 0.01). Several cutoff can be use in the same order as list elements.
#' @param envir objects environment (default to .GlobalEnv).
#' @details This method extracts for each result of GO enrichment test:
#' informations about GO term (identifiant, name, and description),
#' gene frequency (number of significant genes / Annotated genes), pvalue, -log10(pvalue), significant genes
#' identifiants (GeneID, or Ensembl ID, or uniprot accession), and gene symbols.
#' At the last, this method builds a merged data.table of enriched GO terms
#' at least once and provides all mentionned columns.
#' @return an \code{\link{enrich_GO_terms-class}} object.
#' @references
#' Matt Dowle and Arun Srinivasan (2017). data.table: Extension of data.frame. R package version 1.10.4. https://CRAN.R-project.org/package=data.table
#'
#' Herve Pages, Marc Carlson, Seth Falcon and Nianhua Li (2017). AnnotationDbi: Annotation Database Interface. R package version 1.38.0.
#' @include enrich_GO_terms.R
#' @examples
#' ## topGO terms enrichment
#'
#' # load genes identifiants (GeneID,ENS...) universe/background (Expressed genes)
#' background_L<-scan(
#'     system.file(
#'         "extdata/data/input",
#'         "background_L.txt",
#'         package = "ViSEAGO"
#'     ),
#'     quiet=TRUE,
#'     what=""
#' )
#'
#' # load Differentialy Expressed (DE) gene identifiants from files
#' PregnantvslactateDE<-scan(
#'     system.file(
#'         "extdata/data/input",
#'         "pregnantvslactateDE.txt",
#'         package = "ViSEAGO"
#'     ),
#'     quiet=TRUE,
#'     what=""
#' )
#'
#' VirginvslactateDE<-scan(
#'     system.file(
#'         "extdata/data/input",
#'         "virginvslactateDE.txt",
#'         package = "ViSEAGO"
#'     ),
#'     quiet=TRUE,
#'     what=""
#' )
#'
#' VirginvspregnantDE<-scan(
#'     system.file(
#'         "extdata/data/input",
#'         "virginvspregnantDE.txt",
#'         package="ViSEAGO"
#'     ),
#'     quiet=TRUE,
#'     what=""
#' )
#'
#' \dontrun{
#' # connect to Bioconductor
#' Bioconductor<-ViSEAGO::Bioconductor2GO()
#'
#' # load GO annotations from Bioconductor
#' myGENE2GO<-ViSEAGO::annotate(
#'     "org.Mm.eg.db",
#'     Bioconductor
#' )
#'
#' # create topGOdata for BP for each list of DE genes
#' BP_Pregnantvslactate<-ViSEAGO::create_topGOdata(
#'     geneSel=PregnantvslactateDE,
#'     allGenes=background_L,
#'     gene2GO=myGENE2GO,
#'     ont="BP",
#'     nodeSize=5
#' )
#'
#' BP_Virginvslactate<-ViSEAGO::create_topGOdata(
#'     geneSel=VirginvslactateDE,
#'     allGenes=background_L,
#'     gene2GO=myGENE2GO,
#'     ont="BP",
#'     nodeSize=5
#' )
#'
#' BP_Virginvspregnant<-ViSEAGO::create_topGOdata(
#'     geneSel=VirginvspregnantDE,
#'     allGenes=background_L,
#'     gene2GO=myGENE2GO,
#'     ont="BP",
#'     nodeSize=5
#' )
#'
#' # perform TopGO tests
#' elim_BP_Pregnantvslactate<-topGO::runTest(
#'     BP_L_pregnantvslactate,
#'     algorithm ="elim",
#'     statistic = "fisher"
#' )
#'
#' elim_BP_Virginvslactate<-topGO::runTest(
#'     BP_L_virginvslactate,
#'     algorithm ="elim",
#'     statistic = "fisher"
#' )
#'
#' elim_BP_Virginvspregnant<-topGO::runTest(
#'     BP_L_virginvspregnant,
#'     algorithm ="elim",
#'     statistic = "fisher"
#' )
#'
#' # merge topGO results
#' BP_sResults<-ViSEAGO::merge_enrich_terms(
#'     Input=list(
#'         Pregnantvslactate=c("BP_Pregnantvslactate","elim_BP_Pregnantvslactate"),
#'         Virginvslactate=c("BP_Virginvslactate","elim_BP_Virginvslactate"),
#'         Virginvspregnant=c("BP_Virginvspregnant","elim_BP_Virginvspregnant")
#'     )
#' )
#' }
#' 
#' ## fgsea analysis
#' 
#' # load gene identifiants and padj test results from Differential Analysis complete tables
#' PregnantvsLactate<-data.table::fread(
#'     system.file(
#'         "extdata/data/input",
#'         "pregnantvslactate.complete.txt",
#'         package = "ViSEAGO"
#'     ),
#'     select = c("Id","padj")
#' )
#'
#'VirginvsLactate<-data.table::fread(
#'     system.file(
#'         "extdata/data/input",
#'         "virginvslactate.complete.txt",
#'         package = "ViSEAGO"
#'    ),
#'    select = c("Id","padj")
#')
#'
#' VirginvsPregnant<-data.table::fread(
#'     system.file(
#'        "extdata/data/input",
#'        "virginvspregnant.complete.txt",
#'         package = "ViSEAGO"
#'     ),
#'     select = c("Id","padj")
#' )
#'
#' # rank Id based on statistical value (padj)
#' PregnantvsLactate<-data.table::setorder(PregnantvsLactate,padj)
#'
#' VirginvsLactate<-data.table::setorder(VirginvsLactate,padj)
#'
#' VirginvsPregnant<-data.table::setorder(VirginvsPregnant,padj)
#'
#' \dontrun{
#' # connect to Bioconductor
#' Bioconductor<-ViSEAGO::Bioconductor2GO()
#'
#' # load GO annotations from Bioconductor
#' myGENE2GO<-ViSEAGO::annotate(
#'     "org.Mm.eg.db",
#'     Bioconductor
#' )
#'
#' # perform fgseaMultilevel tests
#' BP_PregnantvsLactate<-runfgsea(
#'     geneSel=PregnantvsLactate,
#'     gene2GO=myGENE2GO, 
#'     ont="BP",
#'     params = list(
#'         scoreType = "pos",
#'         minSize=5
#'     )
#' )
#'
#' BP_VirginvsLactate<-runfgsea(
#'     geneSel=VirginvsLactate,
#'     gene2GO=myGENE2GO, 
#'     ont="BP",
#'     params = list(
#'         scoreType = "pos",
#'         minSize=5
#'     )
#' )
#'
#' BP_VirginvsPregnant<-runfgsea(
#'     geneSel=VirginvsPregnant,
#'     gene2GO=myGENE2GO, 
#'     ont="BP",
#'     params = list(
#'         scoreType = "pos",
#'         minSize=5
#'     )
#' )
#'
#' # merge fgsea results
#' BP_sResults<-merge_enrich_terms(
#'     cutoff=0.01,
#'     Input=list(
#'         PregnantvsLactate="BP_PregnantvsLactate",
#'         VirginvsLactate="BP_VirginvsLactate",
#'         VirginvsPregnant="BP_VirginvsPregnant"
#'     )
#' )
#' }
#' @exportMethod merge_enrich_terms
#' @name merge_enrich_terms
#' @rdname merge_enrich_terms-methods
#' @exportMethod merge_enrich_terms
setGeneric(
    name="merge_enrich_terms",
    def=function(
        Input,
        cutoff=0.01,
        envir=.GlobalEnv
    ){
        standardGeneric("merge_enrich_terms")
    }
)

#' @rdname merge_enrich_terms-methods
#' @aliases merge_enrich_terms
setMethod(
    "merge_enrich_terms",
    signature(
        Input="list"
    ),
    definition=function(Input,cutoff,envir){

        ## cutoff

        # check cutoff length according Input
        if(length(cutoff)>1 & length(cutoff)!=length(Input)){
            stop("cutoff must be a single value to repet for each list element, or the same length than list")
        }

        # repet cutoff value according Input length
        if(length(cutoff)==1){
            cutoff<-rep(cutoff,length(Input))
        }

        ## check ontology

        # same ontology ?
        check.onto=unique(
            unlist(
                lapply(seq_along(Input),function(x){

                    # extract quering objects names
                    x=Input[[x]]

                    # check existence
                    values<-ls(envir=envir)

                    # check if available
                    available<-x%in%values

                    # stop if not found
                    if(!all(available)){
                        stop(paste("objects not found:",paste(x[!available],collapse=", "),sep="\n"))
                    }

                    # get objects
                    x=mget(x,envir=envir)

                    # objects type
                    obj.type=vapply(x,class,"")

                    # extract ontology type
                    vapply(seq_along(x),function(y){

                        # extract ontology slot
                        if(obj.type[y]%in%c("topGOdata","fgsea")){

                            # for topGOdata
                            slot(x[[y]],"ontology")

                        }else{

                            # for topGOresult
                            sub("^.+\nOntology: ","",slot(x[[y]],"description"))
                        }
                    },"")
                })
            )
        )

        # check ontology
        if(length(check.onto)>1){
            stop("Only one ontology supported")
        }

        # extract method used (topGO or fgsea)
        check.method=unique(
            unlist(
                lapply(seq_along(Input),function(x){
                    
                    # extract quering objects names
                    x=Input[[x]]
                    
                    # get objects
                    x=mget(x,envir=envir)
                    
                    # objects type
                   vapply(x,class,"")
                })
            )
        )

        # same method used
        if(length(check.method)==3){
            stop("topGO enrichment and fgsea results can't be merged")
        }

        # if two object --> topGO
        if(length(check.method)==2){check.method<-"topGO"}

        ## gene symbols add functions
        genes_symbols_add=function(db,genes){

            # if db match to bioconductor
            if(db[1]=="Bioconductor"){

                # load GeneID and symbols
                annot<-data.table(
                    select(
                        get(db[2]),
                        keys=keys(get(db[2])),
                        columns =c("ENTREZID","SYMBOL")
                    )
                )

                # if found symbols
                if(nrow(annot[!is.na(annot$ENTREZID)])>0){

                    # load GeneID and symbols
                    genes<-merge(
                        genes,
                        annot,
                        by.x="Significant_genes",
                        by.y="ENTREZID",
                        all.x=TRUE
                    )

                }else{

                    # add empty name columns
                    genes[,Name:=NA]
                }
            }

            # if db match to enstrezGene
            if(db[1]=="EntrezGene"){

                # function for generate data packets defined by the "by" argument
                pos=function(Data,by=""){
                    if(by>length(Data)){
                        by=length(Data)
                    }
                    if(length(Data)<=by){
                        data.table(
                            start=1,
                            end=length(Data)
                        )
                    }else{
                        data.table(
                            start=seq(1,length(Data),by=by),
                            end=unique(c(seq(by,length(Data),by=by),length(Data)))
                        )
                    }
                }

                # pattern.extract
                pattern.extract=function(query,m){
                    query=lapply(seq_along(query),function(i){

                        if(length(na.omit(m[[i]][1]))>0){
                            a=attr(m[[i]],"capture.start")
                            t(substring(query[i],a,a+attr(m[[i]],"capture.length")-1))
                        }else{
                            NA
                        }
                    })
                    query=do.call("rbind",query)
                    query[query%in%""]<-NA
                    query[query%in%"\t"]<-NA
                    query
                }

                # esummary
                esummary<-function(...){

                    # submitted Data
                    Data<-na.omit(unique(unique(...)))

                    # batch size
                    wpos=pos(Data,by=400)

                    # core address
                    core="https://eutils.ncbi.nlm.nih.gov/entrez/eutils/esummary.fcgi?version=2.0&db="

                    # Data submission an retrieve
                    query=lapply(seq_len(nrow(wpos)),function(x){

                        # build block of id
                        Dat<-paste(
                            Data[wpos[x,start]:wpos[x,end]],
                            collapse=","
                        )

                        # submit block
                        query <- paste(core,"gene","&id=",Dat,sep = "")

                        # read and return reults
                        readLines(query)
                    })
                    query=paste(
                        unlist(query),
                        collapse=""
                    )

                    # parse results
                    query<-substring(
                        query,
                        unlist(gregexpr("<DocumentSummary ",query)),
                        unlist(gregexpr("</DocumentSummary>",query))
                    )

                    # extraction pattern
                    pattern="<DocumentSummary uid=\"(?<Id>[[:digit:]]*)\">\t<Name>(?<Name>.*)</Name>"

                    # locate elements
                    m1=gregexpr(
                        paste(pattern,collapse=""),
                        query,
                        perl=TRUE
                    )

                    # extract elements
                    res=data.table(
                        pattern.extract(query,m1)
                    )

                    # add an header
                    names(res)=attr(m1[[1]],"capture.name")

                    # return
                    return(res)
                }

                # get esummary from NCBI
                annot<-esummary(genes$Significant_genes)

                # if found symbols
                if(nrow(annot[!is.na(annot$Id)])>0){

                    # load GeneID and symbols
                    genes<-merge(
                        genes,
                        annot,
                        by.x="Significant_genes",
                        by.y="Id",
                        all.x=TRUE
                    )

                }else{

                    # add empty name columns
                    genes[,Name:=NA]
                }
            }

            # if db  match to Ensembl
            if(db[1]=="Ensembl"){

                if(db[4]==""){

                    # Connect to Ensembl
                    if(db[3]=="genes"){

                        # connect to Ensembl
                        mart<-useEnsembl(
                            biomart=db[3],
                            version=tail(db,n=1)
                            
                        )
                        
                    }else{

                        # connect to Ensembl
                        mart<-useEnsemblGenomes(
                            biomart=db[3]
                        )
                    }
                }else{
                    # connect to Ensembl
                    mart<-useEnsembl(
                        biomart="genes",
                        GRCh =db[4],
                        version=tail(db,n=1)
                    )
                }

                # connect to ensembl specified dataset
                myspecies<-useDataset(
                    db[2],
                    mart
                )

                # load Ensembl genes, with GO annotations
                annot<-data.table(
                    getBM(
                        attributes =c("ensembl_gene_id","external_gene_name"),
                        values=TRUE,
                        mart =myspecies
                    )
                )

                # if found symbols
                if(nrow(annot[!"",on="external_gene_name"])>0){

                    # load GeneID and symbols
                    genes<-merge(
                        genes,
                        annot,
                        by.x="Significant_genes",
                        by.y="ensembl_gene_id",
                        all.x=TRUE
                    )

                }else{

                    # add empty name columns
                    genes[,Name:=NA]
                }
            }

            # if db  match to Custom
            if(db[1]=="Uniprot-GOA"){

                # temp file
                temp<-paste(
                    tempfile(),
                    "gz",
                    sep="."
                )

                # load the file
                download.file(
                    paste(
                        'https://ftp.ebi.ac.uk/pub/databases/GO/goa/',
                        toupper(db[2]),
                        '/goa_',
                        db[2],
                        '.gaf.gz',
                        sep=""
                    ),
                    destfile =temp,
                    quiet=TRUE,
                   # method="internal"
                )

                # unzip
                gunzip(temp)
                
                # read file
                annot<-unique(
                    fread(
                        sub("\\.gz","",temp),
                        skip=12,
                        select=c(2,3),
                        col.names=c("gene_id","gene_symbol")
                    )
                )

                # merge gene_symbol
                genes<-merge(
                    genes,
                    annot,
                    by.x="Significant_genes",
                    by.y="gene_id",
                    all.x=TRUE
                )
            }

            # if db  match to Custom
            if(db[1]=="Custom"){

                # load GeneID and symbols
                annot<-unique(
                    fread(
                        db[3],
                        select=c("gene_id","gene_symbol")
                    )
                )

                # merge gene_symbol
                genes<-merge(
                    genes,
                    annot,
                    by.x="Significant_genes",
                    by.y="gene_id",
                    all.x=TRUE
                )
            }
        
            # load GeneID and symbols
            names(genes)[3]<-"Significant_genes_symbol"
        
            # reorder columns
            genes<-genes[,c("GO.ID","Significant_genes","Significant_genes_symbol"),with=FALSE]
        
            # collapse results
            genes<-genes[,lapply(.SD,function(x){paste(x,collapse=";")}),.SDcols=2:3,by="GO.ID"]
        
            # replace all blank cells by NA
            genes[genes==""]<-NA
        
            # return genes
            return(genes)
        }

        ## apply

        # topGO method
        if(check.method=="topGO"){

            ## topGO summary informations

            # build topGO summary
            obj_summary=lapply(seq_along(Input),function(x){

                # keep pos
                pos=x

                # extract quering objects names
                x=Input[[x]]

                # keep names
                x_names=x

                # get objects
                x=mget(x,envir=envir)

                # objects type
                obj.type=vapply(x,class,"")

                # extract topGO objects summary
                topGO<-lapply(seq_along(x),function(y){

                    # extract ontology slot
                    if(obj.type[y]=="topGOdata"){

                        # for topGOdata
                        list(

                            # description
                            description=slot(x[[y]],"description"),

                            # availables genes
                            available_genes=length(slot(x[[y]],"allGenes")),

                            # availables genes significant
                            available_genes_significant=table(slot(x[[y]],"allScores"))[2],

                            # feasibles genes
                            feasible_genes=table(slot(x[[y]],"feasible"))[2],

                            # feasibles genes significant
                            feasible_genes_significant=table(slot(x[[y]],"allScores")==1 & slot(x[[y]],"feasible")==TRUE)[2],

                            # nodes with at least  x genes
                            genes_nodeSize=slot(x[[y]],"nodeSize"),

                            # number of nodes
                            nodes_number=length(slot(slot(x[[y]],"graph"),"nodes")),

                            # number of edges
                            edges_number=length(slot(slot(slot(x[[y]],"graph"),"edgeData"),"data"))
                        )

                    }else{

                        # for topGO result
                        list(

                            # description
                            description=slot(x[[y]],"description"),

                            # test name
                            test_name=sub(": ","p<",slot(x[[y]],"testName")),

                            # algorithm name
                            algorithm_name=slot(x[[y]],"algorithm"),

                            # scored GOs
                            GO_scored=length(slot(x[[y]],"score")),

                            # significant GOs according cutOff
                            GO_significant=table(slot(x[[y]],"score")<cutoff[pos])[2],

                            # feasibles genes
                            feasible_genes=slot(x[[y]],"geneData")[1],

                            # feasibles genes significant
                            feasible_genes_significant=slot(x[[y]],"geneData")[2],

                            # nodes with at least  x genes
                            genes_nodeSize=slot(x[[y]],"geneData")[3],

                            # nodes with at least  x genes
                            Nontrivial_nodes=slot(x[[y]],"geneData")[4]
                        )
                    }
                })

                # extract topGO objects summary
                names(topGO)<-x_names

                # return topGO
                topGO
            })

            # add names to topGO summary
            names(obj_summary)<-names(Input)

            # find enrich GOs in a least one comparison
            GOs<-lapply(seq_along(Input),function(x){

                # objects type
                Data=mget(Input[[x]],envir=envir)

                ## checking step

                # objects type
                obj.type=vapply(Data,class,"")

                # objects type
                if(sum(obj.type%in%"topGOdata")>1){

                    # stop if more than one godata by list
                    stop("Only one topGOdata object is supported by list")
                }

                # objects type
                if(sum(obj.type%in%"topGOresult")>1){

                    # stop if more than one godata by list
                    stop("Only one topGOresult object is supported by list")
                }

                ## find and extract pvalues

                # find topGOresult
                pos=which(obj.type=="topGOresult")

                # extract scores
                pvalues<-topGO::score(Data[[pos]])

                # extract names of enrich terms
                as.vector(
                    names(
                        pvalues[pvalues<cutoff[x]]
                    )
                )
            })

            # remove redondancy and convert to vector
            GOs<-as.vector(unique(unlist(GOs)))

            ## check genes background

            # extract genes background
            allgenes<-lapply(seq_along(Input),function(x){

                # objects type
                Data=mget(Input[[x]],envir=envir)

                # objects type
                obj.type=vapply(Data,class,"")

                # load GOdata
                pos=which(obj.type=="topGOdata")

                slot(Data[[pos]],"allGenes")
            })

            # check if same gene background
            if(length(Input)>1){
                same_genes_background=all(
                    vapply(2:length(allgenes),function(x){
                        identical(sort(allgenes[[1]]),sort(allgenes[[x]]))
                    },TRUE)
                )
            }else{
                same_genes_background=TRUE
            }

            # stop if no enrich GO terms
            if(length(GOs)==0){
                stop("No enrich GO terms available in at least one condition")
            }

            # initialize input
            input=list()

            # combine results
            allResults<-lapply(seq_along(Input),function(x){

                ## extract Data

                # objects type
                Data=mget(Input[[x]],envir=envir)

                # objects type
                obj.type=vapply(Data,class,"")

                # load GOdata
                pos=which(obj.type=="topGOdata")

                # load GOdata
                GOdata=Data[[pos]]

                # tested algorithm
                algorithm<-Data[-pos]

                ## extract some statistics from initial GOdata object (before enrichment test)

                # get the GOdatatermStat(GOdata)
                Stats<-termStat(
                    GOdata,
                    whichGO = GOs
                )

                # convert to data.table
                Stats<-data.table(
                    GO.ID=row.names(Stats),
                    genes_frequency=paste(
                        round(Stats$Significant/Stats$Annotated*100,digits=3),
                        "% (",Stats$Significant,"/",Stats$Annotated,")",
                        sep=""
                    )
                )

                ## extract genes identifiants

                # extract counts genes by term according GeneList
                genes<-scoresInTerm(
                    GOdata,GOs,
                    use.names = TRUE
                )

                # extract  genes Ids according GeneList
                genes=lapply(names(genes),function(x){

                    # extract significant terms
                    val=attr(genes[[x]][genes[[x]]==2],"names")

                    # build a table
                    data.table(
                        GO.ID=x,
                        Significant_genes=if(length(val)>0){val}else{NA}
                    )
                })

                # convert to data.table
                genes<-rbindlist(genes)

                ## add Genes symbols

                # get db
                db=strsplit(slot(GOdata,"description")," ")[[1]]

                # add genes symbols
                genes<-genes_symbols_add(db,genes)

                ## extract pvalue according the algorithm results

                # extract all pvalues from topGOresult
                pvalue<-topGO::score(algorithm[[1]])

                # select pvalues from topGOresult
                pvalue<-pvalue[GOs]

                # extract pvalue in data.table
                pvalues<-data.table(
                    GO.ID=names(pvalue),
                    pvalue=as.numeric(format(pvalue,scientific = T)),
                    `-log10_pvalue`=round(-log10(pvalue),digits=2)
                )

                # algorithm
                algo=slot(algorithm[[1]],"algorithm")

                # return input params
                assign(
                    "input",
                    c(input,list(algo)),
                    inherits=TRUE
                )

                ## combine results

                # all results in list
                Results<-list(
                    data.table(GO.ID=GOs),
                    Stats,
                    pvalues,
                    genes
                )

                # merge all
                Results<-Reduce(
                    function(...){
                        merge(
                            ...,
                            by ="GO.ID",
                            sort=FALSE,
                            all=TRUE
                        )
                    },
                    Results
                )

                # remove NA in GO.Id column
                Results<-Results[!is.na(Results$GO.ID)]

                # Remove gene ID and symbol if GO term not significant
                Results[Results$pvalue>=cutoff[x],`:=`(Significant_genes=NA,Significant_genes_symbol=NA)]

                if(!is.null(names(Input))){

                    # add GOdata name in the header
                    names(Results)[-1]<-paste(
                        names(Input)[x],
                        names(Results)[-1],
                        sep="."
                    )
                }

                # return Results
                Results
            })
        }

        # fgsea method
        if(check.method=="fgsea"){

            # get fgsea parameters
            obj_summary=lapply(seq_along(Input),function(x){

                # extract quering objects names
                x=Input[[x]]

                # get objects
                x=get(x,envir=envir)

                # extract parameters
                c(
                    method=slot(x,"method"),
                    slot(x,"params")
                )
            })

            # add names to fgsea summary
            names(obj_summary)<-names(Input)

            # find enrich GOs in a least one comparison
            GOs<-lapply(seq_along(Input),function(x){

                # objects type
                Data=slot(get(Input[[x]],envir=envir),"data")[[1]]

                # return enrich terms
                Data[pval<cutoff[x],pathway]
            })

            # remove redondancy and convert to vector
            GOs<-as.vector(unique(unlist(GOs)))

            ## check genes background

            # extract genes background
            allgenes<-lapply(seq_along(Input),function(x){

                # load input fgsea data input
                Data=slot(get(Input[[x]],envir=envir),"input")[[1]]

                # return gene identifiants
                Data[,Id]
            })

            # check if same gene background
            if(length(Input)>1){
                same_genes_background=all(
                    vapply(2:length(allgenes),function(x){
                        identical(sort(allgenes[[1]]),sort(allgenes[[x]]))
                    },TRUE)
                )
            }else{
                same_genes_background=TRUE
            }

            # stop if no enrich GO terms
            if(length(GOs)==0){
                stop("No enrich GO terms available in at least one condition")
            }

            # initialize input
            input=list()

            # combine results
            allResults<-lapply(seq_along(Input),function(x){

                # get Data
                Data=get(Input[[x]],envir=envir)

                # if fgseaSimple
                if(slot(Data,"method")=="fgseaSimple"){

                    # build stats table
                    Stats<-slot(Data,"data")[[1]][,.(GO.ID=pathway,padj,nMoreExtreme,ES,NES,size,genes_frequency)]
                }
                
                if(slot(Data,"method")=="fgseaMultilevel"){

                    # build stats table
                    Stats<-slot(Data,"data")[[1]][,.(GO.ID=pathway,padj,log2err,ES,NES,size,genes_frequency)]
                }

                # build genes table
                genes<-slot(Data,"data")[[1]][,.(GO.ID=pathway,Significant_genes=leadingEdge)]

                # unlist Significant_genes
                genes<-genes[,.(Significant_genes=unlist(Significant_genes)),by=GO.ID]

                # get db
                db=strsplit(slot(Data,"description")," ")[[1]]

                # add genes symbols
                genes<-genes_symbols_add(db,genes)

                ## extract pvalue according the algorithm results
                pvalues<-slot(Data,"data")[[1]][,.(GO.ID=pathway,pvalue=as.numeric(format(pval,scientific = T)),`-log10_pvalue`=round(-log10(pval),digits=2))]

                # algorithm
                algo=list(slot(Data,"params"))

                names(algo)<-names(Input)[x]
                
                # return input params
                assign(
                    "input",
                    c(input,algo),
                    inherits=TRUE
                )

                ## combine results

                # all results in list
                Results<-list(
                    data.table(GO.ID=GOs),
                    Stats,
                    pvalues,
                    genes
                )

                # merge all
                Results<-Reduce(
                    function(...){
                        merge(
                            ...,
                            by ="GO.ID",
                            sort=FALSE,
                            all.x=TRUE
                        )
                    },
                    Results
                )

                # remove NA in GO.Id column
                Results<-Results[!is.na(Results$GO.ID)]
                
                # Remove gene ID and symbol if GO term not significant
                Results[as.numeric(Results$pvalue)>=cutoff[x],`:=`(Significant_genes=NA,Significant_genes_symbol=NA)]
                
                if(!is.null(names(Input))){
                    
                    # add GOdata name in the header
                    names(Results)[-1]<-paste(
                        names(Input)[x],
                        names(Results)[-1],
                        sep="."
                    )
                }

                # return Results
                Results
            })
        }

        # number of Godata
        nb=length(allResults)

        # merge results if more than GOdata input
        if(nb>1){

            # merge all results
            allResults=Reduce(
                function(...){
                    merge(...,
                          by ="GO.ID",
                          sort=FALSE,
                          all=TRUE
                    )
                },
                allResults
            )

        }else{

            # if only one GOdata unlist table
            allResults<-allResults[[1]]
        }

        # add Input names
        names(input)<-names(Input)

        # extract GO terms
        GO<-data.table(
            select(
                GO.db,
                keys=allResults$GO.ID,
                columns = c("GOID","TERM","DEFINITION")
            )
        )

        # add GO term description and definition to sResults
        allResults<-data.table(
            GO,
            allResults[,"GO.ID":=NULL]
        )

        # rename the first 3 columns
        names(allResults)[seq_len(3)]<-c("GO.ID","term","definition")

        # significant results in at least one condition
        new(
            "enrich_GO_terms",
            same_genes_background=same_genes_background,
            ont=check.onto,
            method=check.method,
            summary=obj_summary,
            cutoff=list(cutoff),
            data=allResults
        )
    }
)

#' @title genomic_ressource class object definition.
#' @description This class stores the annotations and associated metadata obtained by \code{\link{Bioconductor2GO}},
#' \code{\link{EntrezGene2GO}}, \code{\link{Ensembl2GO}}, or \code{\link{Uniprot2GO}} .
#' @family genomic_ressource
#' @slot db name of database used (Bioconductor, EntrezGene, Ensembl, or Uniprot).
#' @slot stamp date of stamp (for Bioconductor, EntrezGene, and Uniprot), or annotation version for Ensembl database.
#' @slot data GO annotations from \code{\link{EntrezGene2GO}} method.
#' @slot organisms informations about species/datasets availables.
#' @slot mart Ensembl mart from \code{\link{Ensembl2GO}} method.
setClass(
    "genomic_ressource",
    slots=c(
        db="character",
        stamp="character",
        data="data.table",
        organisms="data.table",
        mart="list"
    )
)

#' @aliases genomic_ressource
setMethod(
    "show",
    signature="genomic_ressource",
    function(object){

        # cat some text
        cat("- object class: genomic_ressource",
            "\n- database: ",slot(object,"db"),
            "\n- stamp/version: ",slot(object,"stamp"),
            "\n- available organisms: ",nrow(slot(object,"organisms")),
            sep=""
        )
    }
)

#' @title Check available organisms datasets at Ensembl.
#' @description  List Ensembl referenced organisms datasets from the current (NULL) or archive (number in \code{character}) annotation version.
#' @importFrom biomaRt useEnsembl listEnsembl listEnsemblGenomes useEnsemblGenomes listDatasets 
#' @importFrom data.table data.table
#' @param biomart the biomart name available with \pkg{biomaRt} package \code{\link[biomaRt]{listEnsembl}} ("genes", the default) or \code{\link[biomaRt]{listEnsemblGenomes}} ("protists_mart", "fungi_mart", "plants_mart").
#' @param GRCh GRCh version to connect to if not the current GRCh38, currently this can only be 37
#' @param version the annotation version to use (eg. NULL for the default current version, or a version number in \code{character})
#' @family genomic_ressource
#' @details
#' This function gives referenced organisms genomes at \href{http://ensemblgenomes.org/}{Ensembl}.
#' It uses the \code{\link[biomaRt]{useEnsembl}} and  \code{\link[biomaRt]{listDatasets}} from \pkg{biomaRt} package.
#' @return a  \code{\link{genomic_ressource-class}} object required by \code{\link{annotate}}.
#' @references
#' Durinck S, Spellman P, Birney E and Huber W (2009).
#' Mapping identifiers for the integration of genomic datasets with the R/Bioconductor package biomaRt.
#' Nature Protocols, 4, pp. 1184-1191.
#'
#' Durinck S, Moreau Y, Kasprzyk A, Davis S, De Moor B, Brazma A and Huber W (2005).
#' BioMart and Bioconductor: a powerful link between biological databases and microarray data analysis. Bioinformatics, 21, pp. 3439-3440.
#'
#' Matt Dowle and Arun Srinivasan (2017). data.table: Extension of data.frame. R package version 1.10.4. https://CRAN.R-project.org/package=data.table.
#' @include genomic_ressource.R
#' @examples
#' \dontrun{
#' # check the Ensembl available biomart (if not known)
#' biomaRt::listEnsembl()
#'
#' # List Ensembl available organisms
#' Ensembl<-ViSEAGO::Ensembl2GO(
#'  biomart="genes",
#'  GRCh = NULL,
#'  version=NULL
#' )
#' }
#' @export
Ensembl2GO=function(biomart="genes",GRCh = NULL,version=NULL){

    # check the ensembl host versus mart name
    match.arg(biomart,c("genes","protists_mart","fungi_mart","metazoa_mart","plants_mart"))
    
    # Connect to Ensembl
    if(biomart=="genes"){

        # check the ensembl genes releases
        Ensembl<-listEnsembl(
            GRCh =GRCh,
            version=version
        )

        # connect to Ensembl
        mart<-useEnsembl(
            biomart,
            GRCh =GRCh,
            version=version
        )

    }else{

        # check the ensembl genes releases
        Ensembl<-listEnsemblGenomes()

        # connect to Ensembl
        mart<-useEnsemblGenomes(biomart)
    }

    # return data in genomic_ressource class
    new(
        "genomic_ressource",
        db="Ensembl",
        stamp=paste(biomart,GRCh,Ensembl$version[Ensembl$biomart==biomart]),
        data=data.table(),
        mart=list(mart),
        organisms=data.table(listDatasets(mart))
    )
}

#' @title Download EntrezGene orthologs groups.
#' @description  Download EntrezGene orthologs groups.
#' @importFrom data.table data.table fread .SD
#' @importFrom R.utils gunzip
#' @details Internal function used by \code{\link{annotate}} in order to download orthologs_groups from NCBI Annotation pipeline
#'  stored in the \href{ftp://ftp.ncbi.nih.gov/gene/DATA/gene_orthologs.gz}{gene_group.gz} file.
#' @return a \code{\link[data.table]{data.table}}.
#' @examples
#' \dontrun{
#' # Organism taxid, Scientific name and common name
#' ViSEAGO::EntrezGene_orthologs()
#' }
#' @keywords internal
#' @export
EntrezGene_orthologs=function(){

    # temp file
    temp<-paste(
        tempfile(),
        "gz",
        sep="."
    )

    # import gene_group from NCBI
    download.file(
        "https://ftp.ncbi.nih.gov/gene/DATA/gene_orthologs.gz",
        quiet=TRUE,
        destfile =temp,
        method = "libcurl"
    )

    # read the file
    gene_orthologs=fread(
        temp,
        verbose=FALSE,
        showProgress=FALSE
    )

    # remove # in header
    names(gene_orthologs)<-gsub("#","",names(gene_orthologs))

    # convert to character
    gene_orthologs<-gene_orthologs[,lapply(.SD,as.character),.SDcols=names(gene_orthologs)]

    # return the table
    return(gene_orthologs)
}

#' @title Display available organisms from a specified database.
#' @description Display an interactive table with available organisms from a genomic ressource database (Bioconductor, EntrezGene, Ensembl, Uniprot).
#' @importFrom DT datatable
#' @family genomic_ressource
#' @family visualization
#' @param object a  \code{\link{genomic_ressource-class}} object created by \code{\link{Bioconductor2GO}}, \code{\link{EntrezGene2GO}},
#' \code{\link{Ensembl2GO}},or \code{\link{Uniprot2GO}} methods.
#' @details
#' an interactive \code{datatable}.
#' @return javascript datatable
#' @references
#'  Yihui Xie (2016). DT: A Wrapper of the JavaScript Library 'DataTables'. R package version 0.2. https://CRAN.R-project.org/package=DT
#' @include genomic_ressource.R
#' @examples
#' # display Bioconductor table
#' Bioconductor<-ViSEAGO::Bioconductor2GO()
#' ViSEAGO::available_organisms(Bioconductor)
#' \dontrun{
#'
#' # display EntrezGene table
#' EntrezGene<-ViSEAGO::EntrezGene2GO()
#' ViSEAGO::available_organisms(EntrezGene)
#'
#' # display Ensembl table
#' Ensembl<-ViSEAGO::Ensembl2GO()
#' ViSEAGO::available_organisms(Ensembl)
#'
#' # display Uniprot table
#' Uniprot<-ViSEAGO::Uniprot2GO()
#' ViSEAGO::available_organisms(Uniprot)
#' }
#' @name available_organisms
#' @rdname available_organisms-methods
#' @exportMethod available_organisms
setGeneric(
    name="available_organisms",
    def=function(object){
        standardGeneric("available_organisms")
})

#' @rdname available_organisms-methods
#' @aliases available_organisms
setMethod(
    "available_organisms",
    signature="genomic_ressource",
    definition=function(object){

        # create a datatable
        datatable(

            # table
            slot(object,"organisms"),

            # table width
            width=650,

            # filter column
            filter ='top',

            # extensions to use
            extensions =c('Scroller'),

            # DT options to use
            options=list(

                # the dom
                dom='ltipr',

                # page length
                pageLength =5,

                # y scroller
                deferRender = TRUE,
                scrollY = 200,
                scroller = TRUE
            )
        )
})

#' @title Compute distance between GO terms or GO clusters based on semantic similarity.
#' @description This method computes distance between GO terms or GO clusters based on semantic similarity.
#' @importFrom GO.db GO.db GOMFANCESTOR GOBPANCESTOR GOCCANCESTOR
#' @import GOSemSim
#' @import AnnotationDbi
#' @import data.table
#' @importFrom utils stack unstack
#' @family  GO_semantic_similarity
#' @param object a \code{\link{GO_SS-class}}, or  \code{\link{GO_clusters-class}} objects created by \code{\link{build_GO_SS}}
#'  or \code{\link{GOterms_heatmap}} methods, respectively.
#' @param distance The available methods for calculating GO terms Semantic Similarity (SS) are
#' "Resnik", "Rel", "Lin", and "Jiang" which are based on Information Content (IC), and "Wang" which is based on graph topology.\cr
#' The available methods for calculating clusters of GO terms SS are "max", "avg","rcmax", and "BMA".
#' @details This method computes semantic similarity distances between all GO terms
#' provided by \code{\link{GO_SS-class}} object.\cr
#' This method also computes semantic similarity distances between all GO clusters
#' provided by \code{\link{GO_clusters-class}} object.\cr
#'
#' Semantic Similarity computations are based on \code{\link[GOSemSim]{mgoSim}} method from the \pkg{GoSemSim} package.
#' @return a \code{\link{GO_SS-class}}, or a \code{\link{GO_clusters-class}} object (same class as input object).
#' @references
#'
#' Marc Carlson (2017). GO.db: A set of annotation maps describing the entire Gene Ontology. R package version 3.4.1.
#'
#' Guangchuang Yu, Fei Li, Yide Qin, Xiaochen Bo, Yibo Wu and Shengqi Wang. GOSemSim: an R package for measuring semantic similarity
#' among GO terms and gene products. Bioinformatics 2010 26(7):976-978
#'
#' Herve Pages, Marc Carlson, Seth Falcon and Nianhua Li (2017). AnnotationDbi: Annotation Database Interface. R package version 1.38.0.
#' @include GO_SS.R GO_clusters.R
#' @examples
#' # load data example
#' data(
#'     myGOs,
#'     package="ViSEAGO"
#' )
#'
#' \dontrun{
#' # compute GO terms Semantic Similarity distances
#' myGOs<-ViSEAGO::compute_SS_distances(
#'     myGOs,
#'     distance=c("Resnik","Lin","Rel","Jiang","Wang")
#' )
#'
#' # GOtermsHeatmap with default parameters
#' Wang_clusters_wardD2<-ViSEAGO::GOterms_heatmap(
#'     myGOs,
#'     showIC=TRUE,
#'     showGOlabels=TRUE,
#'     GO.tree=list(
#'         tree=list(
#'             distance="Wang",
#'             aggreg.method="ward.D2",
#'             rotate=NULL
#'         ),
#'         cut=list(
#'             dynamic=list(
#'                 pamStage=TRUE,
#'                 pamRespectsDendro=TRUE,
#'                 deepSplit=2,
#'                 minClusterSize=2
#'             )
#'         )
#'     ),
#'     samples.tree=NULL
#' )
#'
#' # compute clusters of GO terms Semantic Similarity distances
#' Wang_clusters_wardD2<-ViSEAGO::compute_SS_distances(
#'     Wang_clusters_wardD2,
#'     distance=c("max","avg","rcmax","BMA")
#' )
#' }
#' @name compute_SS_distances
#' @rdname compute_SS_distances-methods
#' @exportMethod compute_SS_distances
setGeneric(
    name="compute_SS_distances",
    def=function(object,distance){
        standardGeneric("compute_SS_distances")
    }
)

#' @rdname compute_SS_distances-methods
#' @aliases compute_SS_distances
setMethod(
    "compute_SS_distances",
    signature(
        object="ANY",
        distance="character"
    ),
    definition=function(object,distance) {

        # check object
        if(!is(object,"GO_SS") & !is(object,"GO_clusters")){

            stop("object must be a GO_SS or GO_clusters class from ViSEAGO::build_GO_SS() or ViSEAGO::GOterms_heatmap(), respectively")
        }

        if(is(object,"GO_SS")){

            # extract GO terms
            Terms<-slot(slot(object,"enrich_GOs"),"data")$GO.ID

            # for each distance
            for (x in distance){

                # check measure
                x=match.arg(x,c("Resnik","Lin","Rel","Jiang","Wang"))

                # convert GO similarity matrix to distance
                values=list(
                    as.dist(1-mgoSim(Terms,Terms,semData=object,measure=x,combine=NULL)))

                    # add distance name to values
                    names(values)=x

                    # return values
                    slot(object,"terms_dist")<-c(slot(object,"terms_dist"),values)
            }
        }else{

            # extract only selected colmuns from GOsResults
            GOclusters<-slot(slot(object,"enrich_GOs"),"data")[,c("GO.ID","term","GO.cluster"),with=FALSE]

            # cluster names
            clusters<-GOclusters[,c("GO.ID","GO.cluster"),with=FALSE]

            # convert clusters in factor
            clusters[,"GO.cluster":=factor(clusters$GO.cluster,levels = unique(clusters$GO.cluster))]

            # get ancestors
            onto=switch(
                slot(object,"ont"),
                MF=GOMFANCESTOR,
                BP=GOBPANCESTOR,
                CC=GOCCANCESTOR
            )

            # convert in list
            onto=AnnotationDbi::as.list(onto)

            # keep enrich terms
            onto=onto[clusters$GO.ID]

            # keep enrich terms
            onto=stack(onto)

            # add enrich terms
            onto=rbind(
                onto,
                cbind(
                    values=clusters$GO.ID,
                    ind=clusters$GO.ID
                )
            )

            # merge onto with GOclusters
            onto=merge(
                clusters[,c("GO.ID","GO.cluster"),with=FALSE],
                onto,
                by.x="GO.ID",
                by.y="ind",
                all.y=TRUE,
                sort=FALSE
            )

            # remove GO.ID
            onto=onto[,"GO.ID":=NULL]

            # count the term occurence by cluster
            onto=onto[,.N,by=c("GO.cluster","values")]

            # select the maxima term occurence by cluster
            onto=onto[onto[, .I[N==max(N)], by="GO.cluster"]$V1]

            # add IC to onto
            onto=data.table(onto,IC=slot(object,"IC")[onto$values])

            # select the first maxima term IC by cluster
            onto=onto[onto[, .I[which.max(IC)], by="GO.cluster"]$V1]

            # extract GO terms
            GO<-data.table(
                select(
                    GO.db,
                    keys=onto$values,
                    columns="TERM"
                )
            )

            # add sizes column to GOclusters
            onto=data.table(onto,GO)

            # convert clusters in factor
            clusters<-unstack(data.frame(clusters))

            # rename clusters
            names(clusters)<-paste(onto$GO.cluster,onto$GOID,onto$TERM,sep="_")

            ## compute clusters similarities

            # number of clusters
            n=length(clusters)

            # for each distance
            for (x in distance){

                # check measure
                x=match.arg(x,c("max","avg","rcmax","BMA"))

                # create default scores matrix
                scores <- matrix(NA, nrow = n, ncol = n)

                # create default scores matrix
                for (i in seq_along(clusters)){

                    # extract the first cluster GO terms
                    GO1 <-clusters[[i]]

                    # extract second cluster GO terms
                    for (j in seq_len(i)){

                        # extract the second cluster GO terms
                        GO2<-clusters[[j]]

                        # calculate
                        scores[i, j] <-mgoSim(
                            GO1,
                            GO2,
                            semData = object,
                            measure =names(slot(object,"terms_dist")),
                            combine =x
                        )

                        # add to upper part
                        if (i != j) scores[j, i] <- scores[i, j]
                    }
                }

                # add clusters name
                colnames(scores)<-names(clusters)
                row.names(scores)<-names(clusters)

                # return matrix
                values=list(stats::as.dist(1-scores))

                # add distance name to values
                names(values)=x

                # return values
                slot(object,"clusters_dist")<-c(slot(object,"clusters_dist"),values)
            }
        }

        # return the object
        return(object)
    }
)

#' @title Compute distance matrix between dendrograms partitions.
#' @description Build a distance or correlation matrix between partitions from dendrograms.
#' @importFrom data.table data.table setorderv
#' @importFrom igraph compare
#' @family GO_clusters
#' @param clusters  a \code{list} of \code{\link{GO_clusters-class}} objects,
#' from \code{\link{GOterms_heatmap}} or \code{\link{GOclusters_heatmap}}, named as \code{character}.
#' @param method available methods  ("vi", "nmi", "split.join", "rand", or "adjusted.rand") from \pkg{igraph} package \code{\link[igraph]{compare}} function.
#' @return a distance or a correlation matrix.
#' @references
#' Csardi G, Nepusz T: The igraph software package for complex network research, InterJournal, Complex Systems 1695. 2006. http://igraph.org.
#' @include GO_clusters.R
#' @examples
#' # load example object
#' data(
#'     myGOs,
#'     package="ViSEAGO"
#' )
#'
#' \dontrun{
#' # compute Semantic Similarity (SS)
#' myGOs<-ViSEAGO::compute_SS_distances(
#'     myGOs,
#'     distance=c("Resnik","Lin","Rel","Jiang","Wang")
#' )
#'
#' # Resnik distance GO terms heatmap
#' Resnik_clusters_wardD2<-ViSEAGO::GOterms_heatmap(
#'     myGOs,
#'     showIC=TRUE,
#'     showGOlabels=TRUE,
#'     GO.tree=list(
#'         tree=list(
#'             distance="Resnik",
#'             aggreg.method="ward.D2"
#'         ),
#'         cut=list(
#'             dynamic=list(
#'                 deepSplit=2,
#'                 minClusterSize =2
#'             )
#'         )
#'     ),
#'     samples.tree=NULL
#' )
#'
#' # Lin distance GO terms heatmap
#' Lin_clusters_wardD2<-ViSEAGO::GOterms_heatmap(
#'     myGOs,
#'     showIC=TRUE,
#'     showGOlabels=TRUE,
#'     GO.tree=list(
#'         tree=list(
#'             distance="Lin",
#'             aggreg.method="ward.D2"
#'         ),
#'         cut=list(
#'             dynamic=list(
#'                 deepSplit=2,
#'                 minClusterSize =2
#'             )
#'         )
#'     ),
#'     samples.tree=NULL
#' )
#'
#' # Resnik distance GO terms heatmap
#' Rel_clusters_wardD2<-ViSEAGO::GOterms_heatmap(
#'     myGOs,
#'     showIC=TRUE,
#'     showGOlabels=TRUE,
#'     GO.tree=list(
#'         tree=list(
#'             distance="Rel",
#'             aggreg.method="ward.D2"
#'         ),
#'         cut=list(
#'             dynamic=list(
#'                 deepSplit=2,
#'                 minClusterSize =2
#'             )
#'         )
#'     ),
#'     samples.tree=NULL
#' )
#'
#' # Resnik distance GO terms heatmap
#' Jiang_clusters_wardD2<-ViSEAGO::GOterms_heatmap(
#'     myGOs,
#'     showIC=TRUE,
#'     showGOlabels=TRUE,
#'     GO.tree=list(
#'         tree=list(
#'             distance="Jiang",
#'             aggreg.method="ward.D2"
#'         ),
#'         cut=list(
#'             dynamic=list(
#'                 deepSplit=2,
#'                 minClusterSize =2
#'             )
#'         )
#'     ),
#'     samples.tree=NULL
#' )
#'
#' # Resnik distance GO terms heatmap
#' Wang_clusters_wardD2<-ViSEAGO::GOterms_heatmap(
#'     myGOs,
#'     showIC=TRUE,
#'     showGOlabels=TRUE,
#'     GO.tree=list(
#'         tree=list(
#'             distance="Wang",
#'             aggreg.method="ward.D2"
#'         ),
#'         cut=list(
#'             dynamic=list(
#'                 deepSplit=2,
#'                 minClusterSize =2
#'             )
#'         )
#'     ),
#'     samples.tree=NULL
#' )
#' }
#' # clusters to compare
#' clusters<-list(
#'     Resnik="Resnik_clusters_wardD2",
#'     Lin="Lin_clusters_wardD2",
#'     Rel="Rel_clusters_wardD2",
#'     Jiang="Jiang_clusters_wardD2",
#'     Wang="Wang_clusters_wardD2"
#' )
#'
#' \dontrun{
#' # global dendrogram clustering correlation
#' clust_cor<-ViSEAGO::clusters_cor(
#'     clusters,
#'     method="adjusted.rand"
#' )
#' }
#' @name clusters_cor
#' @rdname clusters_cor-methods
#' @exportMethod clusters_cor
setGeneric(
    name="clusters_cor",
    def=function(clusters,method="adjusted.rand"){
        standardGeneric("clusters_cor")
    }
)

#' @rdname clusters_cor-methods
#' @aliases clusters_cor
setMethod(
    "clusters_cor",
    signature(
        clusters="list",
        method="character"
    ),
    definition=function(clusters,method){

        # method
        method=match.arg(
            method,c("vi", "nmi", "split.join", "rand","adjusted.rand")
        )

        ## extract clusters and GO.ID

        # extract clusters and GO.ID
        Clusters<-lapply(clusters,function(x){

            # load the target object
            object=get(x[[1]])

            # check class
            if(!is(object,"GO_clusters")){
                stop("object must come from ViSEAGO::GOterms_heatmap()")
            }

            # extract GO.ID and correspond cluster assignation to a list
            object<- slot(slot(object,"enrich_GOs"),"data")[,c("GO.ID","GO.cluster"),with=FALSE]

            # ordering by GO.ID
            setorderv(object,"GO.ID")

            # return the object
            object$GO.cluster
        })

        # add names
        names(Clusters)<-names(clusters)

        ## extract clusters and GO.ID

        # number of list elements
        n_list <- length(Clusters)

        # build o matrix of one
        the_cor <- matrix(1, n_list, n_list)

        # build pairwise combinations
        pairwise_combn <- combn(n_list, 2)

        # for each combination
        for (i in seq_len(ncol(pairwise_combn))) {

            # extract l1 elements
            l1 <- pairwise_combn[1, i]

            # extract l2 elements
            l2 <- pairwise_combn[2, i]

            # add the value to the_cor
            the_cor[l1, l2] <- the_cor[l2, l1] <-compare(Clusters[[l1]], Clusters[[l2]],method=method)
        }

        # add headers
        rownames(the_cor) <- colnames(the_cor)<-names(Clusters)

        # return the results
        the_cor
    }
)

#' @title GO_clusters class object
#' @description This class is invoked by \code{\link{GOterms_heatmap}} and  \code{\link{GOclusters_heatmap}} methods to store all results produced.
#' @family GO_clusters
#' @slot ont ontology used "MF", "BP", or "CC".
#' @slot enrich_GOs \code{\link{enrich_GO_terms-class}} object.
#' @slot IC Information Content (IC).
#' @slot terms_dist distance between GO terms based on semantic similiarity.
#' @slot clusters_dist distance between GO groups based on semantic similiarity.
#' @slot hcl_params Hierarchical clustering parameters used.
#' @slot dendrograms GO terms and samples \code{dendrograms}.
#' @slot samples.gp samples groups.
#' @slot heatmap GO terms and GO groups heatmaps.
#' @include enrich_GO_terms.R
setClass(
    "GO_clusters",
    slots=c(
        ont="character",
        enrich_GOs="enrich_GO_terms",
        IC ="numeric",
        terms_dist="list",
        clusters_dist="list",
        hcl_params="list",
        dendrograms="list",
        samples.gp="numeric",
        heatmap="list"
    )
)

#' @aliases GO_clusters
#' @importFrom data.table .SD melt.data.table :=
setMethod(
    "show",
    signature="GO_clusters",
    function(object){

        # keep full object
        full_object<-object

        # extract object
        object<-slot(object,"enrich_GOs")

        # Extract table
        Data<-slot(object,"data")

        # Extract pvalues
        Data<-Data[,grep("\\.pvalue",names(Data)),with=FALSE]

        # pvalues threshlod according condition
        p<-unlist(slot(object,"cutoff"))

        # count significant pvalues by condition
        Data<-lapply(seq_len(ncol(Data)),function(x){
            data.table(
                conditions=sub("\\.pvalue","",names(Data)[x]),
                `significant GO terms number`=sum(Data[,x,with=FALSE]<p[x],na.rm=TRUE)
            )
        })

        # bind results
        Data<-rbindlist(Data)
        
        # get topGO information
        obj_summary<-slot(object,"summary")

        # summary according method
        obj_summary<-vapply(names(obj_summary),function(x){

            # subset
            Data<-obj_summary[[x]]

            # topGO summary
            if(slot(object,"method")=="topGO"){
                
                # topGO summary by element
                elems<-paste(
                    vapply(names(Data),function(y){
                        
                        # summery by element
                        paste(
                            paste("   ",y),
                            "\n       ",
                            paste(
                                paste(
                                    names(Data[[y]]),
                                    sub("\n.+$","",unlist(Data[[y]])),
                                    sep=": "
                                ),
                                collapse="\n        "
                            )
                        )
                    },""),
                    collapse="\n  "
                )

                # summary by element
                return(paste(paste(x,elems,sep ="\n  "),"\n "))
            }

            # fgsea summary
            if(slot(object,"method")=="fgsea"){

                # fgsea summary by element
                elems<-paste(
                    paste(
                        "\n    ",
                        names(
                            Data
                        ),
                        " : ",
                        unlist(
                            Data
                        ),
                        sep=""
                    ),
                    collapse=""
                )

                # summary by element
                return(paste(paste("\n ",x,elems,sep =""),collapse="/n"))
            }
        },"")

        # cat some text
        cat("- object class: GO_clusters",
            "\n- ontology: ",slot(object,"ont"),
            "\n- method: ",slot(object,"method"),
            "\n- summary:\n", obj_summary,
            "- enrichment pvalue cutoff:",
            paste("\n       ",Data$conditions,":",slot(object,"cutoff")[[1]]),
            "\n- enrich GOs (in at least one list): ",nrow(slot(object,"data"))," GO terms of ",nrow(Data)," conditions.",
            paste("\n       ",Data$conditions,":",Data$`significant GO terms number`,"terms"),
            if(length(slot(full_object,"terms_dist"))>0){
                paste(
                    "\n- terms distances: ",
                    paste(
                        names(
                            slot(full_object,"terms_dist")
                        ),
                        collapse=", "
                    )
                )
            },
            "\n- clusters distances: ",
            paste(
                names(
                    slot(full_object,"clusters_dist")
                ),
                collapse=", "
            ),
            "\n- Heatmap:",
            "\n          * GOterms: ",
            !is.null(
                slot(full_object,"heatmap")$GOterms
            ),
            "\n                    - GO.tree:\n                              ",
            paste(
                paste(
                    names(
                        unlist(
                            slot(full_object,"hcl_params")$GO.tree
                        )
                    ),
                    unlist(
                        slot(full_object,"hcl_params")$GO.tree
                    ),sep=": "
                ),
                collapse="\n                              "
            ),
            "\n                              number of clusters: ",
            length(
                unique(
                    unlist(slot(slot(full_object,"enrich_GOs"),"data")[,"GO.cluster",with=FALSE])
                )
            ),
            "\n                              clusters min size: ",
            round(
                min(
                    table(slot(slot(full_object,"enrich_GOs"),"data")[,"GO.cluster",with=FALSE]),
                    na.rm=TRUE
                ),
                digits=0
            ),
            "\n                              clusters mean size: ",
            round(
                mean(
                    table(slot(slot(full_object,"enrich_GOs"),"data")[,"GO.cluster",with=FALSE]),
                    na.rm=TRUE
                ),
                digits=0
            ),
            "\n                              clusters max size: ",
            round(
                max(
                    table(slot(slot(full_object,"enrich_GOs"),"data")[,"GO.cluster",with=FALSE]),
                    na.rm=TRUE
                ),
                digits=0
            ),
            "\n                   - sample.tree: ",
            paste(
                paste(
                    names(
                        unlist(
                            slot(full_object,"hcl_params")$samples.tree
                        )
                    ),
                    unlist(
                        slot(full_object,"hcl_params")$samples.tree
                    ),
                    sep=": "
                ),
                collapse="\n                                 "
            ),
            if(is.null(slot(full_object,"hcl_params")$samples.tree)){"FALSE"},
            "\n          * GOclusters: ",
            !is.null(
                slot(full_object,"heatmap")$GOclusters
            ),
            if(!is.null(slot(full_object,"heatmap")$GOclusters)){
                paste(
                    "\n                       - tree:\n                             ",
                    paste(
                        paste(
                            names(
                                unlist(
                                    slot(full_object,"hcl_params")$GO.clusters
                                )
                            ),
                            unlist(
                                slot(full_object,"hcl_params")$GO.clusters
                            ),
                            sep=": "
                        ),
                        collapse="\n                              "
                    ),
                    collapse=""
                )
            },
            sep=""
        )
    }
)

#' @title myGOs dataset
#' @description  an example of object returned by \code{\link{build_GO_SS}} method
#' from mouse functional analysis of mouse mammary gland RNA-Seq (2_mouse_bioconductor vignette)
#' @docType data
#' @usage data(myGOs,package="ViSEAGO")
#' @format An object of class \code{\link{GO_SS-class}}.
"myGOs"

## ----setup,include=FALSE------------------------------------------------------
# load
library(ViSEAGO)

# knitr document options
knitr::opts_chunk$set(
    eval=FALSE,fig.path='./data/output/',echo=TRUE,fig.pos = 'H',
    fig.width=8,message=FALSE,comment=NA,warning=FALSE
)

## ----vignette_data_used-------------------------------------------------------
# # load vignette data
# data(
#     myGOs,
#     package="ViSEAGO"
# )

## ----SS_build,eval=FALSE------------------------------------------------------
# # compute Semantic Similarity (SS)
# myGOs<-ViSEAGO::compute_SS_distances(
#     myGOs,
#     distance=c("Resnik","Lin","Rel","Jiang","Wang")
# )

## ----SS_terms_Resnik-wardD2---------------------------------------------------
# # GO terms heatmap
# Resnik_clusters_wardD2<-ViSEAGO::GOterms_heatmap(
#     myGOs,
#     showIC=TRUE,
#     showGOlabels=TRUE,
#     GO.tree=list(
#         tree=list(
#             distance="Resnik",
#             aggreg.method="ward.D2"
#         ),
#         cut=list(
#             dynamic=list(
#                 deepSplit=2,
#                 minClusterSize =2
#             )
#         )
#     ),
#     samples.tree=NULL
# )

## ----SS_Lin-wardD2------------------------------------------------------------
# # GO terms heatmap
# Lin_clusters_wardD2<-ViSEAGO::GOterms_heatmap(
#     myGOs,
#     showIC=TRUE,
#     showGOlabels=TRUE,
#     GO.tree=list(
#         tree=list(
#             distance="Lin",
#             aggreg.method="ward.D2"
#         ),
#         cut=list(
#             dynamic=list(
#                 deepSplit=2,
#                 minClusterSize =2
#             )
#         )
#     ),
#     samples.tree=NULL
# )

## ----SS_ Rel-wardD2-----------------------------------------------------------
# # GO terms heatmap
# Rel_clusters_wardD2<-ViSEAGO::GOterms_heatmap(
#     myGOs,
#     showIC=TRUE,
#     showGOlabels=TRUE,
#     GO.tree=list(
#         tree=list(
#             distance="Rel",
#             aggreg.method="ward.D2"
#         ),
#         cut=list(
#             dynamic=list(
#                 deepSplit=2,
#                 minClusterSize =2
#             )
#         )
#     ),
#     samples.tree=NULL
# )

## ----SS_Jiang-wardD2----------------------------------------------------------
# # GO terms heatmap
# Jiang_clusters_wardD2<-ViSEAGO::GOterms_heatmap(
#     myGOs,
#     showIC=TRUE,
#     showGOlabels=TRUE,
#     GO.tree=list(
#         tree=list(
#             distance="Jiang",
#             aggreg.method="ward.D2"
#         ),
#         cut=list(
#             dynamic=list(
#                 deepSplit=2,
#                 minClusterSize =2
#             )
#         )
#     ),
#     samples.tree=NULL
# )

## ----SS_Wang-wardD2-----------------------------------------------------------
# # GO terms heatmap
# Wang_clusters_wardD2<-ViSEAGO::GOterms_heatmap(
#     myGOs,
#     showIC=TRUE,
#     showGOlabels=TRUE,
#     GO.tree=list(
#         tree=list(
#             distance="Wang",
#             aggreg.method="ward.D2"
#         ),
#         cut=list(
#             dynamic=list(
#                 deepSplit=2,
#                 minClusterSize =2
#             )
#         )
#     ),
#     samples.tree=NULL
# )

## ----parameters_dend_correlation----------------------------------------------
# # build the list of trees
# dend<- dendextend::dendlist(
#     "Resnik"=slot(Resnik_clusters_wardD2,"dendrograms")$GO,
#     "Lin"=slot(Lin_clusters_wardD2,"dendrograms")$GO,
#     "Rel"=slot(Rel_clusters_wardD2,"dendrograms")$GO,
#     "Jiang"=slot(Jiang_clusters_wardD2,"dendrograms")$GO,
#     "Wang"=slot(Wang_clusters_wardD2,"dendrograms")$GO
# )
# 
# # build the trees matrix correlation
# dend_cor<-dendextend::cor.dendlist(dend)

## ----parameters_dend_correlation_print----------------------------------------
# # corrplot
# corrplot::corrplot(
#     dend_cor,
#     "pie",
#     "lower",
#     is.corr=FALSEALSE,
#     cl.lim=c(0,1)
# )

## ----parameters_dend_comparison,fig.cap="dendrograms comparison"--------------
# # dendrogram list
# dl<-dendextend::dendlist(
#     slot(Resnik_clusters_wardD2,"dendrograms")$GO,
#     slot(Wang_clusters_wardD2,"dendrograms")$GO
# )
# 
# # untangle the trees (efficient but very highly time consuming)
# tangle<-dendextend::untangle(
#     dl,
#     "step2side"
# )
# 
# # display the entanglement
# dendextend::entanglement(tangle) # 0.08362968
# 
# # display the tanglegram
# dendextend::tanglegram(
#     tangle,
#     margin_inner=5,
#     edge.lwd=1,
#     lwd = 1,
#     lab.cex=0.8,
#     columns_width = c(5,2,5),
#     common_subtrees_color_lines=FALSE
# )

## ----parameters_clusters_correlation------------------------------------------
# # clusters to compare
# clusters=list(
#     Resnik="Resnik_clusters_wardD2",
#     Lin="Lin_clusters_wardD2",
#     Rel="Rel_clusters_wardD2",
#     Jiang="Jiang_clusters_wardD2",
#     Wang="Wang_clusters_wardD2"
# )
# 
# # global dendrogram partition correlation
# clust_cor<-ViSEAGO::clusters_cor(
#     clusters,
#     method="adjusted.rand"
# )

## ----parameters_clusters_correlation_print------------------------------------
# # global dendrogram partition correlation
# corrplot::corrplot(
#     clust_cor,
#     "pie",
#     "lower",
#     is.corr=FALSEALSE,
#     cl.lim=c(0,1)
# )

## ----parameters_clusters_comparison,fig.height=8------------------------------
# # clusters content comparisons
# ViSEAGO::compare_clusters(clusters)

#' @title gene2GO class object definition.
#' @description This class is invoked by \code{\link{annotate}} method in order to store GO annotations for each category (MF, BP, CC).
#' @family GO_terms
#' @slot db database source in \code{character}.
#' @slot stamp date of stamp  in \code{character}.
#' @slot organism target species GO annotation  in \code{character}.
#' @slot MF a \code{list} containing  GO terms for Molecular Function (MF) category for each gene element.
#' @slot BP a \code{list} containing  GO terms for Biological Process (BP) category for each gene element.
#' @slot CC a \code{list} containing  GO terms for Cellular Component (CC) category for each gene element.
setClass(
    "gene2GO",
    slots=c(
        db="character",
        stamp="character",
        organism="character",
        MF="list",
        BP="list",
        CC="list"
    )
)

#' @aliases gene2GO
setMethod(
    "show",
    signature="gene2GO",
    function(object){

        # cat some text
        cat("- object class: gene2GO",
            "\n- database: ",slot(object,"db"),
            "\n- stamp/version: ",slot(object,"stamp"),
            "\n- organism id: ",slot(object,"organism"),
            "\n\nGO annotations:",
            "\n- Molecular Function (MF): ",length(slot(object,"MF"))," annotated genes with ",
            sum(lengths(slot(object,"MF")))," terms (",length(unique(unlist(slot(object,"MF")))),
            " unique terms)",
            "\n- Biological Process (BP): ",length(slot(object,"BP"))," annotated genes with ",
            sum(lengths(slot(object,"BP")))," terms (",length(unique(unlist(slot(object,"BP")))),
            " unique terms)",
            "\n- Cellular Component (CC): ",length(slot(object,"CC"))," annotated genes with ",
            sum(lengths(slot(object,"CC")))," terms (",length(unique(unlist(slot(object,"CC")))),
            " unique terms)",
            sep=""
        )
    }
)

#' @title Store available organisms GO annotations at EntrezGene.
#' @description  Store the available species and current GO annotations from the
#' \href{https://ftp.ncbi.nih.gov/gene/DATA/gene2go.gz}{gene2go.gz} nfile avalable at NCBI \href{https://ftp.ncbi.nih.gov/gene/DATA}{EntrezGene ftp}.
#' @importFrom data.table data.table fread rbindlist :=
#' @importFrom R.utils gunzip
#' @importFrom utils download.file
#' @family genomic_ressource
#' @details This function downloads the \href{https://ftp.ncbi.nih.gov/gene/DATA/gene2go.gz}{gene2go.gz} file from
#' \href{https://ftp.ncbi.nih.gov/gene/DATA}{EntrezGene ftp} which contains available organisms (taxid) with the corresponding GO annotations.
#' @return a  \code{\link{genomic_ressource-class}} object required by \code{\link{annotate}}.
#' @references
#' Matt Dowle and Arun Srinivasan (2017). data.table: Extension of `data.frame`. R package version 1.10.4. https://CRAN.R-project.org/package=data.table.
#'
#' Eric Sayers (2013). Entrez Programming Utilities Help.
#'
#' #' Henrik Bengtsson (2016). R.utils: Various Programming Utilities. R package version 2.5.0. https://CRAN.R-project.org/package=R.utils.
#'
#' Maglott, D, Ostell, J, Pruitt, KD, Tatusova, T (2011). Entrez Gene: gene-centered information at NCBI. Nucleic Acids Res., 39, Database issue:D52-7.
#' @include genomic_ressource.R
#' @examples
#' \dontrun{
#' # Download EntrezGene available organisms GO annotations
#' EntrezGene<-ViSEAGO::EntrezGene2GO()
#' }
#' @export
EntrezGene2GO=function(){

    ## download file

    # temp file
    temp<-paste(
      tempfile(),
      "gz",
      sep="."
    )

    # time out option
    options(timeout = max(300, getOption("timeout")))

    # import Gene to Gene Ontology from NCBI Gene database
    download.file(
      "ftp://ftp.ncbi.nih.gov/gene/DATA/gene2go.gz",
      quiet=TRUE,
      destfile =temp,
      method = "libcurl"
    )

    # uncompress
    gunzip(temp)

    # read the file (linux and windows)
    gene2go=fread(
      sub("\\.gz","",temp),
      verbose=FALSE,
      showProgress=FALSE
    )

    # select columns and rename
    gene2go<-unique(
      gene2go[,c(seq_len(4),8),with=FALSE]
    )
    colnames(gene2go)<-c("taxid","gene_id","GOID","evidence","category")

    # convert columns in character
    gene2go[,
      `:=`(
        taxid=as.character(gene2go$taxid),
        gene_id=as.character(gene2go$gene_id)
      )
    ]

    ## taxonomy

    # extract taxonomique informations
    taxon<-taxonomy(gene2go$taxid)

    # return data genomic_ressource class object
    new(
        "genomic_ressource",
        db="EntrezGene",
        stamp=as.character(Sys.time()),
        data=gene2go,
        organisms=taxon
    )
}

#' @title Check available organisms databases at Uniprot.
#' @description Check the \href{http://www.ebi.ac.uk/GOA}{Uniprot-GOA} available organisms.
#' @importFrom data.table data.table fread
#' @family genomic_ressource
#' @details This function downloads the current_release_numbers file (ftp://ftp.ebi.ac.uk/pub/databases/GO/goa/current_release_numbers.txt) from
#' \href{http://www.ebi.ac.uk/GOA}{Uniprot-GOA} which contains available organisms.
#' @references
#' Matt Dowle and Arun Srinivasan (2017). data.table: Extension of `data.frame`. R package version 1.10.4. https://CRAN.R-project.org/package=data.table.
#'
#' Huntley, RP, Sawford, T, Mutowo-Meullenet, P, Shypitsyna, A, Bonilla, C, Martin, MJ, O'Donovan, C (2015).
#' The GOA database: gene Ontology annotation updates for 2015. Nucleic Acids Res., 43, Database issue:D1057-63.
#' @return a  \code{\link{genomic_ressource-class}} object required by \code{\link{annotate}}.
#' @include genomic_ressource.R
#' @examples
#' \dontrun{
#' # List Uniprot-GOA available organisms
#' Uniprot<-ViSEAGO::Uniprot2GO()
#' }
#' @export
Uniprot2GO=function(){

    # temp file
    tmp<-tempfile()
    
    # temp file
    download.file(
        "ftp://ftp.ebi.ac.uk/pub/databases/GO/goa/current_release_numbers.txt",
        quiet = TRUE,
        method=if(.Platform$OS.type=="unix"){"wget"}else{"auto"},
        destfile=tmp
    )

    # load current release
    Data<-fread(
        tmp,
        verbose=FALSE,
        showProgress=FALSE
    )

    # return data
    new(
        "genomic_ressource",
        db="Uniprot-GOA",
        stamp=as.character(Sys.time()),
        data=data.table(),
        organisms=Data
    )
}

#' @title fgsea class object definition.
#' @description This class is invoked by \code{\link{runfgsea}} method in order to store results.
#' @slot description a \code{character} string with database source, date of stamp, and target species GO annotation.
#' @slot method fgsea method used.
#' @slot params a \code{list} containing used input parameters for perform \code{\link[fgsea]{fgseaSimple}} or \code{\link[fgsea]{fgseaMultilevel}}.
#' @slot input a \code{list} containing input values.
#' @slot data a \code{list} containing \code{data.table} fgsea procedure output.
setClass(
    "fgsea",
    slots=c(
        description="character",
        ontology="character",
        method="character",
        params="list",
        input="list",
        data="list"
    )
)

#' @aliases fgsea
setMethod(
    "show",
    signature="fgsea",
    function(object){

        # extract database informations
        db=unlist(
            strsplit(
                slot(object,"description"),
                " "
            )
        )

        # get parameters informations
        params<-paste(
            "\n    ",
            names(
                slot(object,"params")
            ),
            " : ",
            unlist(
                as.character(slot(object,"params"))
            ),
            sep=""
        )
        
        # get fgsea results
        data<-slot(object,"data")[[1]]

        # cat some text
        cat("- object class: fgsea",
            "\n- database: ",db[1],
            "\n- stamp/version: ",db[3],
            "\n- organism id: ",db[2],
            "\n- ontology: ",slot(object,"ontology"),
            "\n- method: ",slot(object,"method"),
            "\n- fgseaMultilevel parameters: ",params,
            "\n- fgseaMultilevel results:",
            "\n    ",nrow(data), " tested GO terms with enrichment pvalue from ",min(data$pval)," to ",max(data$pval),
            sep=""
        )
    }
)

#' @title build all sets combinations intersections
#' @description  build all sets combinations intersections
#' @family enrich_GO_terms
#' @family visualization
#' @importFrom utils combn
#' @details This internal function use build all sets combinations intersections needed for \code{\link{upset}}.
#' @return a \code{\link[base]{list}}.
#' @examples
#' \dontrun{
#' # build all intersections combinations
#' ViSEAGO::overLapper(setlist)
#' }
#' @keywords internal
#' @export
overLapper<-function(setlist){

    # Create intersect matrix (removes duplicates!)
    complexity=seq_along(setlist)
    setunion <- sort(unique(unlist(setlist)))
    setmatrix <- vapply(names(setlist), function(x){
        setunion %in% unique(setlist[[x]])},
        rep(TRUE,length(setunion))
    )
    rownames(setmatrix) <- setunion
    storage.mode(setmatrix) <- "numeric"

    # Create all possible sample combinations within requested complexity levels
    labels <- names(setlist)
    allcombl <- lapply(complexity, function(x){
        combn(labels, m=x, simplify=FALSE)
    })
    allcombl <- unlist(allcombl, recursive=FALSE)
    complevels <- vapply(allcombl, length,0)

    # vennSets function
    vennSets <- function(setmatrix=setmatrix, allcombl=allcombl, index=1) {
        mycol1 <-which(colnames(setmatrix) %in% allcombl[[index]])
        mycol2 <-which(!colnames(setmatrix) %in% allcombl[[index]])
        cond1 <-rowSums(setmatrix[, rep(mycol1, 2)]) == 2 * length(mycol1)
        cond2 <-rowSums(setmatrix[, rep(mycol2, 2)]) == 0
        return(setunion[cond1 & cond2])
    }

    # apply vennSets to all conbinations
    vennOLlist <- lapply(seq(along=allcombl), function(x){
        vennSets(setmatrix=setmatrix, allcombl=allcombl, index=x)
    })
    names(vennOLlist) <- vapply(allcombl, paste, collapse="-","")

    # returnvennOLlist
    return(vennOLlist)
}

#' @title Multi Dimensional Scale (MDS) plot
#' @description Generate a Multi Dimensional Scale (MDS) plot from distance objects.
#' @import data.table
#' @importFrom dendextend get_leaves_attr
#' @importFrom plotly plot_ly add_markers layout export
#' @importFrom stats cmdscale
#' @family GO_terms GO_clusters semantic_similarity visualization
#' @param object a \code{\link{GO_SS-class}} or \code{\link{GO_clusters-class}} objects from distances computed with \code{\link{compute_SS_distances}}.
#' @param type could be "GOterms" to display GOterms MDSplot, or "GOclusters" to display GOclusters MDSplot.
#' @param file static image output file name (default to NULL).
#' @details This method build and display the javascript MDSplot (if \code{file}=NULL) from \code{\link{GO_SS-class}} or \code{\link{GO_clusters-class}}
#' objects.\cr
#' A static png image could be printed by setting \code{file} argument.
#' @return a MDS plot.
#' @include GO_SS.R GO_clusters.R
#' @examples
#' # load data example
#' utils::data(
#'  myGOs,
#'  package="ViSEAGO"
#' )
#' \dontrun{
#' # compute GO terms Semantic Similarity distances
#' myGOs<-ViSEAGO::compute_SS_distances(
#'     myGOs,
#'     distance="Wang"
#' )
#'
#' # build MDS plot for a GO_SS-class distance object
#' ViSEAGO::MDSplot(myGOs,"GOterms")
#'
#' # GOtermsHeatmap with default parameters
#' Wang_clusters_wardD2<-ViSEAGO::GOterms_heatmap(
#'     myGOs,
#'     showIC=TRUE,
#'     showGOlabels=TRUE,
#'     GO.tree=list(
#'         tree=list(
#'             distance="Wang",
#'             aggreg.method="ward.D2",
#'             rotate=NULL
#'         ),
#'         cut=list(
#'             dynamic=list(
#'                 pamStage=TRUE,
#'                 pamRespectsDendro=TRUE,
#'                 deepSplit=2,
#'                 minClusterSize =2
#'             )
#'         )
#'     ),
#'     samples.tree=NULL
#' )
#'
#' # build MDS plot for a GO_clusters-class distance object, highlighting GO terms clusters.
#' ViSEAGO::MDSplot(
#'     Wang_clusters_wardD2,
#'     "GOterms"
#' )
#'
#' # compute clusters of GO terms Semantic Similarity distances
#' Wang_clusters_wardD2<-ViSEAGO::compute_SS_distances(
#'     Wang_clusters_wardD2,
#'     distance="BMA"
#' )
#'
#' # GOclusters heatmap
#' Wang_clusters_wardD2<-ViSEAGO::GOclusters_heatmap(
#'     Wang_clusters_wardD2,
#'     tree=list(
#'         distance="BMA",
#'         aggreg.method="ward.D2",
#'         rotate=NULL
#'     )
#' )
#'
#' # build MDS plot for a GO_clusters-class distance object, highlighting GO groups clusters.
#' ViSEAGO::MDSplot(
#'     Wang_clusters_wardD2,
#'     "GOclusters"
#' )
#' }
#' @exportMethod MDSplot
#' @name MDSplot
#' @rdname MDSplot-methods
#' @exportMethod MDSplot
setGeneric(
    name="MDSplot",
    def=function(object,type="GOterms",file=NULL){
        standardGeneric("MDSplot")
    }
)

#' @rdname MDSplot-methods
#' @aliases MDSplot
setMethod(
    "MDSplot",
    signature="ANY",
    definition=function(object,type,file){

        # check class
        if(!is(object,"GO_SS") & !is(object,"GO_clusters")){
            stop("object must come from ViSEAGO::compute_SS_distances() or ViSEAGO::GOterms_heatmap()")
        }

        # check class
        if(is(object,"GO_SS") & type=="GOclusters"){
            stop("show_clusters is only available for GO_clusters class after clusters SS distance calculation with ViSEAGO::compute_SS_distances()")
        }

        # check type argument
        type=match.arg(type,c("GOterms","GOclusters"))

        # for SS_dist from object
        if(type=="GOterms"){

            # import SS_dist from object
            d=slot(object,"terms_dist")

            # if empty
            if(length(d)==0){
                stop("Please use GO_terms class object with computed SS distances using ViSEAGO::compute_SS_distance()")
            }

            # measures
            measures=names(d)

            ## MDS

            # run MDS
            res.mds=lapply(measures,function(x){

                # run MDS
                res.mds <-cmdscale(d[[x]], eig =T, k = 2)

                # extract point values
                res.mds<-res.mds$points

                # convert to data.table
                data.table(
                    GO.ID=attr(res.mds,"dimnames")[[1]],
                    Dim.1=res.mds[,1],
                    Dim.2=res.mds[,2],
                    measure=x
                )
            })

            # bind columns
            res.mds=rbindlist(res.mds)

            # for GO_SS
            if(is(object,"GO_SS")){

                # GO names
                res.mds=data.table(
                    res.mds,
                    term=slot(slot(object,"enrich_GOs"),"data")$term
                )

                # add levels to measures
                res.mds$measure<-factor(
                    res.mds$measure,
                    levels=unique(res.mds$measure)
                )

            }else{

                # add clusters and GO names
                res.mds=merge(
                    res.mds,
                    slot(slot(object,"enrich_GOs"),"data")[,c("GO.cluster","GO.ID","term"),with=FALSE],
                    by="GO.ID",
                    sort=FALSE
                )

                # ordering by cluster
                setorder(res.mds,"GO.cluster")

                # add levels to measures
                res.mds$`GO.cluster`<-factor(
                    res.mds$`GO.cluster`,
                    levels=unique(res.mds$`GO.cluster`)
                )
            }

        }else{

            # import SS_dist from object
            d=slot(object,"clusters_dist")

            # if empty
            if(length(d)==0){
                stop("Please use GO_clusters class object with computed clusters distances using ViSEAGO::compute_SS_distance()")
            }

            # measures
            measures=names(d)

            ## MDS

            # run MDS
            res.mds=lapply(measures,function(x){

                # run MDS
                res.mds <-cmdscale(d[[x]], eig = T, k = 2)

                # extract point values
                res.mds<-res.mds$points

                # convert to data.table
                data.table(
                    GO.cluster=attr(res.mds,"dimnames")[[1]],
                    Dim.1=res.mds[,1],
                    Dim.2=res.mds[,2],
                    measure=x
                )
            })

            # bind columns
            res.mds=rbindlist(res.mds)

            # custom text
            res.mds[,
                `:=`(
                    "text"=res.mds$`GO.cluster`,
                    GO.cluster=gsub("_.+$","",res.mds$`GO.cluster`)
                )
            ]

            # add levels to measures
            res.mds$measure<-factor(
                res.mds$measure,
                levels=unique(res.mds$measure)
            )

            # add levels to GO.cluster 
            res.mds$`GO.cluster`<-factor(
                res.mds$`GO.cluster`,
                levels=unique(res.mds$`GO.cluster`)
            )

            # add GO.ID for GO
            res.mds[,"text":=paste("GO.cluster:",text)]

            # add GO.ID for GO
            res.mds[,"text":=gsub("_GO","<br>GO.ID: GO",text)]

            # add GO.name for term definition
            res.mds[,"text":=gsub("_"," <br>GO.name: ",text)]
        }

        ## plot

        # mds coordinates
        values=unlist(res.mds$Dim.1,res.mds$Dim.2)

        # init graph
        p<-plot_ly()

        # for GO_SS
        if(is(object,"GO_SS")){

            # add trace to plot by measure
            for(x in seq_along(measures)){

                # default visualization
                if(x==1){visible=TRUE}else{visible=FALSE}

                # create trace
                p<-add_markers(
                    p,
                    data=res.mds[measures[x],on="measure"],
                    x=~Dim.1,
                    y=~Dim.2,
                    name=measures[x],
                    text=~paste('GO.ID:',`GO.ID`,'<br>GO.name:',term),
                    showlegend=FALSE,
                    marker =list(
                        size =20,
                        opacity = 0.4,
                        color="royalblue"
                    ),
                    visible=visible
                )
            }

            # add custom layout with dropdown menu
            p<-layout(
                p,

                # add title
                title="MultiDimensional Scaling plot",

                # increase font size
                font=list(size=14),

                # add axis legends
                xaxis=list(title="Dimension 1"),
                yaxis=list(title="Dimension 2"),

                # add margin
                margin=list(t=100),

                # add scrolling menu for availables measures
                updatemenus = list(

                    # measure dropdown menu
                    list(

                        # button position
                        x = 0.1,
                        y = 1.1,

                        buttons = lapply(seq_along(measures),function(x){

                            # init visibility to all FASLE
                            values=rep(FALSE,length(measures))

                            # turn to true for each measures
                            values[x]<-TRUE

                            # output button
                            list(
                                method="restyle",
                                args=list("visible",as.list(values)),
                                label=measures[x]
                            )
                        })
                    )
                )
            )
        }

        # for GO_SS
        if(is(object,"GO_clusters")){

            # color labels of cutting tree
            colors= unique(
                get_leaves_attr(
                    slot(object,"dendrograms")$GO,
                    "edgePar"
                )
            )

            # extract the GO. cluster column
            count<-slot(slot( object,"enrich_GOs"),"data")[,"GO.cluster",with=FALSE]

            # count the number of terms by clusters
            count<-count[,list("nb"=.N),by="GO.cluster"]

            # add count to res.mds
            res.mds=merge(
                res.mds,
                count,
                by="GO.cluster",
                sort=FALSE
            )

            # ordering clusters
            res.mds[,
                GO.cluster:=factor(GO.cluster,levels=sort(unique(as.numeric(GO.cluster))))
            ]

            # for terms
            if(type=="GOterms"){

                # create trace
                p<-add_markers(
                    p,
                    data=res.mds,
                    x=~Dim.1,
                    y=~Dim.2,
                    color=~GO.cluster,
                    text = ~paste("cluster:",GO.cluster,"<br>GO.ID:",`GO.ID`,"<br>GO.name:",term),
                    showlegend=TRUE,
                    colors=colors,
                    marker =list(
                        size =20,
                        opacity = 0.4
                    )
                )

                # add custom layout with dropdown menu
                p<-layout(
                    p,

                    # add title
                    title=paste(measures,"distance MultiDimensional Scaling plot"),

                    # increase font size
                    font=list(size=14),

                    # add axis legends
                    xaxis=list(title="Dimension 1"),
                    yaxis=list(title="Dimension 2"),

                    # add margin
                    margin=list(t=100)
                )
            }else{

                # add count to text
                res.mds[,"text":=paste(sub("<br>.+$","",text),"<br>GO.count:",res.mds$nb,sub("^.+<br>GO.ID","<br>GO.ID",text))]

                # add trace to plot by measure
                for(x in seq_along(measures)){

                # default visualization
                if(x==1){visible=TRUE}else{visible=FALSE}

                # create trace
                p<-add_markers(
                    p,
                    data=res.mds[measures[x],on="measure"],
                    x=~Dim.1,
                    y=~Dim.2,
                    name=measures[x],
                    text=~text,
                    showlegend=FALSE,
                    sizes=c(20,50),
                    size=~nb,
                    marker =list(
                        sizemode = 'diameter',
                        opacity = 0.4,
                        line=list(color=colors),
                        color=colors
                    ),
                    visible=visible
                )
            }

        # add custom layout with dropdown menu
        p<-plotly::layout(p,

          # add title
          title="MultiDimensional Scaling plot",

          # increase font size
          font=list(size=14),

          # add axis legends
          xaxis=list(title="Dimension 1"),
          yaxis=list(title="Dimension 2"),

          # add margin
          margin=list(t=100),

          # add scrolling menu for availables measures
          updatemenus = list(

            # measure dropdown menu
            list(

              # button position
              x = 0.1,
              y = 1.1,

              buttons = lapply(seq_along(measures),function(x){

                # init visibility to all FASLE
                values=rep(FALSE,length(measures))

                # turn to true for each measures
                values[x]<-TRUE

                # output button
                list(method ="restyle",
                args = list("visible",as.list(values)),
                label=measures[x])
              })
            )
          )
        )
      }
        }

        # return or print
        if(is.null(file)){

            # return the plot
            p

        }else{

            # print heatmap
            orca(p,file)
        }
    }
)

## ----setup,include=FALSE------------------------------------------------------
# koad ViSEAGO
library(ViSEAGO)

# knitr document options
knitr::opts_chunk$set(
  eval=FALSE,echo=TRUE,fig.pos = 'H',
  fig.width=8,message=FALSE,comment=NA,warning=FALSE
)

## ----ViSEAGO_install----------------------------------------------------------
# # Install ViSEAGO package from Bioconductor
# BiocManager::install("ViSEAGO")

## ----geneList_input_topGO-----------------------------------------------------
# # load genes background
# background<-scan(
#     "background.txt",
#     quiet=TRUE,
#     what=""
# )
# 
# # load gene selection
# selection<-scan(
#     "selection.txt",
#     quiet=TRUE,
#     what=""
# )

## ----geneList_input_fgsea-----------------------------------------------------
# # load gene identifiers column 1) and corresponding statistical value (column 2)
# table<-data.table::fread("table.txt")
# 
# # rank gene identifiers according statistical value
# data.table::setorder(table,value)

## ----databases----------------------------------------------------------------
# # connect to Bioconductor
# Bioconductor<-ViSEAGO::Bioconductor2GO()
# 
# # connect to EntrezGene
# EntrezGene<-ViSEAGO::EntrezGene2GO()
# 
# # connect to Ensembl
# Ensembl<-ViSEAGO::Ensembl2GO()
# 
# # connect to Uniprot-GOA
# Uniprot<-ViSEAGO::Uniprot2GO()
# 
# # connect to Custom file
# Custom<-ViSEAGO::Custom2GO(system.file("extdata/customfile.txt",package = "ViSEAGO"))

## ----organisms----------------------------------------------------------------
# # Display table of available organisms with Bioconductor
# ViSEAGO::available_organisms(Bioconductor)
# 
# # Display table of available organisms with EntrezGene
# ViSEAGO::available_organisms(EntrezGene)
# 
# # Display table of available organisms with Ensembl
# ViSEAGO::available_organisms(Ensembl)
# 
# # Display table of available organisms with Uniprot
# ViSEAGO::available_organisms(Uniprot)
# 
# # Display table of available organisms with Custom
# ViSEAGO::available_organisms(Custom)

## ----annotate-----------------------------------------------------------------
# # load GO annotations from Bioconductor
# myGENE2GO<-ViSEAGO::annotate(
#     "bioconductor_id",
#     Bioconductor
# )
# 
# # load GO annotations from EntrezGene
# myGENE2GO<-ViSEAGO::annotate(
#     "EntrezGene_id",
#     EntrezGene
# )
# 
# # load GO annotations from EntrezGene
# # with the add of GO annotations from orthologs genes (see above)
# myGENE2GO<-ViSEAGO::annotate(
#     "EntrezGene_id",
#     EntrezGene,
#     ortholog = TRUE
# )
# 
# # load GO annotations from Ensembl
# myGENE2GO<-ViSEAGO::annotate(
#     "Ensembl_id",
#     Ensembl
# )
# 
# # load GO annotations from Uniprot
# myGENE2GO<-ViSEAGO::annotate(
#     "Uniprot_id",
#     Uniprot
# )
# 
# # load GO annotations from Custom
# myGENE2GO<-ViSEAGO::annotate(
#     "Custom_id",
#     Custom
# )

## ----Enrichment_data----------------------------------------------------------
# # create topGOdata for BP
# BP<-ViSEAGO::create_topGOdata(
#     geneSel=selection,
#     allGenes=background,
#     gene2GO=myGENE2GO,
#     ont="BP",
#     nodeSize=5
# )

## ----Enrichment_data_tests----------------------------------------------------
# # perform TopGO test using clasic algorithm
# classic<-topGO::runTest(
#     BP,
#     algorithm ="classic",
#     statistic = "fisher"
# )

## ----fgsea--------------------------------------------------------------------
# # perform fgseaMultilevel tests
# BP<-ViSEAGO::runfgsea(
#     geneSel=table,
#     ont="BP",
#     gene2GO=myGENE2GO,
#     method ="fgseaMultilevel",
#     params = list(
#         scoreType = "pos",
#          minSize=5
#     )
# )

## ----Enrichment_merge---------------------------------------------------------
# # merge results from topGO
# BP_sResults<-ViSEAGO::merge_enrich_terms(
#     Input=list(
#         condition=c("BP","classic")
#     )
# )
# 
# # merge results from fgsea
# BP_sResults<-ViSEAGO::merge_enrich_terms(
#     Input=list(
#         condition="BP"
#     )
# )

## ----Enrichment_merge_display-------------------------------------------------
# # display the merged table
# ViSEAGO::show_table(BP_sResults)
# 
# # print the merged table in a file
# ViSEAGO::show_table(
#     BP_sResults,
#     "BP_sResults.xls"
# )

## ----Enrichment_merge_count---------------------------------------------------
# # count significant (or not) pvalues by condition
# ViSEAGO::GOcount(BP_sResults)

## ----Enrichment_merge_interactions,fig.height=4-------------------------------
# # display interactions
# ViSEAGO::Upset(
#     BP_sResults,
#     file="OLexport.xls"
# )

## ----SS_build-----------------------------------------------------------------
# # initialyse
# myGOs<-ViSEAGO::build_GO_SS(
#     gene2GO=myGENE2GO,
#     enrich_GO_terms=BP_sResults
# )
# 
# # compute all available Semantic Similarity (SS) measures
# myGOs<-ViSEAGO::compute_SS_distances(
#     myGOs,
#     distance="Wang"
# )

## ----SS_terms_mdsplot,eval=FALSE----------------------------------------------
# # display MDSplot
# ViSEAGO::MDSplot(myGOs)
# 
# # print MDSplot
# ViSEAGO::MDSplot(
#     myGOs,
#     file="mdsplot1.png"
# )

## ----SS_Wang-wardD2-----------------------------------------------------------
# # GOterms heatmap with the default parameters
# Wang_clusters_wardD2<-ViSEAGO::GOterms_heatmap(
#     myGOs,
#     showIC=TRUE,
#     showGOlabels=TRUE,
#     GO.tree=list(
#         tree=list(
#             distance="Wang",
#             aggreg.method="ward.D2"
#         ),
#         cut=list(
#             dynamic=list(
#                 pamStage=TRUE,
#                 pamRespectsDendro=TRUE,
#                 deepSplit=2,
#                 minClusterSize =2
#             )
#         )
#     ),
#     samples.tree=NULL
# )

## ----SS_Wang-wardD2_clusters-heatmap------------------------------------------
# # Display the clusters-heatmap
# ViSEAGO::show_heatmap(
#     Wang_clusters_wardD2,
#     "GOterms"
# )
# 
# # print the clusters-heatmap
# ViSEAGO::show_heatmap(
#     Wang_clusters_wardD2,
#     "GOterms",
#     "cluster_heatmap_Wang_wardD2.png"
# )

## ----SS_Wang-ward.D2_clusters-heatmap_table-----------------------------------
# # Display the clusters-heatmap table
# ViSEAGO::show_table(Wang_clusters_wardD2)
# 
# # Print the clusters-heatmap table
# ViSEAGO::show_table(
#     Wang_clusters_wardD2,
#     "cluster_heatmap_Wang_wardD2.xls"
# )

## ----SS_Wang-ward.D2_mdsplot,eval=FALSE---------------------------------------
# # display colored MDSplot
# ViSEAGO::MDSplot(
#     Wang_clusters_wardD2,
#     "GOterms"
# )
# 
# # print colored MDSplot
# ViSEAGO::MDSplot(
#     Wang_clusters_wardD2,
#     "GOterms",
#     file="mdsplot2.png"
# )

## ----SS_Wang-wardD2_groups----------------------------------------------------
# # calculate semantic similarites between clusters of GO terms
# Wang_clusters_wardD2<-ViSEAGO::compute_SS_distances(
#     Wang_clusters_wardD2,
#     distance=c("max", "avg","rcmax", "BMA")
# )

## ----SS_Wang-ward.D2_groups_mdsplot-------------------------------------------
# # build and highlight in an interactive MDSplot grouped clusters for one distance object
# ViSEAGO::MDSplot(
#     Wang_clusters_wardD2,
#     "GOclusters"
# )
# 
# # build and highlight in MDSplot grouped clusters for one distance object
# ViSEAGO::MDSplot(
#     Wang_clusters_wardD2,
#     "GOclusters",
#     file="mdsplot3.png"
# )

## ----SS_Wang-wardD2_groups_heatmap--------------------------------------------
# # GOclusters heatmap
# Wang_clusters_wardD2<-ViSEAGO::GOclusters_heatmap(
#     Wang_clusters_wardD2,
#     tree=list(
#         distance="BMA",
#         aggreg.method="ward.D2"
#     )
# )

## ----SS_Wang-ward.D2_groups_heatmap_display-----------------------------------
# # sisplay the GOClusters heatmap
# ViSEAGO::show_heatmap(
#     Wang_clusters_wardD2,
#     "GOclusters"
# )
# 
# # print the GOClusters heatmap in a file
# ViSEAGO::show_heatmap(
#     Wang_clusters_wardD2,
#     "GOclusters",
#     "Wang_clusters_wardD2_heatmap_groups.png"
# )

## ----session,eval=TRUE,echo=FALSE---------------------------------------------
version

#' @title Display an interactive or static table.
#' @description This method is used to display or print the table for \code{\link{enrich_GO_terms-class}} or \code{\link{GO_clusters-class}} objects.
#' @importFrom DT datatable
#' @importFrom data.table fwrite
#' @importFrom htmlwidgets JS
#' @family enrich_GO_terms
#' @family GO_clusters
#' @family visualization
#' @param object an \code{\link{enrich_GO_terms-class}} object from \code{\link{merge_enrich_terms}},
#' or \code{\link{GO_clusters-class}} object from \code{\link{GOterms_heatmap}}.
#' @param file table output file name (default to NULL).
#' @details This method displays an interactive table (if file=NULL) from \code{\link{enrich_GO_terms-class}} or \code{\link{GO_clusters-class}} objects.\cr
#' The table could be printed by setting file argument.
#' @return display or print table
#' @references
#' Yihui Xie (2016). DT: A Wrapper of the JavaScript Library 'DataTables'. R package version 0.2. https://CRAN.R-project.org/package=DT
#' @examples
#' # load example object
#' data(
#'     myGOs,
#'     package="ViSEAGO"
#' )
#'
#' # display merge_enrich_terms output
#' ViSEAGO::show_table(myGOs)
#'
#' # print merge_enrich_terms output
#' ViSEAGO::show_table(
#'     myGOs,
#'     "myGOs.txt"
#' )
#'
#' \dontrun{
#' # compute GO terms Semantic Similarity distances
#' myGOs<-ViSEAGO::compute_SS_distances(
#'     distance="Wang"
#' )
#'
#' # GOtermsHeatmap with default parameters
#' Wang_clusters_wardD2<-ViSEAGO::GOterms_heatmap(
#'     myGOs,
#'     showIC=TRUE,
#'     showGOlabels=TRUE,
#'     GO.tree=list(
#'         tree=list(
#'             distance="Wang",
#'             aggreg.method="ward.D2",
#'             rotate=NULL
#'         ),
#'         cut=list(
#'             dynamic=list(
#'                 pamStage=TRUE,
#'                 pamRespectsDendro=TRUE,
#'                 deepSplit=2,
#'                 minClusterSize =2
#'             )
#'         )
#'     ),
#'     samples.tree=NULL
#' )
#'
#' # display table of GO_clusters-class object
#' ViSEAGO::show_table(Wang_clusters_wardD2)
#'
#' # print table of GO_clusters-class object
#' ViSEAGO::show_table(
#'     Wang_clusters_wardD2,
#'     "Wang_clusters_wardD2.txt"
#' )
#' }
#' @name show_table
#' @rdname show_table-methods
#' @exportMethod show_table
setGeneric(
    name="show_table",
    def=function(object,file=NULL) {
        standardGeneric("show_table")
    }
)

#' @rdname show_table-methods
#' @aliases show_table
setMethod(
    "show_table",
    signature="ANY",
    definition=function(object,file){

        # check class
        if(!is(object,"enrich_GO_terms") & !is(object,"GO_SS") & !is(object,"GO_clusters")){
            stop("object must be enrich_GO_terms, GO_SS, or GO_clusters class objects")
        }

        # Extract data
        if(is(object,"enrich_GO_terms")){

            # data
            data<-slot(object,"data")

        }else{

            # data
            data<-slot(slot(object,"enrich_GOs"),"data")
        }

        # if interactive mode
        if(is.null(file)){

            # remove genes columns for interactive mode
             data<-data[,-grep("genes",names(data)),with=FALSE]

            # add link to amigo Gene Ontology
            data$GO.ID<-paste('<a href="http://amigo.geneontology.org/amigo/term/',data$GO.ID,'">',data$GO.ID,'</a>',sep="")

            # remove . in header
            names(data)<-gsub("\\."," ",names(data))

            # convert NA in characters
            data[is.na(data)]<-"NA"

            # create a datatable
            datatable(data,

                # escape the first column
                width=650,

                # escape the first column
                escape=1,

                # filter column
                filter ='top',

                # extensions to use
                extensions =c('Buttons','FixedColumns','Scroller'),

                # DT options to use
                options = list(

                    # DT options to use
                    columnDefs = list(
                        list(

                            # targets columns
                            targets=if(is(object,"enrich_GO_terms")){
                                c(2:3,grep("genes",names(data)))
                            }else{
                                c(4:5,grep("genes",names(data)))
                            },

                            # show the fist characters
                            render=JS(
                                "function(data, type, row, meta) {",
                                "return type === 'display' && data.length > 50 ?",
                                "'<span title=\"' + data + '\">' + data.substr(0, 50) + '...</span>' : data;",
                                "}"
                            )
                        )
                    ),

                    # the dom
                    dom='Bfltipr',

                    # regex with highlight
                    search = list(regex=TRUE,caseInsensitive=FALSE),
                    searchHighlight = TRUE,

                    # x axis scroller
                    scrollX = TRUE,
                    fixedColumns =list(leftColumns = 1),

                    # y scroller
                    deferRender = TRUE,
                    scrollY = 400,
                    scroller = TRUE,

                    # button download
                    buttons=I('colvis')
                )
            )
        }else{

            # write the table
            fwrite(data,file=file,na="NA",sep="\t")
        }
    }
)

#' @title Display the ViSEAGO package diagram.
#' @description This function build the ViSEAGO package diagram available displayed in the vignette.
#' @importFrom DiagrammeR grViz
#' @param x rectangles colors for the five subgraphs with all white by defaults
#' @details This function use \pkg{DiagrammeR} package \code{\link[DiagrammeR]{grViz}} to display the ViSEAGO methods diagram with colored or not subgraphs rectangles.
#' @references
#' Knut Sveidqvist, Mike Bostock, Chris Pettitt, Mike Daines, Andrei Kashcha and Richard Iannone (2017). DiagrammeR: Create Graph Diagrams and Flowcharts Using R.
#' R package version 0.9.0. https://CRAN.R-project.org/package=DiagrammeR.
#' @return an html diagram
#' @examples
#' # print diagram without focus
#' ViSEAGO::pkgdiagram(x=c("white","white","white","white","white"))
#'
#' # print diagram with focus one the gene list
#' ViSEAGO::pkgdiagram(x=c("black","white","white","white","white"))
#'
#' # print diagram with a focus on the annotation step
#' ViSEAGO::pkgdiagram(x=c("white","black","white","white","white"))
#'
#' # print diagram with a focus on the enrichment step
#' ViSEAGO::pkgdiagram(x=c("white","white","black","white","white"))
#'
#' # print diagram with a focus on the Semantic Similarity step
#' ViSEAGO::pkgdiagram(x=c("white","white","white","black","white"))
#'
#' # print diagram with focus on the visualization step
#' ViSEAGO::pkgdiagram(x=c("white","white","white","white","black"))
#' @keywords internal
#' @export
pkgdiagram<-function(x=c("white","white","white","white","white")){

    # print the diagram
    grViz(
        diagram=paste("
            # init the graph
            digraph ViSEAGO_map {

                # a 'graph' statement
                graph [layout = dot,overlap = false, fontsize = 10, splines= true, shape =oval,fixedsize = true,width = 2.5,fontname = Helvetica]

                node [fillcolor=MistyRose2,style=filled]
                subgraph cluster0 {
                    label='List(s) of genes'
                    fontsize = 25
                    fontcolor = MistyRose2
                    labelloc=t
                    color=",x[1],"
                    'Genes of interest \n and background'
                    'preranked gene set with \nthe statistical values used'
                }

                node [fillcolor=orange,style=filled]
                subgraph cluster1 {
                    label='Genomic ressources'
                    fontsize = 25
                    fontcolor = orange
                    labelloc=t
                    color=",x[2],"
                    Bioconductor2GO
                    EntrezGene2GO
                    Ensembl2GO
                    Uniprot2GO
                    Custom2GO
                    available_organisms
                    annotate
                }

                node [fillcolor=lightblue1,style=filled]
                subgraph cluster2 {
                    label='Enrichment tests'
                    fontsize = 25
                    fontcolor =lightblue1
                    labelloc=t
                    color=",x[3],"
                    create_topGOdata
                    RunTest
                    runfgsea
                    merge_enrich_terms
                }

                node [fillcolor=LightCoral,style=filled]
                subgraph cluster3 {
                    label='Visualization'
                    fontsize = 25
                    fontcolor = LightCoral
                    labelloc=t
                    color=",x[5],"
                    show_table
                    show_heatmap
                    GOcount
                    Upset
                    MDSplot
                    GOterms_heatmap
                    GOclusters_heatmap
                    compare_clusters
                }

                node [fillcolor=LimeGreen,style=filled]
                subgraph cluster4 {
                    label='GO Semantic Similarities'
                    fontsize = 25
                    fontcolor = LimeGreen
                    labelloc=t
                    color=",x[4],"
                    build_GO_SS
                    compute_SS_distances
                }

                # 'edge' statements
                'Genes of interest \n and background'-> create_topGOdata
                'preranked gene set with \nthe statistical values used' -> runfgsea
                Bioconductor2GO->annotate
                EntrezGene2GO->annotate
                Ensembl2GO->annotate
                Uniprot2GO->annotate
                Custom2GO->annotate
                annotate->create_topGOdata
                annotate->runfgsea
                create_topGOdata-> {RunTest merge_enrich_terms}
                RunTest->merge_enrich_terms
                runfgsea->merge_enrich_terms
                merge_enrich_terms->{build_GO_SS show_table GOcount Upset}
                build_GO_SS->{compute_SS_distances show_table GOcount Upset}
                compute_SS_distances->{GOcount MDSplot Upset GOterms_heatmap GOclusters_heatmap}
                GOterms_heatmap->{compute_SS_distances show_table show_heatmap compare_clusters}
                GOclusters_heatmap->{show_table show_heatmap MDSplot}
                Bioconductor2GO->available_organisms
                EntrezGene2GO->available_organisms
                Ensembl2GO->available_organisms
                Uniprot2GO->available_organisms
                Custom2GO->available_organisms
            }",
            sep=""
        )
    )
}

#' @title Check available organisms databases at Bioconductor.
#' @description Retrieve the Bioconductor \href{http://bioconductor.org/packages/release/BiocViews.html#___OrgDb}{OrgDb}
#' available organisms databases packages.
#' @importFrom data.table data.table like
#' @importFrom AnnotationForge loadAnnDbPkgIndex
#' @importFrom AnnotationDbi species
#' @importFrom methods new
#' @family genomic_ressource
#' @details
#' This function gives genome wide annotation for available organisms databases packages from
#' \href{http://bioconductor.org/packages/release/BiocViews.html#___OrgDb}{Bioconductor OrgDb}.
#' It uses \code{loadAnnDbPkgIndex} from \pkg{AnnotationForge} package.
#' @references
#' Carlson M and Pages H (2017). AnnotationForge: Code for Building Annotation Database Packages. R package version 1.18.0.
#' @return a  \code{\link{genomic_ressource-class}} object required by \code{\link{annotate}} method.
#' @include genomic_ressource.R
#' @examples
#' # Check Bioconductor OrgDb available organisms
#' Bioconductor<-ViSEAGO::Bioconductor2GO()
#' @export
Bioconductor2GO=function(){

    # Available databases
    Orgs.db<-data.table(loadAnnDbPkgIndex())

    # select organisms lines and only selected columns
    Orgs.db<-Orgs.db[like(Orgs.db$Package,"^org"),c("Package","Version","species"),with=FALSE]

    # return data
    new(
        "genomic_ressource",
        db="Bioconductor",
        stamp=as.character(Sys.time()),
        data=data.table(),
        organisms=Orgs.db
    )
}

## ----setup,include=FALSE------------------------------------------------------
# load ViSEAGO and mouse db package
library(ViSEAGO)

# knitr document options
knitr::opts_chunk$set(
    eval=FALSE,echo=TRUE,fig.pos = 'H',
    fig.width=6,message=FALSE,comment=NA,warning=FALSE
)

## ----vignette_data_used,eval=TRUE---------------------------------------------
# load vignette data
data(
    myGOs,
    package="ViSEAGO"
)

## ----geneList_input,eval=TRUE-------------------------------------------------
# load genes identifiants (GeneID,ENS...) background (expressed genes) 
background<-scan(
    system.file(
        "extdata/data/input",
        "background_L.txt",
        package = "ViSEAGO"
    ),
    quiet=TRUE,
    what=""
)

# load Differentialy Expressed (DE) gene identifiants from lists
PregnantvsLactateDE<-scan(
    system.file(
        "extdata/data/input",
        "pregnantvslactateDE.txt",
        package = "ViSEAGO"
    ),
    quiet=TRUE,
    what=""
)

VirginvsLactateDE<-scan(
    system.file(
        "extdata/data/input",
        "virginvslactateDE.txt",
        package = "ViSEAGO"
    ),
    quiet=TRUE,
    what=""
)

VirginvsPregnantDE<-scan(
    system.file(
        "extdata/data/input",
        "virginvspregnantDE.txt",
        package = "ViSEAGO"
    ),
    quiet=TRUE,
    what=""
)

## ----geneList_input-head,echo=FALSE-------------------------------------------
# # show the ten first lines of genes_DE (same as genes_ref)
# head(PregnantvsLactateDE)

## ----Genomic-ressources-------------------------------------------------------
# # connect to Bioconductor
# Bioconductor<-ViSEAGO::Bioconductor2GO()
# 
# # load GO annotations from Bioconductor
# myGENE2GO<-ViSEAGO::annotate(
#     "org.Mm.eg.db",
#     Bioconductor
# )

## ----Genomic-ressources_show--------------------------------------------------
# # display summary
# myGENE2GO

## ----Genomic-ressources_display,echo=FALSE,eval=TRUE--------------------------
cat(
"- object class: gene2GO
- database: Bioconductor
- stamp/version: 2019-Jul10
- organism id: org.Mm.eg.db

GO annotations:
- Molecular Function (MF): 22707 annotated genes with 91986 terms (4121 unique terms)
- Biological Process (BP): 23210 annotated genes with 164825 terms (12224 unique terms)
- Cellular Component (CC): 23436 annotated genes with 107852 terms (1723 unique terms)"
)

## ----Enrichment_data----------------------------------------------------------
# # create topGOdata for BP for each list of DE genes
# BP_PregnantvsLactate<-ViSEAGO::create_topGOdata(
#     geneSel=PregnantvsLactateDE,
#     allGenes=background,
#     gene2GO=myGENE2GO,
#     ont="BP",
#     nodeSize=5
# )
# 
# BP_VirginvsLactate<-ViSEAGO::create_topGOdata(
#     geneSel=VirginvsLactateDE,
#     allGenes=background,
#     gene2GO=myGENE2GO,
#     ont="BP",
#     nodeSize=5
# )
# 
# BP_VirginvsPregnant<-ViSEAGO::create_topGOdata(
#     geneSel=VirginvsPregnantDE,
#     allGenes=background,
#     gene2GO=myGENE2GO,
#     ont="BP",
#     nodeSize=5
# )

## ----Enrichment_data_tests----------------------------------------------------
# # perform topGO tests
# elim_BP_PregnantvsLactate<-topGO::runTest(
#     BP_PregnantvsLactate,
#     algorithm ="elim",
#     statistic = "fisher",
#     cutOff=0.01
# )
# 
# elim_BP_VirginvsLactate<-topGO::runTest(
#     BP_VirginvsLactate,
#     algorithm ="elim",
#     statistic = "fisher",
#     cutOff=0.01
# )
# 
# elim_BP_VirginvsPregnant<-topGO::runTest(
#     BP_VirginvsPregnant,
#     algorithm ="elim",
#     statistic = "fisher",
#     cutOff=0.01
# )

## ----Enrichment_merge---------------------------------------------------------
# # merge topGO results
# BP_sResults<-ViSEAGO::merge_enrich_terms(
#     cutoff=0.01,
#     Input=list(
#         PregnantvsLactate=c(
#             "BP_PregnantvsLactate",
#             "elim_BP_PregnantvsLactate"
#         ),
#         VirginvsLactate=c(
#             "BP_VirginvsLactate",
#             "elim_BP_VirginvsLactate"
#         ),
#         VirginvsPregnant=c(
#             "BP_VirginvsPregnant",
#             "elim_BP_VirginvsPregnant"
#         )
#     )
# )

## ----Enrichment_merge_show----------------------------------------------------
# # display a summary
# BP_sResults

## ----Enrichment_merge_display,echo=FALSE,eval=TRUE----------------------------
cat(
"- object class: enrich_GO_terms
- ontology: BP
- method: topGO
- summary:PregnantvsLactate
      BP_PregnantvsLactate 
        description: Bioconductor org.Mm.eg.db 2019-Jul10
        available_genes: 15804
        available_genes_significant: 7699
        feasible_genes: 14091
        feasible_genes_significant: 7044
        genes_nodeSize: 5
        nodes_number: 8463
        edges_number: 19543
      elim_BP_PregnantvsLactate 
        description: Bioconductor org.Mm.eg.db 2019-Jul10 
        test_name: fisher p<0.01
        algorithm_name: elim
        GO_scored: 8463
        GO_significant: 199
        feasible_genes: 14091
        feasible_genes_significant: 7044
        genes_nodeSize: 5
        Nontrivial_nodes: 8433 
 VirginvsLactate
      BP_VirginvsLactate 
        description: Bioconductor org.Mm.eg.db 2019-Jul10
        available_genes: 15804
        available_genes_significant: 9583
        feasible_genes: 14091
        feasible_genes_significant: 8734
        genes_nodeSize: 5
        nodes_number: 8463
        edges_number: 19543
      elim_BP_VirginvsLactate 
        description: Bioconductor org.Mm.eg.db 2019-Jul10 
        test_name: fisher p<0.01
        algorithm_name: elim
        GO_scored: 8463
        GO_significant: 152
        feasible_genes: 14091
        feasible_genes_significant: 8734
        genes_nodeSize: 5
        Nontrivial_nodes: 8457 
 VirginvsPregnant
      BP_VirginvsPregnant 
        description: Bioconductor org.Mm.eg.db 2019-Jul10
        available_genes: 15804
        available_genes_significant: 7302
        feasible_genes: 14091
        feasible_genes_significant: 6733
        genes_nodeSize: 5
        nodes_number: 8463
        edges_number: 19543
      elim_BP_VirginvsPregnant 
        description: Bioconductor org.Mm.eg.db 2019-Jul10 
        test_name: fisher p<0.01
        algorithm_name: elim
        GO_scored: 8463
        GO_significant: 243
        feasible_genes: 14091
        feasible_genes_significant: 6733
        genes_nodeSize: 5
        Nontrivial_nodes: 8413 
 
- enrichment pvalue cutoff:
        PregnantvsLactate : 0.01
        VirginvsLactate : 0.01
        VirginvsPregnant : 0.01
- enrich GOs (in at least one list): 521 GO terms of 3 conditions.
        PregnantvsLactate : 199 terms
        VirginvsLactate : 152 terms
        VirginvsPregnant : 243 terms"
)

## ----Enrichment_merge_table---------------------------------------------------
# # show table in interactive mode
# ViSEAGO::show_table(BP_sResults)

## ----Enrichment_merge_count---------------------------------------------------
# # barchart of significant (or not) GO terms by comparison
# ViSEAGO::GOcount(BP_sResults)

## ----Enrichment_merge_interactions--------------------------------------------
# # display intersections
# ViSEAGO::Upset(
#     BP_sResults,
#     file="upset.xls"
# )

## ----SS_build-----------------------------------------------------------------
# # create GO_SS-class object
# myGOs<-ViSEAGO::build_GO_SS(
#     gene2GO=myGENE2GO,
#     enrich_GO_terms=BP_sResults
# )

## ----SS_compute---------------------------------------------------------------
# # compute Semantic Similarity (SS)
# myGOs<-ViSEAGO::compute_SS_distances(
#     myGOs,
#     distance="Wang"
# )

## ----SS_build_compute_show----------------------------------------------------
# # display a summary
# myGOs

## ----SS_build_compute_display,echo=FALSE,eval=TRUE----------------------------
cat(
"- object class: GO_SS
- ontology: BP
- method: topGO
- summary:
PregnantvsLactate
      BP_PregnantvsLactate 
        description: Bioconductor org.Mm.eg.db 2019-Jul10
        available_genes: 15804
        available_genes_significant: 7699
        feasible_genes: 14091
        feasible_genes_significant: 7044
        genes_nodeSize: 5
        nodes_number: 8463
        edges_number: 19543
      elim_BP_PregnantvsLactate 
        description: Bioconductor org.Mm.eg.db 2019-Jul10 
        test_name: fisher p<0.01
        algorithm_name: elim
        GO_scored: 8463
        GO_significant: 199
        feasible_genes: 14091
        feasible_genes_significant: 7044
        genes_nodeSize: 5
        Nontrivial_nodes: 8433 
 VirginvsLactate
      BP_VirginvsLactate 
        description: Bioconductor org.Mm.eg.db 2019-Jul10
        available_genes: 15804
        available_genes_significant: 9583
        feasible_genes: 14091
        feasible_genes_significant: 8734
        genes_nodeSize: 5
        nodes_number: 8463
        edges_number: 19543
      elim_BP_VirginvsLactate 
        description: Bioconductor org.Mm.eg.db 2019-Jul10 
        test_name: fisher p<0.01
        algorithm_name: elim
        GO_scored: 8463
        GO_significant: 152
        feasible_genes: 14091
        feasible_genes_significant: 8734
        genes_nodeSize: 5
        Nontrivial_nodes: 8457 
 VirginvsPregnant
      BP_VirginvsPregnant 
        description: Bioconductor org.Mm.eg.db 2019-Jul10
        available_genes: 15804
        available_genes_significant: 7302
        feasible_genes: 14091
        feasible_genes_significant: 6733
        genes_nodeSize: 5
        nodes_number: 8463
        edges_number: 19543
      elim_BP_VirginvsPregnant 
        description: Bioconductor org.Mm.eg.db 2019-Jul10 
        test_name: fisher p<0.01
        algorithm_name: elim
        GO_scored: 8463
        GO_significant: 243
        feasible_genes: 14091
        feasible_genes_significant: 6733
        genes_nodeSize: 5
        Nontrivial_nodes: 8413 
 - enrichment pvalue cutoff:
        PregnantvsLactate : 0.01
        VirginvsLactate : 0.01
        VirginvsPregnant : 0.01
- enrich GOs (in at least one list): 521 GO terms of 3 conditions.
        PregnantvsLactate : 199 terms
        VirginvsLactate : 152 terms
        VirginvsPregnant : 243 terms
- terms distances:  Wang"
)

## ----SS_terms_mdsplot---------------------------------------------------------
# # MDSplot
# ViSEAGO::MDSplot(myGOs)

## ----SS_Wang-wardD2-----------------------------------------------------------
# # Create GOterms heatmap
# Wang_clusters_wardD2<-ViSEAGO::GOterms_heatmap(
#     myGOs,
#     showIC=FALSE,
#     showGOlabels =FALSE,
#     GO.tree=list(
#         tree=list(
#             distance="Wang",
#             aggreg.method="ward.D2"
#         ),
#         cut=list(
#             dynamic=list(
#                 pamStage=TRUE,
#                 pamRespectsDendro=TRUE,
#                 deepSplit=2,
#                 minClusterSize =2
#             )
#         )
#     ),
#     samples.tree=NULL
# )

## ----SS_Wang-wardD2_heatmap_display-------------------------------------------
# # display the heatmap
# ViSEAGO::show_heatmap(
#     Wang_clusters_wardD2,
#     "GOterms"
# )

## ----SS_Wang-ward.D2_table----------------------------------------------------
# # display table
# ViSEAGO::show_table(
#     Wang_clusters_wardD2
# )

## ----SS_Wang-ward.D2_mdsplot--------------------------------------------------
# # colored MDSplot
# ViSEAGO::MDSplot(
#     Wang_clusters_wardD2,
#     "GOterms"
# )

## ----SS_Wang-wardD2_groups----------------------------------------------------
# # calculate semantic similarites between clusters of GO terms
# Wang_clusters_wardD2<-ViSEAGO::compute_SS_distances(
#     Wang_clusters_wardD2,
#     distance="BMA"
# )

## ----SS_Wang-ward.D2_groups_mdsplot-------------------------------------------
# # MDSplot
# ViSEAGO::MDSplot(
#     Wang_clusters_wardD2,
#     "GOclusters"
# )

## ----SS_Wang-wardD2_groups_heatmap--------------------------------------------
# # GOclusters heatmap
# Wang_clusters_wardD2<-ViSEAGO::GOclusters_heatmap(
#     Wang_clusters_wardD2,
#     tree=list(
#         distance="BMA",
#         aggreg.method="ward.D2"
#     )
# )

## ----SS_Wang-ward.D2_groups_heatmap_display-----------------------------------
# # display the heatmap
# ViSEAGO::show_heatmap(
#     Wang_clusters_wardD2,
#     "GOclusters"
# )

## ----SS_Wang-wardD2_groups_show-----------------------------------------------
# # display a summary
# Wang_clusters_wardD2

## ----SS_Wang-wardD2_groups_display,echo=FALSE,eval=TRUE-----------------------
cat(
"- object class: GO_clusters
- ontology: BP
- method: topGO
- summary:
PregnantvsLactate
      BP_PregnantvsLactate 
        description: Bioconductor org.Mm.eg.db 2019-Jul10
        available_genes: 15804
        available_genes_significant: 7699
        feasible_genes: 14091
        feasible_genes_significant: 7044
        genes_nodeSize: 5
        nodes_number: 8463
        edges_number: 19543
      elim_BP_PregnantvsLactate 
        description: Bioconductor org.Mm.eg.db 2019-Jul10 
        test_name: fisher p<0.01
        algorithm_name: elim
        GO_scored: 8463
        GO_significant: 199
        feasible_genes: 14091
        feasible_genes_significant: 7044
        genes_nodeSize: 5
        Nontrivial_nodes: 8433 
 VirginvsLactate
      BP_VirginvsLactate 
        description: Bioconductor org.Mm.eg.db 2019-Jul10
        available_genes: 15804
        available_genes_significant: 9583
        feasible_genes: 14091
        feasible_genes_significant: 8734
        genes_nodeSize: 5
        nodes_number: 8463
        edges_number: 19543
      elim_BP_VirginvsLactate 
        description: Bioconductor org.Mm.eg.db 2019-Jul10 
        test_name: fisher p<0.01
        algorithm_name: elim
        GO_scored: 8463
        GO_significant: 152
        feasible_genes: 14091
        feasible_genes_significant: 8734
        genes_nodeSize: 5
        Nontrivial_nodes: 8457 
 VirginvsPregnant
      BP_VirginvsPregnant 
        description: Bioconductor org.Mm.eg.db 2019-Jul10
        available_genes: 15804
        available_genes_significant: 7302
        feasible_genes: 14091
        feasible_genes_significant: 6733
        genes_nodeSize: 5
        nodes_number: 8463
        edges_number: 19543
      elim_BP_VirginvsPregnant 
        description: Bioconductor org.Mm.eg.db 2019-Jul10 
        test_name: fisher p<0.01
        algorithm_name: elim
        GO_scored: 8463
        GO_significant: 243
        feasible_genes: 14091
        feasible_genes_significant: 6733
        genes_nodeSize: 5
        Nontrivial_nodes: 8413 
 - enrichment pvalue cutoff:
        PregnantvsLactate : 0.01
        VirginvsLactate : 0.01
        VirginvsPregnant : 0.01
- enrich GOs (in at least one list): 521 GO terms of 3 conditions.
        PregnantvsLactate : 199 terms
        VirginvsLactate : 152 terms
        VirginvsPregnant : 243 terms
- terms distances:  Wang
- clusters distances: BMA
- Heatmap:
          * GOterms: TRUE
                    - GO.tree:
                              tree.distance: Wang
                              tree.aggreg.method: ward.D2
                              cut.dynamic.pamStage: TRUE
                              cut.dynamic.pamRespectsDendro: TRUE
                              cut.dynamic.deepSplit: 2
                              cut.dynamic.minClusterSize: 2
                              number of clusters: 62
                              clusters min size: 2
                              clusters mean size: 8
                              clusters max size: 32
                   - sample.tree: FALSE
          * GOclusters: TRUE
                       - tree:
                              distance: BMA
                              aggreg.method: ward.D2"
)

## ----session,eval=TRUE,echo=FALSE---------------------------------------------
sessionInfo()

#' @title Barplot for the count of GO terms.
#' @description This method displays in barplot the count of GO terms splitted in two categories (significant or not)
#' for each result of GO enrichment tests.
#' @importFrom data.table data.table melt := .SD
#' @importFrom plotly plot_ly layout export orca
#' @importFrom methods is slot
#' @family GO_terms
#' @family visualization
#' @param object an \code{\link{enrich_GO_terms-class}} object from \code{\link{merge_enrich_terms}} method.
#' @param file the name of the output file (default to NULL for interactive screen display).
#' @details This method displays an interactive barplot, using \pkg{plotly} package, from a \code{\link{merge_enrich_terms}}
#' output object.\cr
#' A static image (in png) could be printed by setting \code{file} argument.
#' @return a barplot.
#' @references
#' Carson Sievert, Chris Parmer, Toby Hocking, Scott Chamberlain, Karthik Ram, Marianne Corvellec and Pedro Despouy (2017).
#' plotly: Create InteractiveWeb Graphics via 'plotly.js'. R package version 4.6.0. https://CRAN.R-project.org/package=plotly.
#' @include enrich_GO_terms.R
#' @examples
#' # load object
#' utils::data(
#'  myGOs,
#'  package="ViSEAGO"
#' )
#'
#' # barplot for the count of GO terms
#' ViSEAGO::GOcount( myGOs)
#' @exportMethod GOcount
#' @name GOcount
#' @rdname GOcount-methods
#' @exportMethod GOcount
setGeneric(
    name="GOcount",
    def=function(object,file=NULL){
        standardGeneric("GOcount")
    }
)

#' @rdname GOcount-methods
#' @aliases GOcount
setMethod(
    "GOcount",
    signature="ANY",
    definition=function(object,file){

        # check class
        if(!is(object,"enrich_GO_terms") & !is(object,"GO_SS") & !is(object,"GO_clusters")){
            stop("object must be enrich_GO_terms, GO_SS, or GO_clusters class objects")
        }

        # Extract data
        if(is(object,"enrich_GO_terms")){

            # data
            data<-slot(
                object,
                "data"
            )

            # cut off
            cutoff<-slot(object,"cutoff")[[1]]
            
        }else{

            # data
            data<-slot(
                slot(
                    object,
                    "enrich_GOs"
                ),
                "data"
            )

            # cutoff
            cutoff<-slot(
                slot(
                    object,
                    "enrich_GOs"
                ),
                "cutoff"
            )[[1]]
        }

        # find pvalues columns
        pvalues<-grep("\\.pvalue",names(data))

        # check more than one pvalues
        if(length(pvalues)==1){
            stop("GOcount is required for multiples comparisons")
        }

        # count significant (or not) pvalues by condition
        Data<-data[,pvalues,with=FALSE]

        # count significant according cutoff
        Data<-lapply(1:ncol(Data), function(x){

            # count
            res<-table(Data[,x,with=FALSE]<cutoff[x])

            # count
            if(length(res)==1){
                
                if(names(res)=="TRUE"){
                    res<-c(0,res)
                }else{
                    res<-c(res,0)
                }

            }else{
                res<-as.vector(res)
            } 

            # convert to data.table
            data.table(
                pvalue=c("not enriched GO terms","enriched GO terms"),
                condition=gsub("\\.pvalue","",names(Data)[x]),
                count=res
            )
        })
        
        # bind results
        Data<-rbindlist(Data)

        # assign class values
        Data[,
            `:=`(
                pvalue=factor(Data$pvalue,levels = c("enriched GO terms","not enriched GO terms")),
                condition=factor(Data$condition,levels =rev(unique(Data$condition))),
                count=as.numeric(Data$count)
            )
        ]

        # barchart
        p<-plot_ly(
            data=Data,
            type = 'bar',
            y = ~condition,
            x = ~count,
            color=~pvalue,
            colors=c("#FED8B1","#87ceeb")
        )

        # barchart layout
        p<-layout(p,
            xaxis =list(
                title = 'GO terms number'
            ),
            yaxis =list(
                title =""
            ),
             margin =list(t=100,b =100,l=150),
            title="number of significant (or not) GO terms by conditions",
            barmode = 'stack'
        )

         # return or print
        if(is.null(file)){

            # return the plot
            p

        }else{

            # print heatmap
            orca(p,file=file)
        }
    }
)

#' @title GO_SS class object definition.
#' @description This class is invoked by \code{\link{build_GO_SS}} method in order to store \code{\link{enrich_GO_terms-class}} object, Information Content (IC),
#' and GO terms or groups distances objects based on semantic similarity.
#' @family GO_semantic_similarity
#' @slot ont ontology used "MF", "BP", or "CC".
#' @slot enrich_GOs \code{\link{merge_enrich_terms}} output object (\code{\link{enrich_GO_terms-class}} object).
#' @slot IC Information Content (IC)
#' @slot terms_dist \code{list} of GO terms or groups distances objects based on semantic similarity.
#' @include enrich_GO_terms.R
setClass(
    "GO_SS",
    slots=c(
        ont="character",
        enrich_GOs="enrich_GO_terms",
        IC ="numeric",
        terms_dist="list"
    )
)

#' @aliases GO_SS
#' @importFrom data.table melt.data.table .SD
setMethod(
    "show",
    signature="GO_SS",
    function(object){

        # keep full object
        full_object<-object
        
        # extract object
        object<-slot(object,"enrich_GOs")

        # Extract table
        Data<-slot(object,"data")

        # Extract pvalues
        Data<-Data[,grep("\\.pvalue",names(Data)),with=FALSE]
        
        # pvalues threshlod according condition
        p<-unlist(slot(object,"cutoff"))
        
        # count significant pvalues by condition
        Data<-lapply(seq_len(ncol(Data)),function(x){
            data.table(
                conditions=sub("\\.pvalue","",names(Data)[x]),
                `significant GO terms number`=sum(Data[,x,with=FALSE]<p[x],na.rm=TRUE)
            )
        })
        
        # bind results
        Data<-rbindlist(Data)
        
        # get topGO information
        obj_summary<-slot(object,"summary")
        
        # summary according method
        obj_summary<-vapply(names(obj_summary),function(x){
            
            # subset
            Data<-obj_summary[[x]]
            
            # topGO summary
            if(slot(object,"method")=="topGO"){
                
                # topGO summary by element
                elems<-paste(
                    vapply(names(Data),function(y){
                        
                        # summery by element
                        paste(
                            paste("   ",y),
                            "\n       ",
                            paste(
                                paste(
                                    names(Data[[y]]),
                                    sub("\n.+$","",unlist(Data[[y]])),
                                    sep=": "
                                ),
                                collapse="\n        "
                            )
                        )
                    },""),
                    collapse="\n  "
                )
                
                # summary by element
                return(paste(paste(x,elems,sep ="\n  "),"\n "))
            }
            
            # fgsea summary
            if(slot(object,"method")=="fgsea"){
                
                # fgsea summary by element
                elems<-paste(
                    paste(
                        "\n    ",
                        names(
                            Data
                        ),
                        " : ",
                        unlist(
                            Data
                        ),
                        sep=""
                    ),
                    collapse=""
                )
                
                # summary by element
                return(paste(paste("\n ",x,elems,sep =""),collapse="/n"))
            }
        },"")

        # cat some text
        cat("- object class: GO_SS",
            "\n- ontology: ",slot(object,"ont"),
            "\n- method: ",slot(object,"method"),
            "\n- summary:\n", obj_summary,
            "- enrichment pvalue cutoff:",
            paste("\n       ",Data$conditions,":",slot(object,"cutoff")[[1]]),
            "\n- enrich GOs (in at least one list): ",nrow(slot(object,"data"))," GO terms of ",nrow(Data)," conditions.",
            paste("\n       ",Data$conditions,":",Data$`significant GO terms number`,"terms"),
            if(length(slot(full_object,"terms_dist"))>0){
                paste(
                    "\n- terms distances: ",
                    paste(
                        names(
                            slot(full_object,"terms_dist")
                        ),
                        collapse=", "
                    )
                )
            },
            sep=""
        )
    }
)

#' @title Retrieve GO annotations for a specie from genomic ressource database.
#' @description This method retrieves and stores GO annotations for the
#' organism of interest from one of genomic ressource database
#' (Bioconductor, EntrezGene, Ensembl, Uniprot).
#' @import AnnotationDbi
#' @importFrom biomaRt listFilters useDataset getBM
#' @import data.table
#' @importFrom utils installed.packages
#' @family genomic_ressource
#' @family GO_terms
#' @param id identifiant corresponding to the organism of interest.
#' This id name is referenced in the first column of the database
#' used (see \code{\link{available_organisms}}).
#' @param object a required \code{\link{genomic_ressource-class}} object created by
#' \code{\link{Bioconductor2GO}}, \code{\link{EntrezGene2GO}},
#' \code{\link{Ensembl2GO}}, or \code{\link{Uniprot2GO}} methods.
#' @param ortholog \code{logical} (default to FALSE). Only available for
#' vertebrates organisms and for object created by \code{\link{EntrezGene2GO}} method (see Details).
#' @details This method uses a \code{\link{genomic_ressource-class}} object to retrieve
#' \href{http://www.geneontology.org/page/ontology-documentation}{GO} annotations for the organism of interest.
#' The stored annotations are structured in 3 slots corresponding to the 3 GO categories: MF (Molecular Function),
#' BP (Biological Process), and CC (Cellular Component). Each slot contains GO terms with
#' associated \href{http://www.geneontology.org/page/guide-go-evidence-codes}{evidence code}.
#'
#' The \code{\link{genomic_ressource-class}} object is created by one of the four available methods:
#' \code{\link{Bioconductor2GO}}, \code{\link{EntrezGene2GO}},
#' \code{\link{Ensembl2GO}}, or \code{\link{Uniprot2GO}}.
#'
#' In the case of vertebrates, setting \code{ortholog} argument to \code{TRUE} is required if you need to add GO terms with experimental
#' \href{http://geneontology.org/page/guide-go-evidence-codes}{evidence codes} from orthologs genes
#' when using \code{\link{EntrezGene2GO}} method. To display organisms supported by NCBI EntrezGene orthologs pipeline,
#' set the arguments \code{id=NULL} and \code{ortholog=TRUE}.
#' This approch is highly similar to the strategy developed by Uniprot-GOA consortium for the Electronic Annotation Method using
#' \href{http://www.ebi.ac.uk/GOA/compara_go_annotations}{Ensembl Compara}.
#' @return \code{annotate} produces an object of \code{\link{gene2GO-class}} required by \code{\link{build_GO_SS}} method.
#' @references
#' Durinck S, Spellman P, Birney E and Huber W (2009). Mapping identifiers for the integration of genomic datasets with the R/Bioconductor
#' package biomaRt. Nature Protocols, 4, pp. 1184-1191.
#'
#' Durinck S, Moreau Y, Kasprzyk A, Davis S, De Moor B, Brazma A and Huber W (2005).
#' BioMart and Bioconductor: a powerful link between biological databases and microarray data analysis. Bioinformatics, 21, pp. 3439-3440.
#'
#' Fong, JH, Murphy, TD, Pruitt, KD (2013). Comparison of RefSeq protein-coding regions in human and vertebrate genomes. BMC Genomics, 14:654.
#'
#' Henrik Bengtsson (2016). R.utils: Various Programming Utilities. R package version 2.5.0. https://CRAN.R-project.org/package=R.utils.
#'
#' Herve Pages, Marc Carlson, Seth Falcon and Nianhua Li (2017). AnnotationDbi: Annotation Database Interface. R package version 1.38.0.
#'
#' Matt Dowle and Arun Srinivasan (2017). data.table: Extension of data.frame. R package version 1.10.4. https://CRAN.R-project.org/package=data.table.
#' @include genomic_ressource.R
#' @examples
#' \dontrun{
#' ## load Mus musculus (mouse) GO annotations
#'
#' # from Bioconductor
#' Bioconductor<-ViSEAGO::Bioconductor2GO()
#' myGENE2GO<-ViSEAGO::annotate(
#'     id="org.Mm.eg.db",
#'     object=Bioconductor
#' )
#'
#' # from EntrezGene
#' EntrezGene<-ViSEAGO::EntrezGene2GO()
#' myGENE2GO<-ViSEAGO::annotate(
#'     id="10090",
#'     object=EntrezGene
#' )
#'
#' # from EntrezGene
#' Ensembl<-ViSEAGO::Ensembl2GO()
#' myGENE2GO<-ViSEAGO::annotate(
#'     id="mmusculus_gene_ensembl",
#'     object=Ensembl
#' )
#'
#' # from Uniprot
#' Uniprot<-ViSEAGO::Uniprot2GO()
#' myGENE2GO<-ViSEAGO::annotate(
#'     id="mouse",
#'     object=Uniprot
#' )
#'
#' ## from Custom GO annotation file
#' Custom<-ViSEAGO::Custom2GO(system.file("extdata/customfile.txt",package = "ViSEAGO"))
#' myGENE2GO<-ViSEAGO::annotate(
#'     id="myspecies1",
#'     object=Custom
#' )
#'
#' ## specific options for EntrezGene database
#'
#' # Chicken GO annotations without adding orthologs
#' EntrezGene<-ViSEAGO::EntrezGene2GO()
#' myGENE2GO<-ViSEAGO::annotate(
#'     id="9031",
#'     object=EntrezGene
#' )
#'
#' # Chicken GO annotation with the add of orthologs GO annotations
#' EntrezGene<-ViSEAGO::EntrezGene2GO()
#' myGENE2GO<-ViSEAGO::annotate(
#'     id="9031",
#'     object=EntrezGene,
#'     ortholog=TRUE
#' )
#'
#' # display organisms supported by NCBI EntrezGene orthologs pipeline
#' EntrezGene<-ViSEAGO::EntrezGene2GO()
#' ViSEAGO::annotate(
#'     id="NULL",
#'     object=EntrezGene,
#'     ortholog=TRUE
#' )
#' }
#' @name annotate
#' @rdname annotate-methods
#' @exportMethod annotate
setGeneric(
    name="annotate",
    def=function(id,object,ortholog=FALSE){
        standardGeneric("annotate")
    }
)

#' @rdname annotate-methods
#' @aliases annotate
setMethod(
    "annotate",
    signature(
        id="character",
        object="genomic_ressource"
    ),
    definition=function(id,object,ortholog){

        ## check object
        if(!is(object,"genomic_ressource")){
            stop(
                "object must be a genomic_ressource class from ViSEAGO::Bioconductor2GO(), ViSEAGO::EntrezGene2GO(), ViSEAGO::Ensembl2GO() or ViSEAGO::Custom2GO()"
            )
        }

        if(slot(object,"db")!="EntrezGene" & ortholog==TRUE){
            stop("ortholog option is only available for genomic_ressource class object from ViSEAGO::EntrezGene2GO()")
        }

        ## Annotate

        # EntrezGene
        if(slot(object,"db")=="EntrezGene"){

        # for orthologs
        if(ortholog==TRUE){

            # select the target species and ortholog
            gene_group<-EntrezGene_orthologs()

            # extract Organisms informations
            taxon=taxonomy(
                unique(
                    c(
                        gene_group$tax_id,
                        gene_group$Other_tax_id
                    )
                )
            )

            # select the target species and ortholog
            if(id=="NULL"){

                # ordering by Scientific name
                setorderv(
                    taxon,
                    "ScientificName"
                )

                # stop scrip execution
                warnings(
                    "Set id argument from ViSEAGO::annotate(id,EntrezGene,ortholog=TRUE) with an  available taxid (see below), and retry.",
                    "Available EntrezGene species with orthologs_groups:\n\n",
                    sep="\n"
                )

                # print taxon
                print(
                    taxon,
                    nrows=nrow(taxon)
                    )

                # silencing stop
                opt<-options(
                    show.error.messages=FALSE
                )
                on.exit(
                    options(opt)
                )
                stop()

            }else{

                # check match id
                id=match.arg(
                    id,
                    taxon$taxid
                )

                # select orthologs relationship
                gene_group<-gene_group["Ortholog",on="relationship"]

                # select target species in tax_id columns
                gene_group<-lapply(c("tax_id","Other_tax_id"),function(x){
                    gene_group[id,on=x]
                })

                # convert to data.table
                gene_group<-rbindlist(gene_group)

                # select the target species and ortholog from the left part of the table
                group1<-gene_group[id,c("GeneID","Other_GeneID"),with=FALSE,on="tax_id"]

                # select the target species and ortholog from the right part of the table
                group2<-gene_group[id,c("Other_GeneID","GeneID"),with=FALSE,on="Other_tax_id"]

                # renames(group2)
                names(group2)<-names(group1)

                # bind and assign to gene_group (GeneID from target species in first column)
                gene_group<-rbind(
                    group1,
                    group2
                )

                # Extract otholog species annotation from data slot
                ortho<-slot(object,"data")[c("EXP","IDA","IPI","IMP","IGI", "IEP"), on="evidence"]

                # merge experimental GO anotation from orthologs
                ortho<-merge(
                    gene_group,
                    ortho,
                    by.x="Other_GeneID",
                    by.y="gene_id"
                )

                # remove ortholog GeneID
                ortho[,"Other_GeneID":=NULL]

                # rename GeneID to gene_id
                names(ortho)[1]<-"gene_id"

                # assign target species id to taxid and replace exprerimental evidence by IEA (computationnal)
                ortho[,`:=`(taxid=id,evidence="IEA")]

                # Extract species annotation from data slot
                annot<-slot(object,"data")[
                    id,
                    on="taxid"
                ]

                # add orthologs annotation to species annotation
                annot<-unique(
                    rbind(
                        annot,
                        ortho
                    )
                )
            }

        }else{

            # Extract species annotation from data slot
            annot<-slot(object,"data")[id,on="taxid"]
        }

        # Extract species annotation from data slot
        annot["Function","category":="MF",on="category"]
        annot["Process","category":="BP",on="category"]
        annot["Component","category":="CC",on="category"]

        # GO database stamp
        stamp=slot(object,"stamp")
    }

        # Bioconductor
        if(slot(object,"db")=="Bioconductor"){

            # check id
            id=match.arg(
                id,
                slot(Bioconductor2GO(),"organisms")$Package
            )

            # install package if needed
            if(!id%in%installed.packages()[,"Package"]){

                # bioclite source
                stop(
                    paste(
                        'Please install database package from Bioconductor using BiocManager::install("',
                        id,
                        '")',
                        sep=""
                    )
                )
            }

            # load db package
            require(id,character.only =TRUE)

            # load GO annotations
            annot<-data.table(
                select(
                    get(id),
                    keys=keys(get(id)),
                    columns =c("ENTREZID","GO","ONTOLOGY")
                )
            )

            # keep targets columns
            annot<-annot[,.(ENTREZID,GO,EVIDENCE,ONTOLOGY)]

            # rename columns
            names(annot)<-c("gene_id","GOID","evidence","category")

            # GO database stamp
            stamp=eval(
                call(
                    sub("\\.db","_dbInfo",id)
                )
            )[15,2]
        }

        # Ensembl
        if(slot(object,"db")=="Ensembl"){

            # connect to ensembl specified dataset
            myspecies<-useDataset(
                id,
                slot(object,"mart")[[1]]
            )

            # with go name according biomart version
            go_filter<-data.table(
                listFilters(myspecies)
            )

            # load Ensembl genes with GO annotations
            annot<-data.table(
                getBM(
                    attributes =c("ensembl_gene_id","go_id","go_linkage_type","namespace_1003"),
                    filters=unlist(go_filter[
                        grep("with GO ID",ignore.case =TRUE,go_filter$description),
                        "name",
                        with=FALSE
                    ]),
                  values=TRUE,
                  mart =myspecies
                )
            )

            # rename columns
            colnames(annot)<-c("gene_id","GOID","evidence","category")

            # Extract species annotation from data slot
            annot["molecular_function","category":="MF",on="category"]
            annot["biological_process","category":="BP",on="category"]
            annot["cellular_component","category":="CC",on="category"]

            # GO database stamp
            stamp=slot(object,"stamp")
        }

        # Uniprot
        if(slot(object,"db")=="Uniprot-GOA"){

            # temp file
            temp<-paste(
                tempfile(),
                "gz",
                sep="."
            )

            # time out option
            options(timeout = max(300, getOption("timeout")))

            # load the file
            download.file(
                paste(
                    'https://ftp.ebi.ac.uk/pub/databases/GO/goa/',
                    toupper(id),
                    '/goa_',
                    id,
                    '.gaf.gz',
                    sep=""
                ),
                destfile =temp,
                quiet=TRUE,
                method = "libcurl"
            )

            # unzip
            gunzip(temp)

            # read file
            annot<-unique(
                fread(
                    sub("\\.gz","",temp),
                    skip=12,
                    select=c(2,5,7,9),
                    col.names=c("gene_id","GOID","evidence","category")
                )
            )

            # Extract species annotation from data slot
            annot["F","category":="MF",on="category"]
            annot["P","category":="BP",on="category"]
            annot["C","category":="CC",on="category"]

            # GO database stamp
            stamp=slot(object,"stamp")
        }

        # Custom
        if(slot(object,"db")=="Custom"){

            # Extract species annotation from data slot
            annot<-slot(object,"data")[id,on="taxid"]

            # add stamp
            stamp=slot(object,"stamp")
        }

        ## convert to list

        # split annot to chuncks
        Data<-split(
            annot[,c("GOID","evidence","category","gene_id"),with=FALSE],
            by=c("category","gene_id"),
            flatten=FALSE,
            keep.by=FALSE
            )

        # convert chunks elements fot topGO compatbility
        Data<-lapply(Data,function(x){

            lapply(x,function(y){

                # extract GOID
                values<-y$GOID

                # add evidence
                names(values)<-y$evidence

                # return
                return(values)
            })
        })

        # ordering category  and add empty list if not available
        Data<-lapply(c("MF","BP","CC"),function(x){

            if(!is.null(Data[[x]])){
                Data[[x]]
            }else{
                list()
            }
        })
        names(Data)<-c("MF","BP","CC")

        ## create GENE2GO object
        new(
            "gene2GO",
            db=slot(object,"db"),
            stamp=stamp,
            organism=id,
            MF=Data$MF,
            BP=Data$BP,
            CC=Data$CC
        )
})

#' @title Build a clustering heatmap on GO groups.
#' @description This method computes a clustering heatmap based on GO groups semantic similarity.
#' @importFrom data.table data.table .N
#' @importFrom ggplot2 scale_fill_gradient
#' @importFrom methods slot slot<-
#' @importFrom stats hclust as.dendrogram order.dendrogram
#' @importFrom plotly layout
#' @importFrom heatmaply heatmaply
#' @importFrom ComplexHeatmap Heatmap max_text_width
#' @importFrom circlize colorRamp2
#' @import dendextend
#' @importFrom RColorBrewer brewer.pal
#' @family GO_clusters
#' @family semantic_similarity
#' @family visualization
#' @param object a  \code{\link{GO_clusters-class}} object from \code{\link{compute_SS_distances}}.
#' @param tree a named list with:
#'  \describe{
#'      \item{distance ("BMA" by default)}{
#'      distance computed from the semantic similarity for GO groups which could be "max", "avg", "rcmax",or "BMA".}
#'      \item{aggreg.method ("ward.D2" by default)}{aggregation method criteria from \code{\link[stats]{hclust}} (ward.D",
#'      "ward.D2", "single", "complete", "average", "mcquitty", "median", or "centroid") to build a \code{dendrogram}.}
#'      \item{rotate}{sort the branches of the tree based on a vector - eithor of labels order or the labels in their new order}
#'  }
#' @details This method computes a clustering heatmap based on GO groups semantic similarity (computed with \code{\link{compute_SS_distances}}).\cr
#' The heatmap color intensity corresponds to the number of GO terms in each GO group.\cr
#' GO group description is defined as the first common GO ancestor with the cluster identifiant in brackets.\cr
#' The dendrogram branches are colored according to GO terms clusters.
#' @references
#' Matt Dowle and Arun Srinivasan (2017). data.table: Extension of `data.frame`. R package version 1.10.4. https://CRAN.R-project.org/package=data.table.
#'
#' Tal Galili (2015). dendextend: an R package for visualizing, adjusting, and comparing trees of hierarchical clustering.
#' Bioinformatics. DOI:10.1093/bioinformatics/btv428.
#'
#' Tal Galili (2017). heatmaply: Interactive Cluster Heat Maps Using 'plotly'.
#' R package version 0.9.1. https://CRAN.R-project.org/package=heatmaply.
#'
#' Erich Neuwirth (2014). RColorBrewer: ColorBrewer Palettes. R package version 1.1-2. https://CRAN.R-project.org/package=RColorBrewer.
#'
#' Carson Sievert, Chris Parmer, Toby Hocking, Scott Chamberlain, Karthik Ram, Marianne Corvellec and Pedro Despouy (2017).
#' plotly: Create Interactive Web Graphics via 'plotly.js'. R package version 4.6.0. https://CRAN.R-project.org/package=plotly.
#'
#' H. Wickham. ggplot2: Elegant Graphics for Data Analysis. Springer-Verlag New York, 2009.
#' @return a \code{\link{GO_clusters-class}} object.
#' @include GO_clusters.R
#' @examples
#' # load data example
#' utils::data(
#'     myGOs,
#'     package="ViSEAGO"
#' )
#' \dontrun{
#' # compute GO terms Semantic Similarity distances
#' myGOs<-ViSEAGO::compute_SS_distances(
#'     myGOs,
#'     distance="Wang"
#' )
#'
#' # GOtermsHeatmap with default parameters
#' Wang_clusters_wardD2<-ViSEAGO::GOterms_heatmap(
#'     myGOs,
#'     showIC=TRUE,
#'     showGOlabels=TRUE,
#'     GO.tree=list(
#'         tree=list(
#'             distance="Wang",
#'             aggreg.method="ward.D2",
#'             rotate=NULL
#'         ),
#'         cut=list(
#'             dynamic=list(
#'                 pamStage=TRUE,
#'                 pamRespectsDendro=TRUE,
#'                 deepSplit=2,
#'                 minClusterSize =2
#'             )
#'         )
#'     ),
#'     samples.tree=NULL
#' )
#'
#' # compute clusters of GO terms Semantic Similarity distances
#' Wang_clusters_wardD2<-ViSEAGO::compute_SS_distances(
#'     Wang_clusters_wardD2,
#'     distance="BMA"
#' )
#'
#' # GOclusters heatmap
#' Wang_clusters_wardD2<-ViSEAGO::GOclusters_heatmap(
#'     Wang_clusters_wardD2,
#'     tree=list(
#'         distance="BMA",
#'         aggreg.method="ward.D2",
#'         rotate=NULL
#'     )
#' )
#' }
#' @exportMethod GOclusters_heatmap
#' @name GOclusters_heatmap
#' @rdname GOclusters_heatmap-methods
#' @exportMethod GOclusters_heatmap
setGeneric(
    name="GOclusters_heatmap",
    def=function(object,tree=list(distance="BMA",aggreg.method="ward.D2",rotate=NULL)){
        standardGeneric("GOclusters_heatmap")
    }
)

#' @rdname GOclusters_heatmap-methods
#' @aliases GOclusters_heatmap
setMethod(
    "GOclusters_heatmap",
    signature(
        object="GO_clusters",
        tree="list"
    ),
    definition=function(object,tree){

        # if empty
        if(is.null(slot(object,"clusters_dist"))){
            stop("Please use GO_clusters class object with computed clusters distances using ViSEAGO::compute_SS_distance()")
        }

        # if empty
        if(!tree$distance%in%names(slot(object,"clusters_dist"))){
            stop(paste("Please use GO_clusters class object with",tree$distance,"computed clusters distances using ViSEAGO::compute_SS_distance()"))
        }

        # add calculated distance
        dist<-slot(object,"clusters_dist")[[tree$distance]]

        # perform hclust
        Tree<-hclust(
            dist,
            method =tree$aggreg.method
          )

        # create dendrogram
        dend<-as.dendrogram(Tree)

        # rotate
        if(!is.null(tree$rotate)){
            dend<-rotate(
                dend,
                tree$rotate
            )
        }

        # create dendrogram
        ord<-order.dendrogram(dend)

        # color labels of cutting tree
        colors= unique(
            get_leaves_attr(
                slot(object,"dendrograms")$GO,
            "edgePar"
            )
        )

        # extract GO terms and clusters
        mat<-slot(slot(object,"enrich_GOs"),"data")[,c("GO.cluster","GO.ID"),with=FALSE]

        # count terms by clusters
        mat<-mat[,list("count"=.N),by="GO.cluster"]

        # count terms by clusters
        mat<-as.matrix(mat[,"count",with=FALSE])

        # colors table
        colors=data.table(
            gp=seq_len(nrow(mat)),
            color=colors
        )

        # merge cluster term assignation and corresponding color
        colors<-merge(
            data.table( gp=colors$gp[ord]),
            colors,
            by="gp",
            all.x=TRUE,
            sort=FALSE
        )

        # color branches according clusters
        dend<-branches_attr_by_clusters(
             dend,
            seq_len(nrow(mat)),
            values =colors$color
        )

        # assign text color
        dend<-dendextend::set(dend,"labels_colors",value=colors$color)

        # create dendrogram
        dend<-dendextend::set(dend,"labels_cex",0.3)

        # extract GO terms and clusters
        row.names(mat)<-attr(dist,"Labels")

        # custom row names
        row.names(mat)<-paste("<br>cluster:", row.names(mat))
        row.names(mat)<-gsub("_GO:","<br>GO.ID: GO:",row.names(mat))
        row.names(mat)<-gsub("_","<br>GO.name: ", row.names(mat))

        # draw heatmapply
        hm<-heatmaply(

            # the initial matrix
            x=mat,

            # row label
            labRow=row.names(mat),

            # row label
            labCol=colnames(mat),

            # the row dendrogram
            Rowv=dend,

            # the ordered matrix according dendrograms for columns
            Colv=FALSE,

            # the color palette
            scale_fill_gradient_fun=scale_fill_gradient(
                name="GO terms count",
                low="#FCFBFD",
                high="#3F007D"
            ),

            # the width of dendrogramm
            branches_lwd =0.1,

            # color bar length
            colorbar_len=0.05
        )

        # remove row tag
        hm$x$data[[1]]$text<-gsub("<br>row: ","",hm$x$data[[1]]$text)

        # custom row text
        row.text=gsub("^.+GO.name: ","",rev(row.names(mat)[ord]))

        # cut very long definition
        row.text[nchar(row.text)>50]<-paste(substring(
        row.text[nchar(row.text)>50],1,50),"...",sep="")

        # add cluster number in brackets
        row.text=paste(row.text," (cl",
        gsub("^<br>cluster: |<br>GO.ID.+$","",
        rev(row.names(mat)[ord])),")",sep="")

        # heatmap layout
        hm<-layout(
            hm,

            # add title
            title=paste(tree$distance,"GOclusters distance heatmap"),

            # x axis
            xaxis=list(
                domain=c(0,0.2),
                family="Times New Roman",
                tickfont=list(size=10)
            ),
            xaxis2=list(
                domain=c(0.2,0.8)
            ),

            # title size
            font=list(size=14),

            # y axis
            yaxis=list(
                family="Times New Roman",
                tickmode="array",
                tickvals=seq_len(nrow(mat)),
                ticktext=row.text,
                tickfont=list(size=10)
            ),
            margin = list(l =300,r=0, b =150,t=100)
        )

        # custom row text
        row.text=gsub("^.+GO.name: ","",row.names(mat))
        
        # cut very long definition
        row.text[nchar(row.text)>50]<-paste(
            substring(
            row.text[nchar(row.text)>50],1,50),"...",
            sep=""
        )
        
        # add cluster number in brackets
        row.text=paste(
            row.text,
            " (cl",
            gsub(
                "^<br>cluster: |<br>GO.ID.+$",
                "",
                row.names(mat)
            ),
            ")",
            sep=""
        )

        # replace
        row.names(mat)<-row.text

        # draw static heatmap
        hs<-Heatmap(
            mat,
            raster_device="png",
            use_raster = TRUE,
            cluster_rows=dend,
            row_names_side = "left",
            row_dend_side = "right",
            show_row_names = TRUE,
            show_column_names = TRUE,
            col=colorRamp2(c(0,max(mat)), c("#FCFBFD","#3F007D")),
            row_dend_width = grid::unit(10, "cm"),
            row_names_max_width = max_text_width(
                rownames(mat), 
            ),
            name="GO terms count"
        )

        # hm to list
        hm<-list(hm,hs)

        # give names to hm list
        names(hm)<-c("GOclusters","GOclusters_static")

        # give names to hm list
        slot(object,"heatmap")<-c(slot(object,"heatmap"),hm)

        # add hcl params
        slot(object,"hcl_params")<-c(slot(object,"hcl_params"),GO.clusters=list(tree))

        # return the object
        return(object)
    }
)

#' @title ViSEAGO package
#' @description Easier data mining of biological functions organized into clusters using Gene Ontology and semantic.
#' @details The main objective of ViSEAGO workflow is to carry out a data mining of biological functions and establish
#' links between genes involved in the study. We developed ViSEAGO in R to facilitate functional Gene Ontology (GO)
#' analysis of complex experimental design with multiple comparisons of interest.
#'
#' It allows to study large-scale datasets together and visualize GO profiles to capture biological knowledge.
#' The acronym stands for three major concepts of the analysis: Visualization, Semantic similarity and Enrichment Analysis of Gene Ontology
#' (\code{\link{pkgdiagram}}).
#'
#' It provides access to the last current GO annotations (\code{\link{annotate}}), which are retrieved from one of
#' NCBI EntrezGene (\code{\link{Bioconductor2GO}}, \code{\link{EntrezGene2GO}}),
#' Ensembl (\code{\link{Ensembl2GO}}) or Uniprot (\code{\link{Uniprot2GO}}) databases
#' for available species (\code{\link{available_organisms}}).
#'
#' ViSEAGO extends classical functional GO analysis (\code{\link{create_topGOdata}}) to focus on functional coherence
#' by aggregating closely related biological themes while studying multiple datasets at once (\code{\link{merge_enrich_terms}}).
#'
#' It provides both a synthetic and detailed view using interactive functionalities respecting the GO graph structure
#' (\code{\link{MDSplot}}, \code{\link{GOterms_heatmap}}, \code{\link{GOclusters_heatmap}}), and ensuring functional
#' coherence supplied by semantic similarity (\code{\link{build_GO_SS}}, \code{\link{compute_SS_distances}}).
#'
#' ViSEAGO has been successfully applied on several datasets from different species with a variety
#' of biological questions. Results can be easily shared between bioinformaticians and biologists, enhancing reporting capabilities while
#'  maintaining reproducibility.
#'
#' @docType package
#' @name ViSEAGO
NULL

#' @title enrich_GO_terms class object definition.
#' @description This class is invoked by \code{\link{merge_enrich_terms}} method in order to store the merged \code{data.table} and associated metadata.
#' @family enrich_GO_terms
#' @slot same_genes_background logical.
#' object(s) to combinate (see examples in \code{\link{merge_enrich_terms}}).
#' @slot ont ontology used "MF", "BP", or "CC".
#' @slot method enrichment test used "topGO", or "fgsea".
#' @slot summary  a \code{list} with topGO or fgsea object(s) summary informations.
#' @slot data a merged \code{data.table}  of enriched GO terms (p<0.01) in at least once with GO descriptions and statistical values.
setClass(
    "enrich_GO_terms",
    slots=c(
        same_genes_background="logical",
        ont="character",
        method="character",
        cutoff="list",
        summary="list",
        data="data.table"
    )
)

#' @aliases enrich_GO_terms
#' @importFrom data.table .SD melt.data.table :=
setMethod(
    "show",
    "enrich_GO_terms",
    function(object){

        # Extract table
        Data<-slot(object,"data")

        # Extract pvalues
        Data<-Data[,grep("\\.pvalue",names(Data)),with=FALSE]

        # pvalues threshlod according condition
        p<-unlist(slot(object,"cutoff"))

        # count significant pvalues by condition
        Data<-lapply(seq_len(ncol(Data)),function(x){
            data.table(
                conditions=sub("\\.pvalue","",names(Data)[x]),
                `significant GO terms number`=sum(Data[,x,with=FALSE]<p[x],na.rm=TRUE)
            )
        })
        
        # bind results
        Data<-rbindlist(Data)

        # get topGO information
        obj_summary<-slot(object,"summary")

        # summary according method
        obj_summary<-vapply(names(obj_summary),function(x){

            # subset
            Data<-obj_summary[[x]]

            # topGO summary
            if(slot(object,"method")=="topGO"){

                # topGO summary by element
                elems<-paste(
                    vapply(names(Data),function(y){

                        # summery by element
                        paste(
                            paste("   ",y),
                            "\n       ",
                            paste(
                                paste(
                                    names(Data[[y]]),
                                    sub("\n.+$","",unlist(Data[[y]])),
                                    sep=": "
                                ),
                                collapse="\n        "
                            )
                        )
                    },""),
                    collapse="\n  "
                )

                # summary by element
                return(paste(paste(x,elems,sep ="\n  "),"\n "))
            }

            # fgsea summary
            if(slot(object,"method")=="fgsea"){

                # fgsea summary by element
                elems<-paste(
                    paste(
                        "\n    ",
                        names(
                            Data
                        ),
                        " : ",
                        unlist(
                            Data
                        ),
                        sep=""
                    ),
                    collapse=""
                )
                
                # summary by element
                return(paste(paste(x,elems,"\n",sep =""),collapse="/n"))
            }
        },"")

        # cat some text
        cat("- object class: enrich_GO_terms",
            "\n- ontology: ",slot(object,"ont"),
            "\n- method: ",slot(object,"method"),
            "\n- summary:\n ", obj_summary,
            "- enrichment pvalue cutoff:",
            paste("\n       ",Data$conditions,":",slot(object,"cutoff")[[1]]),
            "\n- enrich GOs (in at least one list): ",nrow(slot(object,"data"))," GO terms of ",nrow(Data)," conditions.",
            paste("\n       ",Data$conditions,":",Data$`significant GO terms number`,"terms"),sep=""
        )
    }
)

#' @title Display an interactive or static heatmap.
#' @description Display a heatmap in interactive or static mode.
#' @importFrom plotly export layout
#' @importFrom grDevices dev.off png
#' @importFrom graphics text
#' @importFrom stats end start
#' @family enrich_GO_terms
#' @family GO_clusters
#' @family visualization
#' @param object a \code{\link{GO_clusters-class}} object from \code{\link{GOterms_heatmap}} or \code{\link{GOclusters_heatmap}}.
#' @param type could be "GOterms" to display GOterms clustering heatmap, or "GOclusters" to display GOclusters heatmap.
#' @param file static png output file name (default to NULL).
#' @param width static image width (default to 1000).
#' @param height static image height (default to 1000).
#' @param plotly_update update plotly html dependencies (default to FALSE).
#' @details
#' This method displays an interactive heatmap (if \code{file}=NULL) from \code{\link{GO_clusters-class}} object for "GOterms" or "GOclusters" type.\cr
#' A static png image could be printed by setting \code{file} argument.\cr
#' Interactive heatmap cannot be displayed between two R versions.
#' Then interactive view (build with previous R version) can be updated to new R version using \code{plotly_update} argument setting to TRUE.
#' @return display or print heatmap.
#' @examples
#' # load data example
#' data(
#'     myGOs,
#'     package="ViSEAGO"
#' )
#' \dontrun{
#' # compute GO terms Semantic Similarity distances
#' myGOs<-ViSEAGO::compute_SS_distances(
#'     myGOs,
#'     distance="Wang"
#' )
#'
#' # build MDS plot for a GO_SS-class distance object
#' ViSEAGO::MDSplot(myGOs)
#'
#' # GOtermsHeatmap with default parameters
#' Wang_clusters_wardD2<-ViSEAGO::GOterms_heatmap(
#'     myGOs,
#'     showIC=TRUE,
#'     showGOlabels=TRUE,
#'     GO.tree=list(
#'         tree=list(
#'             distance="Wang",
#'             aggreg.method="ward.D2",
#'             rotate=NULL
#'         ),
#'         cut=list(
#'             dynamic=list(
#'                 pamStage=TRUE,
#'                 pamRespectsDendro=TRUE,
#'                 deepSplit=2,
#'                 minClusterSize =2
#'             )
#'         )
#'     ),
#'     samples.tree=NULL
#' )
#'
#' # Display GO terms heatmap
#' ViSEAGO::show_heatmap(
#'     Wang_clusters_wardD2,
#'     "GOterms"
#' )
#'
#' # Print GO terms heatmap
#' ViSEAGO::show_heatmap(
#'     Wang_clusters_wardD2,
#'     "GOterms",
#'     "GOterms_heatmap.png"
#' )
#'
#' # compute clusters of GO terms Semantic Similarity distances
#' Wang_clusters_wardD2<-ViSEAGO::compute_SS_distances(
#'     Wang_clusters_wardD2,
#'     distance="BMA"
#' )
#'
#' # GOclusters heatmap
#' Wang_clusters_wardD2<-ViSEAGO::GOclusters_heatmap(
#'     Wang_clusters_wardD2,
#'     tree=list(
#'         distance="BMA",
#'         aggreg.method="ward.D2",
#'         rotate=NULL
#'     )
#' )
#'
#' # Display GO clusters heatmap
#' ViSEAGO::show_heatmap(
#'     Wang_clusters_wardD2,
#'     "GOclusters"
#' )
#'
#' # Print GO clusters heatmap
#' ViSEAGO::show_heatmap(
#'     Wang_clusters_wardD2,
#'     "GOclusters",
#'     "GOclusters_heatmap.png"
#' )
#' }
#' @name show_heatmap
#' @rdname show_heatmap-methods
#' @exportMethod show_heatmap
setGeneric(
    name="show_heatmap",
    def=function(object,type,file=NULL,plotly_update=FALSE,height=1000,width=800) {
        standardGeneric("show_heatmap")
    }
)

#' @rdname show_heatmap-methods
#' @aliases show_heatmap
setMethod(
    "show_heatmap",
    signature(
        object="GO_clusters",
        type="character"
    ),
    definition=function(object,type,file,plotly_update,height,width){

        # check type argument
        type=match.arg(type,c("GOterms","GOclusters"))

        # heatmap according type
        heatmap<-switch(type,

            # GOTermsHeatmap
            GOterms=slot(object,"heatmap")$GOterms,

            # GOclusters_heatmap
            GOclusters=slot(object,"heatmap")$GOclusters
        )

        # redirect plotly html_dependency to local packages
        if(plotly_update==TRUE){

            for(i in seq_len(length(heatmap[["dependencies"]]))){
                heatmap[["dependencies"]][[i]][["src"]][["file"]]<-gsub(
                    "^.+[[:digit:]]\\.[[:digit:]]",
                    .libPaths()[1],
                    heatmap[["dependencies"]][[i]][["src"]][["file"]]
                )
            }
        }

        # return interactive or static heatmap according file
        if(is.null(file)){

            # display heatmap
            heatmap

        }else{

            # static heatmap according type
            heatmap<-switch(
                type,
                GOterms=slot(object,"heatmap")$GOterms_static,
                GOclusters=slot(object,"heatmap")$GOclusters_static
            )

            # plot
            png(file,width=width,height=height)
                print(
                    heatmap
                )
            dev.off()
        }
    }
)

#' @title build GO Semantic Similarity object.
#' @description
#' Compute the Information content (IC) on the given ontology, and
#' create a \code{\link{GO_SS-class}} object required by \code{\link{compute_SS_distances}} method to compute GO semantic similarity
#' between enriched GO terms or groups of terms.
#' @importFrom GO.db GO.db GOMFOFFSPRING GOBPOFFSPRING GOCCOFFSPRING
#' @importFrom topGO inverseList
#' @importFrom AnnotationDbi select keys as.list
#' @family GO_semantic_similarity
#' @param gene2GO a \code{\link{gene2GO-class}} object from \code{\link{annotate}} method.
#' @param enrich_GO_terms a \code{\link{enrich_GO_terms-class}} from \code{\link{merge_enrich_terms}} method.
#' @details This method use \code{\link{annotate}} and \code{\link{merge_enrich_terms}} output objects (see Arguments),
#' and compute the Information content (IC) using the internal code of \code{\link[GOSemSim]{godata}} function from \pkg{GOSemSim} package.
#' @return a \code{\link{GO_SS-class}} object required by \code{\link{compute_SS_distances}}.
#' @references
#' Alexa A, Rahnenfuhrer J, Lengauer T. Improved scoring of functional groups from gene expression data by
#' decorrelating GO graph structure. Bioinformatics 2006; 22:1600-1607.
#'
#' Guangchuang Yu, Fei Li, Yide Qin, Xiaochen Bo, Yibo Wu and Shengqi Wang. GOSemSim: an R package for measuring semantic similarity
#' among GO terms and gene products. Bioinformatics 2010 26(7):976-978.
#'
#' Herve Pages, Marc Carlson, Seth Falcon and Nianhua Li (2017). AnnotationDbi: Annotation Database Interface. R package version 1.38.0.
#' @include gene2GO.R
#' @include enrich_GO_terms.R
#' @include GO_SS.R
#' @examples
#' \dontrun{
#' # initialyse object for compute GO Semantic Similarity
#' myGOs<-ViSEAGO::build_GO_SS(
#'     myGENE2GO,
#'     BP_sResults
#' )
#' }
#' # load data example
#' utils::data(
#'     myGOs,
#'     package="ViSEAGO"
#' )
#' @name build_GO_SS
#' @rdname build_GO_SS-methods
#' @exportMethod build_GO_SS
setGeneric(
    name="build_GO_SS",
    def=function(gene2GO,enrich_GO_terms){
        standardGeneric("build_GO_SS")
})

#' @rdname build_GO_SS-methods
#' @aliases build_GO_SS
setMethod(
    "build_GO_SS",
    signature(
        gene2GO="gene2GO",
        enrich_GO_terms="enrich_GO_terms"
    ),
    definition=function(gene2GO,enrich_GO_terms){

        ## check object class

        if(!is(gene2GO,"gene2GO")){
            stop("object must be a gene2GO class object from ViSEAGO::annotate()")
        }
        if(!is(enrich_GO_terms,"enrich_GO_terms")){
            stop("object must be a enrich_GO_terms class object from ViSEAGO::merge_enrich_terms()")
        }

        ## load data

        # onto match argument
        ont<-slot(enrich_GO_terms,"ont")

        # from GO database
        go<-select(
            GO.db,
            keys=keys(GO.db),
            columns=c("GOID","ONTOLOGY")
        )

        # select a given category (MF,BP,CC)
        goids<-go$GOID[go$ONTOLOGY==ont]

        # count number of genes by term
        gocount<-lengths(
            inverseList(
                slot(gene2GO,ont)
            )
        )

        ## From GOSemsim compute IC content

        # extract gcount term names
        goname<-names(gocount)

        # ensure goterms not appearing in the specific annotation have 0 frequency
        go.diff<-setdiff(goids, goname)
        m<- double(length(go.diff))
        names(m)<-go.diff
        gocount<- as.vector(gocount)
        names(gocount)<- goname
        gocount<-c(gocount, m)

        # select offspings
        Offsprings <- switch(
            ont,
            MF =as.list(GOMFOFFSPRING),
            BP =as.list(GOBPOFFSPRING),
            CC =as.list(GOCCOFFSPRING)
        )

        # add for each term offsprings their counted values
        cnt <- gocount[goids] + vapply(goids, function(i){
            sum(gocount[Offsprings[[i]]], na.rm=TRUE)
        },0)
        names(cnt)<-goids

        # the probabilities of occurrence of GO terms in a specific corpus.
        IC<- -log(cnt/sum(gocount))

        ## return value et S4 object with compatibility with GOSemSim package

        # create object
        new(
            "GO_SS",
            ont=slot(enrich_GO_terms,"ont"),
            enrich_GOs=enrich_GO_terms,
            IC = IC
        )
    }
)

#' @title Display Organism Scientific and common name from taxid.
#' @description  Display Organism Scientific and common name from taxid.
#' @importFrom data.table data.table rbindlist
#' @family genomic_ressource
#' @details This internal function use \href{https://www.ncbi.nlm.nih.gov/books/NBK25500/}{E-utilities} summary to display in
#'  \code{\link[data.table]{data.table}} Organism Scientific name and common name, form a \code{\link[base]{vector}} of taxid.
#' @return a \code{\link[data.table]{data.table}}.
#' @examples
#' # Organism Scientific and common name from taxid
#' Data<-ViSEAGO::taxonomy("9031")
#' @keywords internal
#' @export
taxonomy=function(...){

    # taxonomy ids
    taxid=unique(...)

    # internal function for pattern  extraction
    pattern.extract<-function(query,m){

        # for each query line
        Data=lapply(seq_along(query),function(i){

            # if not empty m match
            if(length(stats::na.omit(m[[i]][1]))>0){

                # extract capture.start argument
                capture=attr(m[[i]],"capture.start")

                # extract corresponding values in query
                capture=substring(
                    query[i],
                    capture,
                    capture+attr(m[[i]],"capture.length")-1
                )

                # convert in data.table
                data.table(t(capture))

            }else{

                # else NA
                NA
            }
        })

        # rbind.data.table
        Data<-rbindlist(Data)

        # add header
        colnames(Data)<-attr(m[[1]],"capture.name")

        # replace "" or \t values by NA
         Data[c("","\t"),"CommonName":="NA",on="CommonName"]

        # return query
        return(Data)
    }

    # core address
    core="https://eutils.ncbi.nlm.nih.gov/entrez/eutils/esummary.fcgi?version=2.0&db=taxonomy"

    # split species per block
    species<-split(
        taxid,
        ceiling(seq_along(taxid) / 500)
    )
    
    # extract
    Data<-lapply(species,function(x){

        # create submission query
        query <-paste(
            core,
            "&id=",
            paste(x,collapse=","),
            sep = ""
        )

        # submit and retrieve
        query=paste(
            scan(
                query,
                what ="",
                sep="\n",
                quiet = TRUE
            ),
            collapse=""
        )

        # parse results
        query<-substring(
            query,
            unlist(gregexpr("<DocumentSummary ",query)),
            unlist(gregexpr("</DocumentSummary>",query))
        )

        # extraction pattern
        pattern=c("<DocumentSummary uid=\"(?<taxid>[[:digit:]]*)\"",
        ".*<ScientificName>(?<ScientificName>.*)</ScientificName>",
        "\t<CommonName>(?<CommonName>.*)</CommonName>.*")

        # find pattern
        m=gregexpr(
            paste(pattern,collapse=""),
            query,
            perl=TRUE
        )

        # extract  results in data.frame and return
        pattern.extract(query,m)
    })
    
    # bind and return
    data.table::rbindlist(Data)
}

#' @title Create topGOdata object for enrichment test with topGO package.
#' @description This method create a \code{\link[topGO]{topGOdata-class}} object required by \pkg{topGO} package in order
#' to perform GO enrichment test.
#' @import topGO
#' @family GO_terms
#' @param geneSel genes of interest.
#' @param allGenes customized background genes.
#' @param geneList logical factor (1: genes of interest, 0: genes background,
#' and gene identifiants in \code{names}) (default value to NULL).
#' @param gene2GO a \code{\link{gene2GO-class}} object created by \code{\link{annotate}} method.
#' @param ont the ontology used is "MF" (Molecuar Function), "BP" (Biological Process), or "CC" (Cellular Component).
#' @param nodeSize the minimum number of genes for each GO term.
#' @details This method is a convenient wrapper building a \code{\link[topGO]{topGOdata-class}} object
#' using a given ontology category (\code{ont} argument) in order to perform GO enrichment test.
#' The complete GO annotation is required (\code{gene2GO} argument) and also
#' the list of genes of interest (\code{geneSel} argument) against the corresponding background (\code{allGenes} argument)
#' separately, or grouped together in a \code{factor} (\code{geneList} argument).
#' @return a  \code{\link[topGO]{topGOdata-class}} object required by \code{runTest} from \pkg{topGO} package.
#' @references
#' Alexa A, Rahnenfuhrer J, Lengauer T. Improved scoring of functional groups from gene expression data by
#' decorrelating GO graph structure. Bioinformatics 2006; 22:1600-1607.
#' @include gene2GO.R
#' @examples
#'  # load genes identifiants (GeneID,ENS...) background (Expressed genes)
#'  background<-scan(
#'   system.file(
#'    "extdata/data/input",
#'    "background_L.txt",
#'    package = "ViSEAGO"
#'   ),
#'   quiet=TRUE,
#'   what=""
#'  )
#'
#'  # load Differentialy Expressed (DE) gene identifiants from files
#'  pregnantvslactateDE<-scan(
#'   system.file(
#'    "extdata/data/input",
#'    "pregnantvslactateDE.txt",
#'    package = "ViSEAGO"
#'  ),
#'   quiet=TRUE,
#'   what=""
#'  )
#'
#' \dontrun{
#' # create topGOdata for BP for each list of DE genes
#' BP_L_pregnantvslactate<-ViSEAGO::create_topGOdata(
#'  geneSel=pregnantvslactateDE,
#'  allGenes=background,
#'  gene2GO=myGENE2GO,
#'  ont="BP",
#'  nodeSize=5
#' )
#' }
#' @name create_topGOdata
#' @rdname create_topGOdata-methods
#' @exportMethod create_topGOdata
setGeneric(
    name="create_topGOdata",
    def=function(geneSel,allGenes,geneList=NULL,gene2GO,ont,nodeSize){
        standardGeneric("create_topGOdata")
    }
)

#' @rdname create_topGOdata-methods
#' @aliases create_topGOdata
setMethod(
    "create_topGOdata",
    signature(
        gene2GO="gene2GO",
        ont="character",
        nodeSize="numeric"
    ),
    definition=function(geneSel,allGenes,geneList,gene2GO,ont,nodeSize){

        ## check list


        # check object class
        if(!is(gene2GO,"gene2GO")){
            stop("object must be a gene2GO class obtained using ViSEAGO::annotate()")
        }

        # check ontology
        ont=match.arg(ont,c("MF","BP","CC"))

        ## build allGenes factor

        if(is.null(geneList)){

            # check duplicate in allGenes
            if(length(allGenes)!=length(unique(allGenes))){

                # warning message
                message("allGenes contain genes redondancy.\nduplicate elements were removed.")

                # remove duplicate
                allGenes=unique(allGenes)
            }

            # check duplicate in geneSel
            if(length(geneSel)!=length(unique(geneSel))){

                # warning message
                message("geneSel contain genes redondancy.\nduplicate elements were removed.")

                # remove duplicate
                geneSel=unique(geneSel)
            }

            # create geneList object
            AllGenes<-rep(0,length(allGenes))

            # assign genes_ref names to geneList
            attr(AllGenes,"names")<-allGenes

            # assign value 1 for DE genes in geneList
            AllGenes[attr(AllGenes,"names")%in%geneSel]<-1

            # convert geneList in factor (used by topGO)
            AllGenes<-factor(AllGenes)

        }else{

            # check duplicate in geneSel
            if(length(names(geneList))!=length(unique(names(geneList)))){

                # warning message
                stop("geneList contain genes redondancy.")
            }

            # rename geneList
            AllGenes<-geneList
        }

        # load topGO
        require("topGO")

        # create GOdata
        new(
            "topGOdata",
            description=paste(
                slot(gene2GO,"db"),
                slot(gene2GO,"organism"),
                slot(gene2GO,"stamp")
            ),
            ontology =ont,
            allGenes = AllGenes,
            annot = annFUN.gene2GO,
            nodeSize =nodeSize,
            gene2GO =slot(gene2GO,ont)
        )
    }
)

---
title: "Transcriptomic Differential analysis of mouse mammary gland RNA-Seq"
author:
- name: Aurelien Brionne
  affiliation: BOA, Institut National de la Recherche Agronomique (INRA), Universitée de Tours, 37380 Nouzilly, France
- name: Amelie Juanchich
  affiliation: BOA, Institut National de la Recherche Agronomique (INRA), Universitée de Tours, 37380 Nouzilly, France
- name: Christelle Hennequet-Antier
  affiliation: BOA, Institut National de la Recherche Agronomique (INRA), Universitée de Tours, 37380 Nouzilly, France
date: "`r format(Sys.time(), '%d %B, %Y')`"
output:
  BiocStyle::html_document:
    highlight: tango
vignette: >
  %\VignetteIndexEntry{edgeR_mouse_mammary_gland}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
bibliography: "`r system.file('extdata/','bibliography.bib',package='ViSEAGO')`"
csl: "`r system.file('extdata/','bmc-genomics.csl',package='ViSEAGO')`"
---

```{r setup,include=FALSE,cache=F}
###################
# knitr document options
knitr::opts_chunk$set(eval=TRUE,fig.pos = 'H',
fig.width=9,message=FALSE,comment=NA,warning=FALSE)
```

```{r LoadData}
###################
# library
library(edgeR)
library(limma)
library(graphics)

###################
# Get Raw Count data
###################

  ###################
  # temp file
  temp<-paste(
    tempfile(),
    "gz",
    sep="."
  )

  ###################
  # load the file
  download.file(
    "ftp://ftp.ncbi.nlm.nih.gov/geo/series/GSE60nnn/GSE60450/suppl/GSE60450_Lactation-GenewiseCounts.txt.gz",
    destfile =temp,
    quiet=TRUE
  )

  ###################
  # unzip
  gunzip(temp)

  ###################
  # load raw data
  rawdata<-data.table::fread(sub("\\.gz","",temp))

  ###################
  # genecount
  geneCount <-as.matrix(rawdata[,-c(1,2),with=FALSE])

  ###################
  # add rownames
  rownames(geneCount)<-rawdata$EntrezGeneID

  ###################
  # design and factors
  targets<-data.table::data.table(
    File=c(
      "GSM1480297_MCL1-DG_BC2CTUACXX_ACTTGA_L002_R1.txt",
      "GSM1480298_MCL1-DH_BC2CTUACXX_CAGATC_L002_R1.txt",
      "GSM1480299_MCL1-DI_BC2CTUACXX_ACAGTG_L002_R1.txt",
      "GSM1480300_MCL1-DJ_BC2CTUACXX_CGATGT_L002_R1.txt",
      "GSM1480301_MCL1-DK_BC2CTUACXX_TTAGGC_L002_R1.txt",
      "GSM1480302_MCL1-DL_BC2CTUACXX_ATCACG_L002_R1.txt",
      "GSM1480291_MCL1-LA_BC2CTUACXX_GATCAG_L001_R1.txt",
      "GSM1480292_MCL1-LB_BC2CTUACXX_TGACCA_L001_R1.txt",
      "GSM1480293_MCL1-LC_BC2CTUACXX_GCCAAT_L001_R1.txt",
      "GSM1480294_MCL1-LD_BC2CTUACXX_GGCTAC_L001_R1.txt",
      "GSM1480295_MCL1-LE_BC2CTUACXX_TAGCTT_L001_R1.txt",
      "GSM1480296_MCL1-LF_BC2CTUACXX_CTTGTA_L001_R1.txt"
    ),
    Sample=c(
      "GSM1480297",
      "GSM1480298",
      "GSM1480299",
      "GSM1480300",
      "GSM1480301",
      "GSM1480302",
      "GSM1480291",
      "GSM1480292",
      "GSM1480293",
      "GSM1480294",
      "GSM1480295",
      "GSM1480296"
    ),
    CellType=rep(c("Basal","Luminal"),each=6),
    Status=rep(rep(c("virgin","pregnant","lactate"),2),each=2)
  )
```
</br>
```{r CountAnnot}
###################
# design and factors
group <- factor(
  paste0(
    targets$CellType, ".", targets$Status)
)

###################
# create DGElist
y <-DGEList(geneCount, group=group)
```
</br>
```{r FiltNorm}
###################
# minima count filtering
keep <- rowSums(cpm(y) > 0.5) >= 2

###################
# filter y
y <- y[keep, , keep.lib.sizes=FALSE]

###################
# Normalization
y <-calcNormFactors(y)
```
</br>
```{r MDSplot}
###################
# graphic parameters
colors<-rep(c("blue", "darkgreen", "red"),2)
points<-c(0,1,2,15,16,17)

###################
# MDSplot
plotMDS(
  y,
  col=colors[group],
  pch=points[group]
)

###################
# add MDSplot legend
legend(
  "topleft",
  legend=levels(group),
  pch=points,
  col=colors,
  ncol=2
)
```
</br>
```{r Design}
###################
# model
design <-stats::model.matrix(~ 0 + group)

###################
# renames
colnames(design) <- levels(group)
```
</br>
```{r Dispersion}
###################
# Estimate Common, Trended and Tagwise Negative Binomial dispersions by weighted likelihood empirical Bayes robustified against outliers
y <- estimateDisp(y, design, robust=TRUE)
y$common.dispersion

###################
# Plot Biological Coefficient of Variation
plotBCV(y)

###################
# Genewise Negative Binomial Generalized Linear Models with Quasi-likelihood Tests
fit <-glmQLFit(y, design, robust=TRUE)

###################
# Plot the quasi-likelihood dispersion
plotQLDisp(fit)
```
</br>
```{r Differential}
###################
# Construct Matrix of Custom Contrasts
con<-makeContrasts(
  L.PvsL = Luminal.pregnant - Luminal.lactate,
  L.VvsL = Luminal.virgin - Luminal.lactate,
  L.VvsP = Luminal.virgin - Luminal.pregnant,
  levels=design
)

###################
# Genewise Negative Binomial Generalized Linear Models with Quasi-likelihood Tests
qlf2<-glmQLFTest(fit, contrast=con)

###################
# Construct Matrix of Custom Contrasts
con.L.pregnantvslactate <- makeContrasts(
  L.PvsL = Luminal.pregnant - Luminal.lactate,
  levels=design
)

con.L.virginvslactate <-makeContrasts(
  L.VvsL = Luminal.virgin - Luminal.lactate,
  levels=design
)

con.L.virginvspregnant <-makeContrasts(
  L.VvsP = Luminal.virgin - Luminal.pregnant,
  levels=design
)

###################
# Genewise Negative Binomial Generalized Linear Models with Quasi-likelihood Tests
res<-lapply(ls(pattern="con."),function(x){
  
  ###################
  # Genewise Negative Binomial Generalized Linear Models with Quasi-likelihood Tests
  qlf<-glmQLFTest(
    fit,
    contrast=get(x)
  )
  
  ###################
  # Table of the Top Differentially Expressed Genes/Tags
   topTags(
     qlf,
     n=nrow(y$counts),
     adjust.method="BH",
     sort.by="none"
    )$table
})
names(res)<-gsub("^con\\.","",ls(pattern="con."))

###################
# exporting results of the differential analysis
write(
  rownames(res$L.pregnantvslactate),
  file="background_L.txt"
)

write(
  rownames(res$L.pregnantvslactate)[res$L.pregnantvslactate$FDR<0.05],
  file="pregnantvslactateDE.txt"
)

write(
  rownames(res$L.virginvslactate)[res$L.virginvslactate$FDR<0.05],
  file="virginvslactateDE.txt"
)

write(
  rownames(res$L.virginvspregnant)[res$L.virginvspregnant$FDR<0.05],
  file="virginvspregnantDE.txt"
)
```

---
title: "Functional analysis of mouse mammary gland RNA-Seq"
author:
- name: Aurelien Brionne
  affiliation: "Institut national de recherche pour l'agriculture, l'alimentation et l'environnement (INRAE)"
- name: Amelie Juanchich
  affiliation: "Institut national de recherche pour l'agriculture, l'alimentation et l'environnement (INRAE)"
- name: Christelle Hennequet-Antier
  affiliation: "Institut national de recherche pour l'agriculture, l'alimentation et l'environnement (INRAE)"
date: "`r format(Sys.time(), '%d %B, %Y')`"
output:
  BiocStyle::html_document:
    highlight: tango
vignette: >
  %\VignetteIndexEntry{2: mouse_bionconductor}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
bibliography: "`r system.file('extdata/','bibliography.bib',package='ViSEAGO')`"
csl: "`r system.file('extdata/','bmc-genomics.csl',package='ViSEAGO')`"
---

```{r setup,include=FALSE}
# load ViSEAGO and mouse db package
library(ViSEAGO)

# knitr document options
knitr::opts_chunk$set(
    eval=FALSE,echo=TRUE,fig.pos = 'H',
    fig.width=6,message=FALSE,comment=NA,warning=FALSE
)
```

# Introduction{-}

This study explores expression profiles of basal stem-cell enriched cells (B) and committed luminal cells (L) of the mammary gland of virgin, pregnant and lactating mice @pmid25730472. Six groups are compared corresponding to a combination of cell type and mouse status. Each group contains two biological replicates. Sequences and counts datasets are publicly available from the Gene Expression Omnibus (GEO) with the serie accession number GSE60450. The RNA-Seq data is analysed by `r BiocStyle::Biocpkg("edgeR")` development team @edgeR.

As described in edgeR user guide, we tested for significant differential expression in gene, using the QL F-test @pmid23104842. Here, we focused in gene expression analysis in only one type of cell, i.e. luminal cells for the three developmental status (virgin, pregnant and lactating mice). Among the 15,804 expressed genes, we obtained 7,302 significantly differentially expressed (DE) genes for the comparison virgin versus pregnant, 7,699 for the comparison pregnant versus lactate, and 9,583 for the comparison virgin versus lactate, with Benjamini-Hochberg correction to control the false discovery rate at 5%.

# Data{-}

Vignette build convenience (for less build time and package size) need that data were pre-calculated (provided by the package), and that illustrations were not interactive.

```{r vignette_data_used,eval=TRUE}
# load vignette data
data(
    myGOs,
    package="ViSEAGO"
)
```

# Genes of interest

We load examples files from `r BiocStyle::Biocpkg("ViSEAGO")` package using `system.file` from the locally installed package. We read the gene identifiers for the background (= expressed genes) file and the three lists of DE genes of the study. Here, gene identifiers are GeneID from EntrezGene database.

```{r geneList_input,eval=TRUE}
# load genes identifiants (GeneID,ENS...) background (expressed genes) 
background<-scan(
    system.file(
        "extdata/data/input",
        "background_L.txt",
        package = "ViSEAGO"
    ),
    quiet=TRUE,
    what=""
)

# load Differentialy Expressed (DE) gene identifiants from lists
PregnantvsLactateDE<-scan(
    system.file(
        "extdata/data/input",
        "pregnantvslactateDE.txt",
        package = "ViSEAGO"
    ),
    quiet=TRUE,
    what=""
)

VirginvsLactateDE<-scan(
    system.file(
        "extdata/data/input",
        "virginvslactateDE.txt",
        package = "ViSEAGO"
    ),
    quiet=TRUE,
    what=""
)

VirginvsPregnantDE<-scan(
    system.file(
        "extdata/data/input",
        "virginvspregnantDE.txt",
        package = "ViSEAGO"
    ),
    quiet=TRUE,
    what=""
)
```

Here, we display the first 6 GeneID from the *PregnantvsLactateDE* list.

```{r geneList_input-head,echo=FALSE}
# show the ten first lines of genes_DE (same as genes_ref)
head(PregnantvsLactateDE)
```

# GO annotation of genes

In this study, we build a `myGENE2GO` object using the Bioconductor `r BiocStyle::Biocpkg("org.Mm.eg.db")` database package for the mouse species. This object contains all available GO annotations for categories Molecular Function (MF), Biological Process (BP), and Cellular Component (CC).

<u>NB</u>: Don't forget to check if the last current annotation database version is installed in your R session! See `ViSEAGO::available_organisms(Bioconductor)`.

```{r Genomic-ressources}
# connect to Bioconductor
Bioconductor<-ViSEAGO::Bioconductor2GO()

# load GO annotations from Bioconductor
myGENE2GO<-ViSEAGO::annotate(
    "org.Mm.eg.db",
    Bioconductor
)
```
</br> 
```{r Genomic-ressources_show}
# display summary
myGENE2GO
```
</br> 
```{r Genomic-ressources_display,echo=FALSE,eval=TRUE}
cat(
"- object class: gene2GO
- database: Bioconductor
- stamp/version: 2019-Jul10
- organism id: org.Mm.eg.db

GO annotations:
- Molecular Function (MF): 22707 annotated genes with 91986 terms (4121 unique terms)
- Biological Process (BP): 23210 annotated genes with 164825 terms (12224 unique terms)
- Cellular Component (CC): 23436 annotated genes with 107852 terms (1723 unique terms)"
)
```

# Functional GO enrichment

## GO enrichment tests

We perform a functional Gene Ontology (GO) enrichment analysis from differentially expressed (DE) genes of luminal cells in the mammary gland. The enriched **Biological process** (BP) are obtained using a Fisher's exact test with `elim` algorithm developped in `r BiocStyle::Biocpkg("topGO")` package.

First, we create three `topGOdata` objects, using `ViSEAGO::create_topGOdata` method, corresponding to the three DE genes lists for the comparison virgin versus pregnant, pregnant versus lactate, and virgin versus lactate. The gene background corresponding to expressed genes in luminal cells of the mammary gland and GO annotations are also provided.

```{r Enrichment_data}
# create topGOdata for BP for each list of DE genes
BP_PregnantvsLactate<-ViSEAGO::create_topGOdata(
    geneSel=PregnantvsLactateDE,
    allGenes=background,
    gene2GO=myGENE2GO, 
    ont="BP",
    nodeSize=5
)

BP_VirginvsLactate<-ViSEAGO::create_topGOdata(
    geneSel=VirginvsLactateDE,
    allGenes=background,
    gene2GO=myGENE2GO,
    ont="BP",
    nodeSize=5
)

BP_VirginvsPregnant<-ViSEAGO::create_topGOdata(
    geneSel=VirginvsPregnantDE,
    allGenes=background,
    gene2GO=myGENE2GO,
    ont="BP",
    nodeSize=5
)
```

Now, we perform the GO enrichment tests for BP category with Fisher's exact test and *elim* algorithm using `topGO::runTest` method.

<u>NB</u>: p-values of enriched GO terms are not adjusted and considered significant if below 0.01.

```{r Enrichment_data_tests}
# perform topGO tests
elim_BP_PregnantvsLactate<-topGO::runTest(
    BP_PregnantvsLactate,
    algorithm ="elim",
    statistic = "fisher",
    cutOff=0.01
)

elim_BP_VirginvsLactate<-topGO::runTest(
    BP_VirginvsLactate,
    algorithm ="elim",
    statistic = "fisher",
    cutOff=0.01
)

elim_BP_VirginvsPregnant<-topGO::runTest(
    BP_VirginvsPregnant,
    algorithm ="elim",
    statistic = "fisher",
    cutOff=0.01
)
```

## Combine enriched GO terms

We combine the results of the three enrichment tests into an object using `ViSEAGO::merge_enrich_terms` method. A table of enriched GO terms in at least one comparison is displayed in interactive mode, or printed in a file using `ViSEAGO::show_table` method.
The printed table contains for each enriched GO terms, additional columns including the list of significant genes and frequency (ratio between the number of significant genes and number of background genes in a specific GO tag) evaluated by comparison. 

```{r Enrichment_merge}
# merge topGO results
BP_sResults<-ViSEAGO::merge_enrich_terms(
    cutoff=0.01,
    Input=list(
        PregnantvsLactate=c(
            "BP_PregnantvsLactate",
            "elim_BP_PregnantvsLactate"
        ),
        VirginvsLactate=c(
            "BP_VirginvsLactate",
            "elim_BP_VirginvsLactate"
        ),
        VirginvsPregnant=c(
            "BP_VirginvsPregnant",
            "elim_BP_VirginvsPregnant"
        )
    )
)
```
</br>
```{r Enrichment_merge_show}
# display a summary
BP_sResults
```
</br>
```{r Enrichment_merge_display,echo=FALSE,eval=TRUE}
cat(
"- object class: enrich_GO_terms
- ontology: BP
- method: topGO
- summary:PregnantvsLactate
      BP_PregnantvsLactate 
        description: Bioconductor org.Mm.eg.db 2019-Jul10
        available_genes: 15804
        available_genes_significant: 7699
        feasible_genes: 14091
        feasible_genes_significant: 7044
        genes_nodeSize: 5
        nodes_number: 8463
        edges_number: 19543
      elim_BP_PregnantvsLactate 
        description: Bioconductor org.Mm.eg.db 2019-Jul10 
        test_name: fisher p<0.01
        algorithm_name: elim
        GO_scored: 8463
        GO_significant: 199
        feasible_genes: 14091
        feasible_genes_significant: 7044
        genes_nodeSize: 5
        Nontrivial_nodes: 8433 
 VirginvsLactate
      BP_VirginvsLactate 
        description: Bioconductor org.Mm.eg.db 2019-Jul10
        available_genes: 15804
        available_genes_significant: 9583
        feasible_genes: 14091
        feasible_genes_significant: 8734
        genes_nodeSize: 5
        nodes_number: 8463
        edges_number: 19543
      elim_BP_VirginvsLactate 
        description: Bioconductor org.Mm.eg.db 2019-Jul10 
        test_name: fisher p<0.01
        algorithm_name: elim
        GO_scored: 8463
        GO_significant: 152
        feasible_genes: 14091
        feasible_genes_significant: 8734
        genes_nodeSize: 5
        Nontrivial_nodes: 8457 
 VirginvsPregnant
      BP_VirginvsPregnant 
        description: Bioconductor org.Mm.eg.db 2019-Jul10
        available_genes: 15804
        available_genes_significant: 7302
        feasible_genes: 14091
        feasible_genes_significant: 6733
        genes_nodeSize: 5
        nodes_number: 8463
        edges_number: 19543
      elim_BP_VirginvsPregnant 
        description: Bioconductor org.Mm.eg.db 2019-Jul10 
        test_name: fisher p<0.01
        algorithm_name: elim
        GO_scored: 8463
        GO_significant: 243
        feasible_genes: 14091
        feasible_genes_significant: 6733
        genes_nodeSize: 5
        Nontrivial_nodes: 8413 
 
- enrichment pvalue cutoff:
        PregnantvsLactate : 0.01
        VirginvsLactate : 0.01
        VirginvsPregnant : 0.01
- enrich GOs (in at least one list): 521 GO terms of 3 conditions.
        PregnantvsLactate : 199 terms
        VirginvsLactate : 152 terms
        VirginvsPregnant : 243 terms"
)
```
</br>
```{r Enrichment_merge_table}
# show table in interactive mode
ViSEAGO::show_table(BP_sResults)
```

<img src=`r system.file("extdata/data/output","bioconductor_table1.png",package="ViSEAGO")` alt="bioconductor_table1" style="width:100%;height:100%;"/>

## Graphs of GO enrichment tests

Several graphs summarize important features. An interactive barchart showing the number of GO terms enriched or not in each comparison, using `ViSEAGO::GOcount` method. Intersections of lists of enriched GO terms between comparisons are displayed in an Upset plot using `ViSEAGO::Upset` method.

```{r Enrichment_merge_count}
# barchart of significant (or not) GO terms by comparison
ViSEAGO::GOcount(BP_sResults)
```

<img src=`r system.file("extdata/data/output","gocount.png",package="ViSEAGO")` alt="gocount" style="width:100%;height:100%;"/>

```{r Enrichment_merge_interactions}
# display intersections
ViSEAGO::Upset(
    BP_sResults,
    file="upset.xls"
)
```

<img src=`r system.file("extdata/data/output","upset.png",package="ViSEAGO")` alt="upset" style="width:100%;height:100%;"/>

# GO terms Semantic Similarity

GO annotations of genes created at a previous step and enriched GO terms are combined using `ViSEAGO::build_GO_SS` method. The Semantic Similarity (SS) between enriched GO terms are calculated using `ViSEAGO::compute_SS_distances` method which is a wrapper of functions implemented in the `r BiocStyle::Biocpkg("GOSemSim")` package @pmid20179076. Here, we choose *Wang* method based on the topology of GO graph structure. The built object `myGOs` contains all informations of enriched GO terms and the SS distances between them.

```{r SS_build}
# create GO_SS-class object
myGOs<-ViSEAGO::build_GO_SS(
    gene2GO=myGENE2GO,
    enrich_GO_terms=BP_sResults
)
```
</br>
```{r SS_compute}
# compute Semantic Similarity (SS)
myGOs<-ViSEAGO::compute_SS_distances(
    myGOs,
    distance="Wang"
)
```
</br>
```{r SS_build_compute_show}
# display a summary
myGOs
```
</br>
```{r SS_build_compute_display,echo=FALSE,eval=TRUE}
cat(
"- object class: GO_SS
- ontology: BP
- method: topGO
- summary:
PregnantvsLactate
      BP_PregnantvsLactate 
        description: Bioconductor org.Mm.eg.db 2019-Jul10
        available_genes: 15804
        available_genes_significant: 7699
        feasible_genes: 14091
        feasible_genes_significant: 7044
        genes_nodeSize: 5
        nodes_number: 8463
        edges_number: 19543
      elim_BP_PregnantvsLactate 
        description: Bioconductor org.Mm.eg.db 2019-Jul10 
        test_name: fisher p<0.01
        algorithm_name: elim
        GO_scored: 8463
        GO_significant: 199
        feasible_genes: 14091
        feasible_genes_significant: 7044
        genes_nodeSize: 5
        Nontrivial_nodes: 8433 
 VirginvsLactate
      BP_VirginvsLactate 
        description: Bioconductor org.Mm.eg.db 2019-Jul10
        available_genes: 15804
        available_genes_significant: 9583
        feasible_genes: 14091
        feasible_genes_significant: 8734
        genes_nodeSize: 5
        nodes_number: 8463
        edges_number: 19543
      elim_BP_VirginvsLactate 
        description: Bioconductor org.Mm.eg.db 2019-Jul10 
        test_name: fisher p<0.01
        algorithm_name: elim
        GO_scored: 8463
        GO_significant: 152
        feasible_genes: 14091
        feasible_genes_significant: 8734
        genes_nodeSize: 5
        Nontrivial_nodes: 8457 
 VirginvsPregnant
      BP_VirginvsPregnant 
        description: Bioconductor org.Mm.eg.db 2019-Jul10
        available_genes: 15804
        available_genes_significant: 7302
        feasible_genes: 14091
        feasible_genes_significant: 6733
        genes_nodeSize: 5
        nodes_number: 8463
        edges_number: 19543
      elim_BP_VirginvsPregnant 
        description: Bioconductor org.Mm.eg.db 2019-Jul10 
        test_name: fisher p<0.01
        algorithm_name: elim
        GO_scored: 8463
        GO_significant: 243
        feasible_genes: 14091
        feasible_genes_significant: 6733
        genes_nodeSize: 5
        Nontrivial_nodes: 8413 
 - enrichment pvalue cutoff:
        PregnantvsLactate : 0.01
        VirginvsLactate : 0.01
        VirginvsPregnant : 0.01
- enrich GOs (in at least one list): 521 GO terms of 3 conditions.
        PregnantvsLactate : 199 terms
        VirginvsLactate : 152 terms
        VirginvsPregnant : 243 terms
- terms distances:  Wang"
)
```

# Visualization and interpretation of enriched GO terms

## Multi Dimensional Scaling of GO terms - A preview

A Multi Dimensional Scale (MDS) plot with `ViSEAGO::MDSplot` method provides a representation of distances among a set of enriched GO terms on the two first dimensions. Some patterns could appear at this time and could be investigated in the interactive mode. The plot could also be printed in a png file.

```{r SS_terms_mdsplot}
# MDSplot
ViSEAGO::MDSplot(myGOs)
```

<img src=`r system.file("extdata/data/output","bioconductor_mds1.png",package="ViSEAGO")` alt="mds1" style="width:100%;height:100%;"/>

## Clustering heatmap of GO terms

To fully explore the results of this functional analysis, a hierarchical clustering using `ViSEAGO::GOterms_heatmap` is performed based on `Wang` SS distance between enriched GO terms and `ward.D2` aggregation criteria.

Clusters of enriched GO terms are obtained by cutting branches off the dendrogram. Here, we choose a dynamic branch cutting method based on the shape of clusters using `r BiocStyle::CRANpkg("dynamicTreeCut")` [@pmid18024473; @dynamicTreeCut]. Here, we use parameters to detect small clusters in agreement with the dendrogram structure.

Enriched GO terms are ranked in a dendrogram and colored depending on their cluster assignation. Additional illustrations are displayed: the GO description of terms (trimmed if more than 50 characters), a heatmap of -log10(pvalue) of enrichment test for each comparison, and optionally the Information Content (IC). The IC of a GO term is computed by the negative log probability of the term occurring in the GO annotations database of the studied species. A rarely used term contains a greater amount of IC. The illustration is displayed with `ViSEAGO::show_heatmap` method.

```{r SS_Wang-wardD2}
# Create GOterms heatmap
Wang_clusters_wardD2<-ViSEAGO::GOterms_heatmap(
    myGOs,
    showIC=FALSE,
    showGOlabels =FALSE,
    GO.tree=list(
        tree=list(
            distance="Wang",
            aggreg.method="ward.D2"
        ),
        cut=list(
            dynamic=list(
                pamStage=TRUE,
                pamRespectsDendro=TRUE,
                deepSplit=2,
                minClusterSize =2
            )
        )
    ),
    samples.tree=NULL
)
```
</br>
```{r SS_Wang-wardD2_heatmap_display} 
# display the heatmap
ViSEAGO::show_heatmap(
    Wang_clusters_wardD2,
    "GOterms"
)
```

<img src=`r system.file("extdata/data/output","bioconductor_GOtermsheatmap.png",package="ViSEAGO")` alt="GOtermsheatmap"  style="width:100%;height:100%;"/>

A table of enriched GO terms in at least one of the comparison is displayed in interactive mode, or printed in a file using `ViSEAGO::show_table` method. The columns GO cluster number and IC value are added to the previous table of enriched terms table. 
The printed table contains for each enriched GO terms, additional columns including the list of significant genes and frequency (ratio between the number of significant genes and number of background genes in a specific GO tag) evaluated by comparison. 

```{r SS_Wang-ward.D2_table}
# display table
ViSEAGO::show_table(
    Wang_clusters_wardD2
)
```

<img src=`r system.file("extdata/data/output","bioconductor_table2.png",package="ViSEAGO")` alt="bioconductor_table2" style="width:100%;height:100%;"/>

<u>NB:</u> For this vignette, this illustration is not interactive.

## Multi Dimensional Scaling of GO terms

We also display a colored Multi Dimensional Scale (MDS) plot showing the overlay of GO terms clusters from `Wang_clusters_wardD2` object with `ViSEAGO::MDSplot` method. It is a way to check the coherence of GO terms clusters on the MDS plot.

```{r SS_Wang-ward.D2_mdsplot}
# colored MDSplot
ViSEAGO::MDSplot(
    Wang_clusters_wardD2,
    "GOterms"
)
```

<img src=`r system.file("extdata/data/output","bioconductor_mds2.png",package="ViSEAGO")` alt="mds2" style="width:100%;height:100%;"/>

# Visualization and interpretation of GO clusters

This part is dedicated to explore relationships between clusters of GO terms defined by the last hierarchical clustering with `ViSEAGO::GOterms_heatmap` method. This analysis should be conducted if the number of clusters of GO terms is large enough and therefore difficult to investigate.

## Compute semantic similarity between GO clusters

A Semantic Similarity (SS) between clusters of GO terms, previously defined, is calculated using `ViSEAGO::compute_SS_distances` method. Here, we choose the Best-Match Average, also known as *BMA* method implemented in the `r BiocStyle::Biocpkg("GOSemSim")` package @pmid20179076. It calculates the average of all maximum similarities between two clusters of GO terms. 

A colored Multi Dimensional Scale (MDS) plot with `ViSEAGO::MDSplot` method provides a representation of distances between the clusters of GO terms. Each circle represents a cluster of GO terms and its size depends on the number of GO terms that it contains. Clusters of GO terms that are close should share a functional coherence.

```{r SS_Wang-wardD2_groups} 
# calculate semantic similarites between clusters of GO terms
Wang_clusters_wardD2<-ViSEAGO::compute_SS_distances(
    Wang_clusters_wardD2,
    distance="BMA"
)
```
</br>
```{r SS_Wang-ward.D2_groups_mdsplot}
# MDSplot
ViSEAGO::MDSplot(
    Wang_clusters_wardD2,
    "GOclusters"
)
```

<img src=`r system.file("extdata/data/output","bioconductor_mds3.png",package="ViSEAGO")` alt="mds3" style="width:100%;height:100%;"/>

## GO clusters semantic similarities heatmap

A new hierarchical clustering using `ViSEAGO::GOclusters_heatmap` is performed based on the `BMA` SS distance between the clusters of GO terms, previously computed, and the `ward.D2` aggregation criteria.

Clusters of GO terms are ranked in a dendrogram and colored depending on their cluster assignation. Additional illustrations are displayed: the definition of the cluster of GO terms corresponds to the first common GO term ancestor followed by the cluster label in brackets, and the heatmap of GO terms count in the corresponding cluster. The illustration is displayed with `ViSEAGO::show_heatmap` method.

```{r SS_Wang-wardD2_groups_heatmap}
# GOclusters heatmap
Wang_clusters_wardD2<-ViSEAGO::GOclusters_heatmap(
    Wang_clusters_wardD2,
    tree=list(
        distance="BMA",
        aggreg.method="ward.D2"
    )
)
```
</br>
```{r SS_Wang-ward.D2_groups_heatmap_display}
# display the heatmap
ViSEAGO::show_heatmap(
    Wang_clusters_wardD2,
    "GOclusters"
)
```

<img src=`r system.file("extdata/data/output","bioconductor_BMA.png",package="ViSEAGO")` alt="BMA" style="width:100%;height:100%;"/>

An history of the enrichment functional analysis is reported.

```{r SS_Wang-wardD2_groups_show}
# display a summary
Wang_clusters_wardD2
```
</br>
```{r SS_Wang-wardD2_groups_display,echo=FALSE,eval=TRUE}
cat(
"- object class: GO_clusters
- ontology: BP
- method: topGO
- summary:
PregnantvsLactate
      BP_PregnantvsLactate 
        description: Bioconductor org.Mm.eg.db 2019-Jul10
        available_genes: 15804
        available_genes_significant: 7699
        feasible_genes: 14091
        feasible_genes_significant: 7044
        genes_nodeSize: 5
        nodes_number: 8463
        edges_number: 19543
      elim_BP_PregnantvsLactate 
        description: Bioconductor org.Mm.eg.db 2019-Jul10 
        test_name: fisher p<0.01
        algorithm_name: elim
        GO_scored: 8463
        GO_significant: 199
        feasible_genes: 14091
        feasible_genes_significant: 7044
        genes_nodeSize: 5
        Nontrivial_nodes: 8433 
 VirginvsLactate
      BP_VirginvsLactate 
        description: Bioconductor org.Mm.eg.db 2019-Jul10
        available_genes: 15804
        available_genes_significant: 9583
        feasible_genes: 14091
        feasible_genes_significant: 8734
        genes_nodeSize: 5
        nodes_number: 8463
        edges_number: 19543
      elim_BP_VirginvsLactate 
        description: Bioconductor org.Mm.eg.db 2019-Jul10 
        test_name: fisher p<0.01
        algorithm_name: elim
        GO_scored: 8463
        GO_significant: 152
        feasible_genes: 14091
        feasible_genes_significant: 8734
        genes_nodeSize: 5
        Nontrivial_nodes: 8457 
 VirginvsPregnant
      BP_VirginvsPregnant 
        description: Bioconductor org.Mm.eg.db 2019-Jul10
        available_genes: 15804
        available_genes_significant: 7302
        feasible_genes: 14091
        feasible_genes_significant: 6733
        genes_nodeSize: 5
        nodes_number: 8463
        edges_number: 19543
      elim_BP_VirginvsPregnant 
        description: Bioconductor org.Mm.eg.db 2019-Jul10 
        test_name: fisher p<0.01
        algorithm_name: elim
        GO_scored: 8463
        GO_significant: 243
        feasible_genes: 14091
        feasible_genes_significant: 6733
        genes_nodeSize: 5
        Nontrivial_nodes: 8413 
 - enrichment pvalue cutoff:
        PregnantvsLactate : 0.01
        VirginvsLactate : 0.01
        VirginvsPregnant : 0.01
- enrich GOs (in at least one list): 521 GO terms of 3 conditions.
        PregnantvsLactate : 199 terms
        VirginvsLactate : 152 terms
        VirginvsPregnant : 243 terms
- terms distances:  Wang
- clusters distances: BMA
- Heatmap:
          * GOterms: TRUE
                    - GO.tree:
                              tree.distance: Wang
                              tree.aggreg.method: ward.D2
                              cut.dynamic.pamStage: TRUE
                              cut.dynamic.pamRespectsDendro: TRUE
                              cut.dynamic.deepSplit: 2
                              cut.dynamic.minClusterSize: 2
                              number of clusters: 62
                              clusters min size: 2
                              clusters mean size: 8
                              clusters max size: 32
                   - sample.tree: FALSE
          * GOclusters: TRUE
                       - tree:
                              distance: BMA
                              aggreg.method: ward.D2"
)
```

# Conclusion

A functional Gene Ontology (GO) enrichment analysis was performed using `r BiocStyle::Biocpkg("ViSEAGO")` package, from differentially expressed (DE) genes in luminal cells of the mouse mammary gland. It facilitates the interpretation of biological processes involved between these three comparisons of virgin, pregnant and lactating mice. It provides both a synthetic and detailed view using interactive functionalities respecting the GO graph structure and ensuring functional coherence.

# Information session{-}

```{r session,eval=TRUE,echo=FALSE} 
sessionInfo()
```

# References{-}

---
title: "An overview of ViSEAGO: Visualisation, Semantic similarity, Enrichment Analysis of Gene Ontology."
author:
- name: Aurelien Brionne
  affiliation: "Institut national de recherche pour l'agriculture, l'alimentation et l'environnement (INRAE)"
- name: Amelie Juanchich
  affiliation: "Institut national de recherche pour l'agriculture, l'alimentation et l'environnement (INRAE)"
- name: Christelle Hennequet-Antier
  affiliation: "Institut national de recherche pour l'agriculture, l'alimentation et l'environnement (INRAE)"
date: "`r format(Sys.time(), '%d %B, %Y')`"
output:
  BiocStyle::html_document:
    highlight: tango
vignette: >
  %\VignetteIndexEntry{1: ViSEAGO}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
bibliography: "`r system.file('extdata/','bibliography.bib',package='ViSEAGO')`"
csl: "`r system.file('extdata/','bmc-genomics.csl',package='ViSEAGO')`"
---

```{r setup,include=FALSE}
# koad ViSEAGO
library(ViSEAGO)

# knitr document options
knitr::opts_chunk$set(
  eval=FALSE,echo=TRUE,fig.pos = 'H',
  fig.width=8,message=FALSE,comment=NA,warning=FALSE
)
```

# Introduction{-}

The main objective of ViSEAGO package is to carry out a data mining of biological functions and establish links between genes involved in the study. We developed ViSEAGO in R to facilitate functional Gene Ontology (GO) analysis of complex experimental design with multiple comparisons of interest. It allows to study large-scale datasets together and visualize GO profiles to capture biological knowledge. The acronym stands for three major concepts of the analysis: **Vi**sualization, **S**emantic similarity and **E**nrichment **A**nalysis of **G**ene **O**ntology. It provides access to the last current GO annotations, which are retrieved from one of NCBI EntrezGene, Ensembl or Uniprot databases for several species. Using available R packages and novel developments, ViSEAGO extends classical functional GO analysis to focus on functional coherence by aggregating closely related biological themes while studying multiple datasets at once. It provides both a synthetic and detailed view using interactive functionalities respecting the GO graph structure and ensuring functional coherence supplied by semantic similarity. ViSEAGO has been successfully applied on several datasets from different species with a variety of biological questions. Results can be easily shared between bioinformaticians and biologists, enhancing reporting capabilities while maintaining reproducibility.

# package diagram {-}

ViSEAGO is like a workflow with input lists of genes and associated genomic resources in order to perform enrichment tests, followed by a clustering of GO terms based on theirs semantic similarity. Most of outputs (tables and graphs) could be visualized in static or interactive mode.

<img src=`r system.file("extdata/","workflow.png",package="ViSEAGO")` alt="workflow" style="width:100%;height:100%;"/>

# Installation{-}

Firtsly, install the `r BiocStyle::Biocpkg("ViSEAGO")` R package from [Bioconductor](https://www.bioconductor.org/), or [ForgeMIA](https://forgemia.inra.fr/umr-boa/viseago) gitlab.

```{r ViSEAGO_install}
# Install ViSEAGO package from Bioconductor
BiocManager::install("ViSEAGO")
```

<u>NB:</u> This vignette is not runnable and provide a pseudo-code to illustrate ViSEAGO package capabilities.

# Genes of interest

## topGO{-}

Firstly, you need to provide the gene *background* (= expressed genes) and *selection* files.

```{r geneList_input_topGO}
# load genes background
background<-scan(
    "background.txt",
    quiet=TRUE,
    what=""
)

# load gene selection
selection<-scan(
    "selection.txt",
    quiet=TRUE,
    what=""
)
```

## fgsea{-}

Or a table with gene identifiers and statistical value to use for ranking.

```{r geneList_input_fgsea}
# load gene identifiers column 1) and corresponding statistical value (column 2)
table<-data.table::fread("table.txt")

# rank gene identifiers according statistical value
data.table::setorder(table,value)
```

# GO annotation of genes

The [Gene Ontology](http://www.geneontology.org) (GO) is a community-based bioinformatics resource that supplies information about gene product function using ontologies to represent biological knowledge @pmid25428369.

Currently, the 3 predominant genomic ressources are [EntrezGene](https://www.ncbi.nlm.nih.gov/gene) @pmid25355515, [Ensembl](http://ensemblgenomes.org/) @pmid27899575, and [Uniprot-GOA](http://www.ebi.ac.uk/GOA) @pmid25378336.
Furthermore, an easy access with the R langage @R to some genomic GO annotations from EntrezGene is available using [Bioconductor](https://www.bioconductor.org/) @pmid25633503 *org.xx.eg.db* databases.
However, there are important differences between supported species, and corresponding genes knowledge according to database used. This has necessary an impact on GO annotations and enrichment and downstream analyses.

More precisely, since December 2006, the GOA and [Ensembl Compara](http://www.ebi.ac.uk/GOA/compara_go_annotations) teams have used the gene orthology data obtained from Ensembl Compara to project GO terms from a source species onto one or more target species. For this pipeline, only one to one and apparent one to one orthologies are used, and only manually annotated GO terms with an experimental evidence type of either EXP, IDA, IEP, IGI, IMP or IPI are projected. GO annotations created using this technique receive the evidence code 'IEA' (Inferred from Electronic Annotation).

A similar approach for [EntrezGene](https://www.ncbi.nlm.nih.gov/gene) database and with the [orthologs_groups](https://www.ncbi.nlm.nih.gov/genome/annotation_euk/process/) @pmid24063302 from the NCBI annotation pipeline.
This method is available using the `ViSEAGO::annotate(id,EntrezGene,ortholog = TRUE)`.

The easy access to GO annotations for all the availables species from these databases are provided by:

```{r databases}
# connect to Bioconductor
Bioconductor<-ViSEAGO::Bioconductor2GO()

# connect to EntrezGene
EntrezGene<-ViSEAGO::EntrezGene2GO()

# connect to Ensembl
Ensembl<-ViSEAGO::Ensembl2GO()

# connect to Uniprot-GOA
Uniprot<-ViSEAGO::Uniprot2GO()

# connect to Custom file
Custom<-ViSEAGO::Custom2GO(system.file("extdata/customfile.txt",package = "ViSEAGO"))
```

In order to know if your target species is supported by the database, we can display the available organisms in an interactive table using `ViSEAGO::available_organisms` method. The **organism  key identifiant** is always displayed in the **first column** of the table.

```{r organisms}
# Display table of available organisms with Bioconductor
ViSEAGO::available_organisms(Bioconductor)

# Display table of available organisms with EntrezGene
ViSEAGO::available_organisms(EntrezGene)

# Display table of available organisms with Ensembl
ViSEAGO::available_organisms(Ensembl)

# Display table of available organisms with Uniprot
ViSEAGO::available_organisms(Uniprot)

# Display table of available organisms with Custom
ViSEAGO::available_organisms(Custom)
```

At this step, we need to provide the **organism  key identifiant** based on the genomic ressource used (Bioconductor, EntrezGene, Ensembl, or Uniprot), in order to extract and store the species GO annotations using `ViSEAGO::annotate()` method.

```{r annotate}
# load GO annotations from Bioconductor
myGENE2GO<-ViSEAGO::annotate(
    "bioconductor_id",
    Bioconductor
)

# load GO annotations from EntrezGene
myGENE2GO<-ViSEAGO::annotate(
    "EntrezGene_id",
    EntrezGene
)

# load GO annotations from EntrezGene
# with the add of GO annotations from orthologs genes (see above)
myGENE2GO<-ViSEAGO::annotate(
    "EntrezGene_id",
    EntrezGene,
    ortholog = TRUE
)

# load GO annotations from Ensembl
myGENE2GO<-ViSEAGO::annotate(
    "Ensembl_id",
    Ensembl
)

# load GO annotations from Uniprot
myGENE2GO<-ViSEAGO::annotate(
    "Uniprot_id",
    Uniprot
)

# load GO annotations from Custom
myGENE2GO<-ViSEAGO::annotate(
    "Custom_id",
    Custom
)
```

# Functionnal GO enrichment

## GO enrichment tests

### topGO{-}

The GO terms enrichment tests can be performed with the `r BiocStyle::Biocpkg("topGO")` package @pmid16606683, which support several algorithms that improve GO group scoring using the underlying GO graph topology.

First, we create a `topGOdata` object, using `ViSEAGO::create_topGOdata` method, with inputs as: genes selection, genes background, GO terms category used (*MF*, *BP*, or *CC*), and minimum of annotated genes by GO terms (nodeSize).

```{r Enrichment_data}
# create topGOdata for BP
BP<-ViSEAGO::create_topGOdata(
    geneSel=selection,
    allGenes=background,
    gene2GO=myGENE2GO, 
    ont="BP",
    nodeSize=5
)
```

Now, we perform the GO enrichment tests for *BP* category with Fisher's exact test with one of the four available algorithms (*classic*, *elim*, *weight*, and *weight01*) using `topGO::runTest` method.  

```{r Enrichment_data_tests}
# perform TopGO test using clasic algorithm
classic<-topGO::runTest(
    BP,
    algorithm ="classic",
    statistic = "fisher"
)
```

### fgsea{-}

The GO terms enrichment tests can be performed with the `r BiocStyle::Biocpkg("fgsea")` package @fgsea, with inputs as: preranked gene identifiers with statistical values , GO terms category used (*MF*, *BP*, or *CC*), fgsea algorithm, and fgsea parameters to use.

```{r fgsea}
# perform fgseaMultilevel tests
BP<-ViSEAGO::runfgsea(
    geneSel=table,
    ont="BP",
    gene2GO=myGENE2GO, 
    method ="fgseaMultilevel",
    params = list(
        scoreType = "pos",
         minSize=5
    )
)
```

## Combine enriched GO terms

We combine results of enrichment tests into an object using `ViSEAGO::merge_enrich_terms` method. A table of enriched GO terms in at least one comparison is displayed in interactive mode, or printed in a file using `ViSEAGO::show_table` method. The printed table contains for each enriched GO terms, additional columns including the list of significant genes and frequency (ratio of number of significant genes to number of background genes) evaluated by comparison. 

```{r Enrichment_merge}
# merge results from topGO
BP_sResults<-ViSEAGO::merge_enrich_terms(
    Input=list(
        condition=c("BP","classic")
    )
)

# merge results from fgsea
BP_sResults<-ViSEAGO::merge_enrich_terms(
    Input=list(
        condition="BP"
    )
)
```

```{r Enrichment_merge_display}
# display the merged table
ViSEAGO::show_table(BP_sResults)

# print the merged table in a file
ViSEAGO::show_table(
    BP_sResults,
    "BP_sResults.xls"
)
```

## Graphs of GO enrichment tests

Several graphs summarize important features. An interactive barchart showing the number of GO terms enriched or not in each comparison, using `ViSEAGO::GOcount` method. Intersections of lists of enriched GO terms between comparisons are displayed in an Upset plot using `ViSEAGO::Upset` method.

```{r Enrichment_merge_count}
# count significant (or not) pvalues by condition
ViSEAGO::GOcount(BP_sResults)
```

```{r Enrichment_merge_interactions,fig.height=4}
# display interactions
ViSEAGO::Upset(
    BP_sResults,
    file="OLexport.xls"
)
```

# GO terms Semantic Similarity

GO annotations of genes created at a previous step and enriched GO terms are combined using `ViSEAGO::build_GO_SS` method. The Semantic Similarity (SS) between enriched GO terms are calculated using `ViSEAGO::compute_SS_distances` method which is a wrapper of functions implemented in the `r BiocStyle::Biocpkg("GOSemSim")` package @pmid20179076. Among the SS algorithms between GO terms, 4 of them use information content (IC) of a GO term which is computed by the negative log probability of the term occurring in GO corpus. A rarely used term contains a greater amount of information. These IC-based mehods are *Resnik*, *Lin*, *Rel*, and *Jiang*. A Graph-based method, using the topology of GO graph structure to compute semantic similarity is also available with the *Wang* method. The built object `myGOs` contains all informations of enriched GO terms and computed SS distances between them.

```{r SS_build}
# initialyse 
myGOs<-ViSEAGO::build_GO_SS(
    gene2GO=myGENE2GO,
    enrich_GO_terms=BP_sResults
)

# compute all available Semantic Similarity (SS) measures
myGOs<-ViSEAGO::compute_SS_distances(
    myGOs,
    distance="Wang"
)
```

# Visualization and interpretation of enriched GO terms

## Multi Dimensional Scaling of GO terms - A preview

A Multi Dimensional Scale (MDS) plot with `ViSEAGO::MDSplot` method provides a representation of distances among a set of enriched GO terms on the two first dimensions. Some patterns could appear at this time and could be investigated in an interactive mode. The plot could be also printed in a png file.

```{r SS_terms_mdsplot,eval=FALSE}
# display MDSplot
ViSEAGO::MDSplot(myGOs)

# print MDSplot
ViSEAGO::MDSplot(
    myGOs,
    file="mdsplot1.png"
)
```

## Clustering heatmap of GO terms

To fully explore the results of this functional analysis, a hierarchical clustering method using `ViSEAGO::GOterms_heatmap` is performed based on one of SS distances (i.e `Wang`) between the enriched GO terms and a chosen aggregation criteria (i.e `ward.D2`).

Clusters of enriched GO terms are obtained by cutting branches off the dendrogram. Here, we choose a dynamic branch cutting method based on the shape of clusters using `r BiocStyle::CRANpkg("dynamicTreeCut")` [@pmid18024473; @dynamicTreeCut]. It builds clusters in a bottom top manner in two steps. 
In step 1, branches that satisfy specific criteria for being clusters, based on the merging information of the dendrogram, are detected.
A simple control of the relative sensitivity to cluster splitting is provided in the parameter *deepSplit* (The higher the value, the more and smaller clusters will be produced), and the minimum cluster size with *minClusterSize*.
In step 2, all previously unassigned objects are tested for sufficient proximity to clusters detected in the first step using a modified Partitioning Around Medoids (PAM). If the closest cluster is close enough in a precise sense that we define below, the object is assigned to that cluster. The PAM stage will be performed using *pamStage=TRUE* parameter, and respect the dendrogram using *pamRespectsDendro=TRUE* parameter, in the sense an object can be PAM-assigned only to clusters that lie below it on the branch that the object is merged into.

Enriched GO terms are ranked in a dendrogram as a result of a hierarchical clustering and colored depending on their cluster assignation. Additional illustrations are displayed: the GO description of GO terms (trimmed if more than 50 characters), a heatmap of -log10(pvalue) of enrichment test for each comparison, and optionally the Information Content (IC). The illustration is displayed with `ViSEAGO::show_heatmap` method.

```{r SS_Wang-wardD2}
# GOterms heatmap with the default parameters
Wang_clusters_wardD2<-ViSEAGO::GOterms_heatmap(
    myGOs,
    showIC=TRUE,
    showGOlabels=TRUE,
    GO.tree=list(
        tree=list(
            distance="Wang",
            aggreg.method="ward.D2"
        ),
        cut=list(
            dynamic=list(
                pamStage=TRUE,
                pamRespectsDendro=TRUE,
                deepSplit=2,
                minClusterSize =2
            )
        )
    ),
    samples.tree=NULL
)
```

```{r SS_Wang-wardD2_clusters-heatmap} 
# Display the clusters-heatmap
ViSEAGO::show_heatmap(
    Wang_clusters_wardD2,
    "GOterms"
)

# print the clusters-heatmap
ViSEAGO::show_heatmap(
    Wang_clusters_wardD2,
    "GOterms",
    "cluster_heatmap_Wang_wardD2.png"
)
```

A table of enriched GO terms in at least one of the comparison is displayed in interactive mode, or printed in a file using `ViSEAGO::show_table` method. The columns GO cluster number and IC value are added to the previous table of enriched terms table. The printed table contains for each enriched GO terms, additional columns including the list of significant genes and frequency (ratio between the number of significant genes and number of background genes in a specific GO tag) evaluated by comparison.

```{r SS_Wang-ward.D2_clusters-heatmap_table}
# Display the clusters-heatmap table
ViSEAGO::show_table(Wang_clusters_wardD2)

# Print the clusters-heatmap table
ViSEAGO::show_table(
    Wang_clusters_wardD2,
    "cluster_heatmap_Wang_wardD2.xls"
)
```

## Multi Dimensional Scaling of GO terms

We also display a Multi Dimensional Scale (MDS) plot with the overlay of GO terms clusters from `Wang_clusters_wardD2` object with `ViSEAGO::MDSplot` method. It is a way to check the coherence of GO terms clusters on the MDS plot.

```{r SS_Wang-ward.D2_mdsplot,eval=FALSE}
# display colored MDSplot
ViSEAGO::MDSplot(
    Wang_clusters_wardD2,
    "GOterms"
)

# print colored MDSplot
ViSEAGO::MDSplot(
    Wang_clusters_wardD2,
    "GOterms",
    file="mdsplot2.png"
)
```

# Visualization and interpretation of GO clusters

This part is dedicated to explore relationships between clusters of GO terms defined by the last hierarchical clustering with `ViSEAGO::GOterms_heatmap` method. This analysis should be conducted if the number of clusters of GO terms is large enough and therefore difficult to investigate.

## Compute semantic similiarity between GO clusters

A Semantic Similarity (SS) between clusters of GO terms, previously defined, are calculated using `ViSEAGO::compute_SS_distances` method. 
The availables SS algorithms between GO terms clusters, implemented in the `r BiocStyle::Biocpkg("GOSemSim")` package @pmid20179076, could be *max*, *average*, *rcmax*, or *BMA*.

A colored Multi Dimensional Scale (MDS) plot with `ViSEAGO::MDSplot` method provides a representation of distances between the clusters of GO terms. Each circle represents a cluster of GO terms and its size depends on the number of GO terms that it contains. Clusters of GO terms that are close should share a functional coherence.

```{r SS_Wang-wardD2_groups} 
# calculate semantic similarites between clusters of GO terms
Wang_clusters_wardD2<-ViSEAGO::compute_SS_distances(
    Wang_clusters_wardD2,
    distance=c("max", "avg","rcmax", "BMA")
)
```

```{r SS_Wang-ward.D2_groups_mdsplot}
# build and highlight in an interactive MDSplot grouped clusters for one distance object
ViSEAGO::MDSplot(
    Wang_clusters_wardD2,
    "GOclusters"
)

# build and highlight in MDSplot grouped clusters for one distance object
ViSEAGO::MDSplot(
    Wang_clusters_wardD2,
    "GOclusters",
    file="mdsplot3.png"
)
```

## GO clusters semantic similarities heatmap

A new hierarchical clustering using `ViSEAGO::GOclusters_heatmap` is performed based on one of the SS distance (i.e. `BMA`) between the clusters of GO terms, previously computed, and a chosen aggregation criteria (i.e `ward.D2`).

Clusters of GO terms are ranked in a dendrogram and branches are colored depending on the cluster assignation. Additional illustrations are displayed: definition of the cluster of GO terms corresponds to the first common GO term ancestor followed by the cluster label in brackets, and heatmap of GO terms count from the corresponding cluster. The illustration is displayed with `ViSEAGO::show_heatmap` method.

```{r SS_Wang-wardD2_groups_heatmap} 
# GOclusters heatmap
Wang_clusters_wardD2<-ViSEAGO::GOclusters_heatmap(
    Wang_clusters_wardD2,
    tree=list(
        distance="BMA",
        aggreg.method="ward.D2"
    )
)
```

```{r SS_Wang-ward.D2_groups_heatmap_display} 
# sisplay the GOClusters heatmap
ViSEAGO::show_heatmap(
    Wang_clusters_wardD2,
    "GOclusters"
)

# print the GOClusters heatmap in a file
ViSEAGO::show_heatmap(
    Wang_clusters_wardD2,
    "GOclusters",
    "Wang_clusters_wardD2_heatmap_groups.png"
)
```

# Conclusion

A complete functional Gene Ontology (GO) enrichment analysis can be performed using `r BiocStyle::Biocpkg("ViSEAGO")` package. It facilitates the interpretation of biological processes, functions, or cellular component  involved in the study. It provides both a synthetic and detailed view using interactive functionalities respecting GO graph structure and ensuring functional coherence.
Also, comparisons of enrichment can be made on large datasets and for complex experimental designs, such as explained in mouse mammary gland status comparison example (see `utils::vignette("2_mouse_bionconductor",package ="ViSEAGO")`).
Moreover, we can explore the effect of the GO semantic similarity algorithms on the tree structure, such as illustrate in the clustering comparions example (see`vignette("3_SS_choice",package ="ViSEAGO")`).
 
# Information session{-}

```{r session,eval=TRUE,echo=FALSE} 
version
```

# References{-}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/build_GO_SS.R
\name{build_GO_SS}
\alias{build_GO_SS}
\alias{build_GO_SS,gene2GO,enrich_GO_terms-method}
\title{build GO Semantic Similarity object.}
\usage{
build_GO_SS(gene2GO, enrich_GO_terms)

\S4method{build_GO_SS}{gene2GO,enrich_GO_terms}(gene2GO, enrich_GO_terms)
}
\arguments{
\item{gene2GO}{a \code{\link{gene2GO-class}} object from \code{\link{annotate}} method.}

\item{enrich_GO_terms}{a \code{\link{enrich_GO_terms-class}} from \code{\link{merge_enrich_terms}} method.}
}
\value{
a \code{\link{GO_SS-class}} object required by \code{\link{compute_SS_distances}}.
}
\description{
Compute the Information content (IC) on the given ontology, and
create a \code{\link{GO_SS-class}} object required by \code{\link{compute_SS_distances}} method to compute GO semantic similarity
between enriched GO terms or groups of terms.
}
\details{
This method use \code{\link{annotate}} and \code{\link{merge_enrich_terms}} output objects (see Arguments),
and compute the Information content (IC) using the internal code of \code{\link[GOSemSim]{godata}} function from \pkg{GOSemSim} package.
}
\examples{
\dontrun{
# initialyse object for compute GO Semantic Similarity
myGOs<-ViSEAGO::build_GO_SS(
    myGENE2GO,
    BP_sResults
)
}
# load data example
utils::data(
    myGOs,
    package="ViSEAGO"
)
}
\references{
Alexa A, Rahnenfuhrer J, Lengauer T. Improved scoring of functional groups from gene expression data by
decorrelating GO graph structure. Bioinformatics 2006; 22:1600-1607.

Guangchuang Yu, Fei Li, Yide Qin, Xiaochen Bo, Yibo Wu and Shengqi Wang. GOSemSim: an R package for measuring semantic similarity
among GO terms and gene products. Bioinformatics 2010 26(7):976-978.

Herve Pages, Marc Carlson, Seth Falcon and Nianhua Li (2017). AnnotationDbi: Annotation Database Interface. R package version 1.38.0.
}
\seealso{
Other GO_semantic_similarity: 
\code{\link{GO_SS-class}},
\code{\link{compute_SS_distances}()}
}
\concept{GO_semantic_similarity}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GO_clusters.R
\docType{class}
\name{GO_clusters-class}
\alias{GO_clusters-class}
\title{GO_clusters class object}
\description{
This class is invoked by \code{\link{GOterms_heatmap}} and  \code{\link{GOclusters_heatmap}} methods to store all results produced.
}
\section{Slots}{

\describe{
\item{\code{ont}}{ontology used "MF", "BP", or "CC".}

\item{\code{enrich_GOs}}{\code{\link{enrich_GO_terms-class}} object.}

\item{\code{IC}}{Information Content (IC).}

\item{\code{terms_dist}}{distance between GO terms based on semantic similiarity.}

\item{\code{clusters_dist}}{distance between GO groups based on semantic similiarity.}

\item{\code{hcl_params}}{Hierarchical clustering parameters used.}

\item{\code{dendrograms}}{GO terms and samples \code{dendrograms}.}

\item{\code{samples.gp}}{samples groups.}

\item{\code{heatmap}}{GO terms and GO groups heatmaps.}
}}

\seealso{
Other GO_clusters: 
\code{\link{GOclusters_heatmap}()},
\code{\link{clusters_cor}()},
\code{\link{compare_clusters}()},
\code{\link{show_heatmap}()},
\code{\link{show_table}()}
}
\concept{GO_clusters}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/show_heatmap.R
\name{show_heatmap}
\alias{show_heatmap}
\alias{show_heatmap,GO_clusters,character-method}
\title{Display an interactive or static heatmap.}
\usage{
show_heatmap(
  object,
  type,
  file = NULL,
  plotly_update = FALSE,
  height = 1000,
  width = 800
)

\S4method{show_heatmap}{GO_clusters,character}(
  object,
  type,
  file = NULL,
  plotly_update = FALSE,
  height = 1000,
  width = 800
)
}
\arguments{
\item{object}{a \code{\link{GO_clusters-class}} object from \code{\link{GOterms_heatmap}} or \code{\link{GOclusters_heatmap}}.}

\item{type}{could be "GOterms" to display GOterms clustering heatmap, or "GOclusters" to display GOclusters heatmap.}

\item{file}{static png output file name (default to NULL).}

\item{plotly_update}{update plotly html dependencies (default to FALSE).}

\item{height}{static image height (default to 1000).}

\item{width}{static image width (default to 1000).}
}
\value{
display or print heatmap.
}
\description{
Display a heatmap in interactive or static mode.
}
\details{
This method displays an interactive heatmap (if \code{file}=NULL) from \code{\link{GO_clusters-class}} object for "GOterms" or "GOclusters" type.\cr
A static png image could be printed by setting \code{file} argument.\cr
Interactive heatmap cannot be displayed between two R versions.
Then interactive view (build with previous R version) can be updated to new R version using \code{plotly_update} argument setting to TRUE.
}
\examples{
# load data example
data(
    myGOs,
    package="ViSEAGO"
)
\dontrun{
# compute GO terms Semantic Similarity distances
myGOs<-ViSEAGO::compute_SS_distances(
    myGOs,
    distance="Wang"
)

# build MDS plot for a GO_SS-class distance object
ViSEAGO::MDSplot(myGOs)

# GOtermsHeatmap with default parameters
Wang_clusters_wardD2<-ViSEAGO::GOterms_heatmap(
    myGOs,
    showIC=TRUE,
    showGOlabels=TRUE,
    GO.tree=list(
        tree=list(
            distance="Wang",
            aggreg.method="ward.D2",
            rotate=NULL
        ),
        cut=list(
            dynamic=list(
                pamStage=TRUE,
                pamRespectsDendro=TRUE,
                deepSplit=2,
                minClusterSize =2
            )
        )
    ),
    samples.tree=NULL
)

# Display GO terms heatmap
ViSEAGO::show_heatmap(
    Wang_clusters_wardD2,
    "GOterms"
)

# Print GO terms heatmap
ViSEAGO::show_heatmap(
    Wang_clusters_wardD2,
    "GOterms",
    "GOterms_heatmap.png"
)

# compute clusters of GO terms Semantic Similarity distances
Wang_clusters_wardD2<-ViSEAGO::compute_SS_distances(
    Wang_clusters_wardD2,
    distance="BMA"
)

# GOclusters heatmap
Wang_clusters_wardD2<-ViSEAGO::GOclusters_heatmap(
    Wang_clusters_wardD2,
    tree=list(
        distance="BMA",
        aggreg.method="ward.D2",
        rotate=NULL
    )
)

# Display GO clusters heatmap
ViSEAGO::show_heatmap(
    Wang_clusters_wardD2,
    "GOclusters"
)

# Print GO clusters heatmap
ViSEAGO::show_heatmap(
    Wang_clusters_wardD2,
    "GOclusters",
    "GOclusters_heatmap.png"
)
}
}
\seealso{
Other enrich_GO_terms: 
\code{\link{Upset}()},
\code{\link{enrich_GO_terms-class}},
\code{\link{overLapper}()},
\code{\link{show_table}()}

Other GO_clusters: 
\code{\link{GO_clusters-class}},
\code{\link{GOclusters_heatmap}()},
\code{\link{clusters_cor}()},
\code{\link{compare_clusters}()},
\code{\link{show_table}()}

Other visualization: 
\code{\link{GOclusters_heatmap}()},
\code{\link{GOcount}()},
\code{\link{GOterms_heatmap}()},
\code{\link{Upset}()},
\code{\link{available_organisms}()},
\code{\link{overLapper}()},
\code{\link{show_table}()}
}
\concept{GO_clusters}
\concept{enrich_GO_terms}
\concept{visualization}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/taxonomy.R
\name{taxonomy}
\alias{taxonomy}
\title{Display Organism Scientific and common name from taxid.}
\usage{
taxonomy(...)
}
\value{
a \code{\link[data.table]{data.table}}.
}
\description{
Display Organism Scientific and common name from taxid.
}
\details{
This internal function use \href{https://www.ncbi.nlm.nih.gov/books/NBK25500/}{E-utilities} summary to display in
 \code{\link[data.table]{data.table}} Organism Scientific name and common name, form a \code{\link[base]{vector}} of taxid.
}
\examples{
# Organism Scientific and common name from taxid
Data<-ViSEAGO::taxonomy("9031")
}
\seealso{
Other genomic_ressource: 
\code{\link{Bioconductor2GO}()},
\code{\link{Custom2GO}()},
\code{\link{Ensembl2GO}()},
\code{\link{EntrezGene2GO}()},
\code{\link{Uniprot2GO}()},
\code{\link{annotate}()},
\code{\link{available_organisms}()},
\code{\link{genomic_ressource-class}}
}
\concept{genomic_ressource}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pkgdiagram.R
\name{pkgdiagram}
\alias{pkgdiagram}
\title{Display the ViSEAGO package diagram.}
\usage{
pkgdiagram(x = c("white", "white", "white", "white", "white"))
}
\arguments{
\item{x}{rectangles colors for the five subgraphs with all white by defaults}
}
\value{
an html diagram
}
\description{
This function build the ViSEAGO package diagram available displayed in the vignette.
}
\details{
This function use \pkg{DiagrammeR} package \code{\link[DiagrammeR]{grViz}} to display the ViSEAGO methods diagram with colored or not subgraphs rectangles.
}
\examples{
# print diagram without focus
ViSEAGO::pkgdiagram(x=c("white","white","white","white","white"))

# print diagram with focus one the gene list
ViSEAGO::pkgdiagram(x=c("black","white","white","white","white"))

# print diagram with a focus on the annotation step
ViSEAGO::pkgdiagram(x=c("white","black","white","white","white"))

# print diagram with a focus on the enrichment step
ViSEAGO::pkgdiagram(x=c("white","white","black","white","white"))

# print diagram with a focus on the Semantic Similarity step
ViSEAGO::pkgdiagram(x=c("white","white","white","black","white"))

# print diagram with focus on the visualization step
ViSEAGO::pkgdiagram(x=c("white","white","white","white","black"))
}
\references{
Knut Sveidqvist, Mike Bostock, Chris Pettitt, Mike Daines, Andrei Kashcha and Richard Iannone (2017). DiagrammeR: Create Graph Diagrams and Flowcharts Using R.
R package version 0.9.0. https://CRAN.R-project.org/package=DiagrammeR.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MDSplot.R
\name{MDSplot}
\alias{MDSplot}
\alias{MDSplot,ANY-method}
\title{Multi Dimensional Scale (MDS) plot}
\usage{
MDSplot(object, type = "GOterms", file = NULL)

\S4method{MDSplot}{ANY}(object, type = "GOterms", file = NULL)
}
\arguments{
\item{object}{a \code{\link{GO_SS-class}} or \code{\link{GO_clusters-class}} objects from distances computed with \code{\link{compute_SS_distances}}.}

\item{type}{could be "GOterms" to display GOterms MDSplot, or "GOclusters" to display GOclusters MDSplot.}

\item{file}{static image output file name (default to NULL).}
}
\value{
a MDS plot.
}
\description{
Generate a Multi Dimensional Scale (MDS) plot from distance objects.
}
\details{
This method build and display the javascript MDSplot (if \code{file}=NULL) from \code{\link{GO_SS-class}} or \code{\link{GO_clusters-class}}
objects.\cr
A static png image could be printed by setting \code{file} argument.
}
\examples{
# load data example
utils::data(
 myGOs,
 package="ViSEAGO"
)
\dontrun{
# compute GO terms Semantic Similarity distances
myGOs<-ViSEAGO::compute_SS_distances(
    myGOs,
    distance="Wang"
)

# build MDS plot for a GO_SS-class distance object
ViSEAGO::MDSplot(myGOs,"GOterms")

# GOtermsHeatmap with default parameters
Wang_clusters_wardD2<-ViSEAGO::GOterms_heatmap(
    myGOs,
    showIC=TRUE,
    showGOlabels=TRUE,
    GO.tree=list(
        tree=list(
            distance="Wang",
            aggreg.method="ward.D2",
            rotate=NULL
        ),
        cut=list(
            dynamic=list(
                pamStage=TRUE,
                pamRespectsDendro=TRUE,
                deepSplit=2,
                minClusterSize =2
            )
        )
    ),
    samples.tree=NULL
)

# build MDS plot for a GO_clusters-class distance object, highlighting GO terms clusters.
ViSEAGO::MDSplot(
    Wang_clusters_wardD2,
    "GOterms"
)

# compute clusters of GO terms Semantic Similarity distances
Wang_clusters_wardD2<-ViSEAGO::compute_SS_distances(
    Wang_clusters_wardD2,
    distance="BMA"
)

# GOclusters heatmap
Wang_clusters_wardD2<-ViSEAGO::GOclusters_heatmap(
    Wang_clusters_wardD2,
    tree=list(
        distance="BMA",
        aggreg.method="ward.D2",
        rotate=NULL
    )
)

# build MDS plot for a GO_clusters-class distance object, highlighting GO groups clusters.
ViSEAGO::MDSplot(
    Wang_clusters_wardD2,
    "GOclusters"
)
}
}
\concept{GO_terms GO_clusters semantic_similarity visualization}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/overlapper.R
\name{overLapper}
\alias{overLapper}
\title{build all sets combinations intersections}
\usage{
overLapper(setlist)
}
\value{
a \code{\link[base]{list}}.
}
\description{
build all sets combinations intersections
}
\details{
This internal function use build all sets combinations intersections needed for \code{\link{upset}}.
}
\examples{
\dontrun{
# build all intersections combinations
ViSEAGO::overLapper(setlist)
}
}
\seealso{
Other enrich_GO_terms: 
\code{\link{Upset}()},
\code{\link{enrich_GO_terms-class}},
\code{\link{show_heatmap}()},
\code{\link{show_table}()}

Other visualization: 
\code{\link{GOclusters_heatmap}()},
\code{\link{GOcount}()},
\code{\link{GOterms_heatmap}()},
\code{\link{Upset}()},
\code{\link{available_organisms}()},
\code{\link{show_heatmap}()},
\code{\link{show_table}()}
}
\concept{enrich_GO_terms}
\concept{visualization}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/datasets.R
\docType{data}
\name{myGOs}
\alias{myGOs}
\title{myGOs dataset}
\format{
An object of class \code{\link{GO_SS-class}}.
}
\usage{
data(myGOs,package="ViSEAGO")
}
\description{
an example of object returned by \code{\link{build_GO_SS}} method
from mouse functional analysis of mouse mammary gland RNA-Seq (2_mouse_bioconductor vignette)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clusters_cor.R
\name{clusters_cor}
\alias{clusters_cor}
\alias{clusters_cor,list,character-method}
\title{Compute distance matrix between dendrograms partitions.}
\usage{
clusters_cor(clusters, method = "adjusted.rand")

\S4method{clusters_cor}{list,character}(clusters, method = "adjusted.rand")
}
\arguments{
\item{clusters}{a \code{list} of \code{\link{GO_clusters-class}} objects,
from \code{\link{GOterms_heatmap}} or \code{\link{GOclusters_heatmap}}, named as \code{character}.}

\item{method}{available methods  ("vi", "nmi", "split.join", "rand", or "adjusted.rand") from \pkg{igraph} package \code{\link[igraph]{compare}} function.}
}
\value{
a distance or a correlation matrix.
}
\description{
Build a distance or correlation matrix between partitions from dendrograms.
}
\examples{
# load example object
data(
    myGOs,
    package="ViSEAGO"
)

\dontrun{
# compute Semantic Similarity (SS)
myGOs<-ViSEAGO::compute_SS_distances(
    myGOs,
    distance=c("Resnik","Lin","Rel","Jiang","Wang")
)

# Resnik distance GO terms heatmap
Resnik_clusters_wardD2<-ViSEAGO::GOterms_heatmap(
    myGOs,
    showIC=TRUE,
    showGOlabels=TRUE,
    GO.tree=list(
        tree=list(
            distance="Resnik",
            aggreg.method="ward.D2"
        ),
        cut=list(
            dynamic=list(
                deepSplit=2,
                minClusterSize =2
            )
        )
    ),
    samples.tree=NULL
)

# Lin distance GO terms heatmap
Lin_clusters_wardD2<-ViSEAGO::GOterms_heatmap(
    myGOs,
    showIC=TRUE,
    showGOlabels=TRUE,
    GO.tree=list(
        tree=list(
            distance="Lin",
            aggreg.method="ward.D2"
        ),
        cut=list(
            dynamic=list(
                deepSplit=2,
                minClusterSize =2
            )
        )
    ),
    samples.tree=NULL
)

# Resnik distance GO terms heatmap
Rel_clusters_wardD2<-ViSEAGO::GOterms_heatmap(
    myGOs,
    showIC=TRUE,
    showGOlabels=TRUE,
    GO.tree=list(
        tree=list(
            distance="Rel",
            aggreg.method="ward.D2"
        ),
        cut=list(
            dynamic=list(
                deepSplit=2,
                minClusterSize =2
            )
        )
    ),
    samples.tree=NULL
)

# Resnik distance GO terms heatmap
Jiang_clusters_wardD2<-ViSEAGO::GOterms_heatmap(
    myGOs,
    showIC=TRUE,
    showGOlabels=TRUE,
    GO.tree=list(
        tree=list(
            distance="Jiang",
            aggreg.method="ward.D2"
        ),
        cut=list(
            dynamic=list(
                deepSplit=2,
                minClusterSize =2
            )
        )
    ),
    samples.tree=NULL
)

# Resnik distance GO terms heatmap
Wang_clusters_wardD2<-ViSEAGO::GOterms_heatmap(
    myGOs,
    showIC=TRUE,
    showGOlabels=TRUE,
    GO.tree=list(
        tree=list(
            distance="Wang",
            aggreg.method="ward.D2"
        ),
        cut=list(
            dynamic=list(
                deepSplit=2,
                minClusterSize =2
            )
        )
    ),
    samples.tree=NULL
)
}
# clusters to compare
clusters<-list(
    Resnik="Resnik_clusters_wardD2",
    Lin="Lin_clusters_wardD2",
    Rel="Rel_clusters_wardD2",
    Jiang="Jiang_clusters_wardD2",
    Wang="Wang_clusters_wardD2"
)

\dontrun{
# global dendrogram clustering correlation
clust_cor<-ViSEAGO::clusters_cor(
    clusters,
    method="adjusted.rand"
)
}
}
\references{
Csardi G, Nepusz T: The igraph software package for complex network research, InterJournal, Complex Systems 1695. 2006. http://igraph.org.
}
\seealso{
Other GO_clusters: 
\code{\link{GO_clusters-class}},
\code{\link{GOclusters_heatmap}()},
\code{\link{compare_clusters}()},
\code{\link{show_heatmap}()},
\code{\link{show_table}()}
}
\concept{GO_clusters}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GO_SS.R
\docType{class}
\name{GO_SS-class}
\alias{GO_SS-class}
\title{GO_SS class object definition.}
\description{
This class is invoked by \code{\link{build_GO_SS}} method in order to store \code{\link{enrich_GO_terms-class}} object, Information Content (IC),
and GO terms or groups distances objects based on semantic similarity.
}
\section{Slots}{

\describe{
\item{\code{ont}}{ontology used "MF", "BP", or "CC".}

\item{\code{enrich_GOs}}{\code{\link{merge_enrich_terms}} output object (\code{\link{enrich_GO_terms-class}} object).}

\item{\code{IC}}{Information Content (IC)}

\item{\code{terms_dist}}{\code{list} of GO terms or groups distances objects based on semantic similarity.}
}}

\seealso{
Other GO_semantic_similarity: 
\code{\link{build_GO_SS}()},
\code{\link{compute_SS_distances}()}
}
\concept{GO_semantic_similarity}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Bioconductor2GO.R
\name{Bioconductor2GO}
\alias{Bioconductor2GO}
\title{Check available organisms databases at Bioconductor.}
\usage{
Bioconductor2GO()
}
\value{
a  \code{\link{genomic_ressource-class}} object required by \code{\link{annotate}} method.
}
\description{
Retrieve the Bioconductor \href{http://bioconductor.org/packages/release/BiocViews.html#___OrgDb}{OrgDb}
available organisms databases packages.
}
\details{
This function gives genome wide annotation for available organisms databases packages from
\href{http://bioconductor.org/packages/release/BiocViews.html#___OrgDb}{Bioconductor OrgDb}.
It uses \code{loadAnnDbPkgIndex} from \pkg{AnnotationForge} package.
}
\examples{
# Check Bioconductor OrgDb available organisms
Bioconductor<-ViSEAGO::Bioconductor2GO()
}
\references{
Carlson M and Pages H (2017). AnnotationForge: Code for Building Annotation Database Packages. R package version 1.18.0.
}
\seealso{
Other genomic_ressource: 
\code{\link{Custom2GO}()},
\code{\link{Ensembl2GO}()},
\code{\link{EntrezGene2GO}()},
\code{\link{Uniprot2GO}()},
\code{\link{annotate}()},
\code{\link{available_organisms}()},
\code{\link{genomic_ressource-class}},
\code{\link{taxonomy}()}
}
\concept{genomic_ressource}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/custom2GO.R
\name{Custom2GO}
\alias{Custom2GO}
\title{Store organisms GO annotations from custom database file.}
\usage{
Custom2GO(file)
}
\arguments{
\item{file}{custom GO annotation file}
}
\value{
a  \code{\link{genomic_ressource-class}} object required by \code{\link{annotate}}.
}
\description{
Store the available species and current GO annotations from a custom table file
}
\details{
This function load a custom GO annotation database table that must contain columns:
\describe{
     \item{taxid}{custom taxonomic identifiants}
     \item{gene_id}{custom gene identifiants}
     \item{gene_symbol}{custom gene symbols}
     \item{GOID}{Known GO identifiants (see \code{select(GO.db,columns=columns(GO.db),keys=keys(GO.db))}}
     \item{evidence}{Known GO \href{http://geneontology.org/page/guide-go-evidence-codes}{evidence codes}}
}
}
\examples{
\dontrun{
# Download custom GO annotations
Custom<-ViSEAGO::Custom2GO(
    system.file(
        "extdata/customfile.txt",
        package = "ViSEAGO"
    )
)
}
}
\references{
Matt Dowle and Arun Srinivasan (2017). data.table: Extension of `data.frame`. R package version 1.10.4. https://CRAN.R-project.org/package=data.table.
}
\seealso{
Other genomic_ressource: 
\code{\link{Bioconductor2GO}()},
\code{\link{Ensembl2GO}()},
\code{\link{EntrezGene2GO}()},
\code{\link{Uniprot2GO}()},
\code{\link{annotate}()},
\code{\link{available_organisms}()},
\code{\link{genomic_ressource-class}},
\code{\link{taxonomy}()}
}
\concept{genomic_ressource}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GOclusters_heatmap.R
\name{GOclusters_heatmap}
\alias{GOclusters_heatmap}
\alias{GOclusters_heatmap,GO_clusters,list-method}
\title{Build a clustering heatmap on GO groups.}
\usage{
GOclusters_heatmap(
  object,
  tree = list(distance = "BMA", aggreg.method = "ward.D2", rotate = NULL)
)

\S4method{GOclusters_heatmap}{GO_clusters,list}(
  object,
  tree = list(distance = "BMA", aggreg.method = "ward.D2", rotate = NULL)
)
}
\arguments{
\item{object}{a  \code{\link{GO_clusters-class}} object from \code{\link{compute_SS_distances}}.}

\item{tree}{a named list with:
\describe{
    \item{distance ("BMA" by default)}{
    distance computed from the semantic similarity for GO groups which could be "max", "avg", "rcmax",or "BMA".}
    \item{aggreg.method ("ward.D2" by default)}{aggregation method criteria from \code{\link[stats]{hclust}} (ward.D",
    "ward.D2", "single", "complete", "average", "mcquitty", "median", or "centroid") to build a \code{dendrogram}.}
    \item{rotate}{sort the branches of the tree based on a vector - eithor of labels order or the labels in their new order}
}}
}
\value{
a \code{\link{GO_clusters-class}} object.
}
\description{
This method computes a clustering heatmap based on GO groups semantic similarity.
}
\details{
This method computes a clustering heatmap based on GO groups semantic similarity (computed with \code{\link{compute_SS_distances}}).\cr
The heatmap color intensity corresponds to the number of GO terms in each GO group.\cr
GO group description is defined as the first common GO ancestor with the cluster identifiant in brackets.\cr
The dendrogram branches are colored according to GO terms clusters.
}
\examples{
# load data example
utils::data(
    myGOs,
    package="ViSEAGO"
)
\dontrun{
# compute GO terms Semantic Similarity distances
myGOs<-ViSEAGO::compute_SS_distances(
    myGOs,
    distance="Wang"
)

# GOtermsHeatmap with default parameters
Wang_clusters_wardD2<-ViSEAGO::GOterms_heatmap(
    myGOs,
    showIC=TRUE,
    showGOlabels=TRUE,
    GO.tree=list(
        tree=list(
            distance="Wang",
            aggreg.method="ward.D2",
            rotate=NULL
        ),
        cut=list(
            dynamic=list(
                pamStage=TRUE,
                pamRespectsDendro=TRUE,
                deepSplit=2,
                minClusterSize =2
            )
        )
    ),
    samples.tree=NULL
)

# compute clusters of GO terms Semantic Similarity distances
Wang_clusters_wardD2<-ViSEAGO::compute_SS_distances(
    Wang_clusters_wardD2,
    distance="BMA"
)

# GOclusters heatmap
Wang_clusters_wardD2<-ViSEAGO::GOclusters_heatmap(
    Wang_clusters_wardD2,
    tree=list(
        distance="BMA",
        aggreg.method="ward.D2",
        rotate=NULL
    )
)
}
}
\references{
Matt Dowle and Arun Srinivasan (2017). data.table: Extension of `data.frame`. R package version 1.10.4. https://CRAN.R-project.org/package=data.table.

Tal Galili (2015). dendextend: an R package for visualizing, adjusting, and comparing trees of hierarchical clustering.
Bioinformatics. DOI:10.1093/bioinformatics/btv428.

Tal Galili (2017). heatmaply: Interactive Cluster Heat Maps Using 'plotly'.
R package version 0.9.1. https://CRAN.R-project.org/package=heatmaply.

Erich Neuwirth (2014). RColorBrewer: ColorBrewer Palettes. R package version 1.1-2. https://CRAN.R-project.org/package=RColorBrewer.

Carson Sievert, Chris Parmer, Toby Hocking, Scott Chamberlain, Karthik Ram, Marianne Corvellec and Pedro Despouy (2017).
plotly: Create Interactive Web Graphics via 'plotly.js'. R package version 4.6.0. https://CRAN.R-project.org/package=plotly.

H. Wickham. ggplot2: Elegant Graphics for Data Analysis. Springer-Verlag New York, 2009.
}
\seealso{
Other GO_clusters: 
\code{\link{GO_clusters-class}},
\code{\link{clusters_cor}()},
\code{\link{compare_clusters}()},
\code{\link{show_heatmap}()},
\code{\link{show_table}()}

Other visualization: 
\code{\link{GOcount}()},
\code{\link{GOterms_heatmap}()},
\code{\link{Upset}()},
\code{\link{available_organisms}()},
\code{\link{overLapper}()},
\code{\link{show_heatmap}()},
\code{\link{show_table}()}
}
\concept{GO_clusters}
\concept{semantic_similarity}
\concept{visualization}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Ensembl2GO.R
\name{Ensembl2GO}
\alias{Ensembl2GO}
\title{Check available organisms datasets at Ensembl.}
\usage{
Ensembl2GO(biomart = "genes", GRCh = NULL, version = NULL)
}
\arguments{
\item{biomart}{the biomart name available with \pkg{biomaRt} package \code{\link[biomaRt]{listEnsembl}} ("genes", the default) or \code{\link[biomaRt]{listEnsemblGenomes}} ("protists_mart", "fungi_mart", "plants_mart").}

\item{GRCh}{GRCh version to connect to if not the current GRCh38, currently this can only be 37}

\item{version}{the annotation version to use (eg. NULL for the default current version, or a version number in \code{character})}
}
\value{
a  \code{\link{genomic_ressource-class}} object required by \code{\link{annotate}}.
}
\description{
List Ensembl referenced organisms datasets from the current (NULL) or archive (number in \code{character}) annotation version.
}
\details{
This function gives referenced organisms genomes at \href{http://ensemblgenomes.org/}{Ensembl}.
It uses the \code{\link[biomaRt]{useEnsembl}} and  \code{\link[biomaRt]{listDatasets}} from \pkg{biomaRt} package.
}
\examples{
\dontrun{
# check the Ensembl available biomart (if not known)
biomaRt::listEnsembl()

# List Ensembl available organisms
Ensembl<-ViSEAGO::Ensembl2GO(
 biomart="genes",
 GRCh = NULL,
 version=NULL
)
}
}
\references{
Durinck S, Spellman P, Birney E and Huber W (2009).
Mapping identifiers for the integration of genomic datasets with the R/Bioconductor package biomaRt.
Nature Protocols, 4, pp. 1184-1191.

Durinck S, Moreau Y, Kasprzyk A, Davis S, De Moor B, Brazma A and Huber W (2005).
BioMart and Bioconductor: a powerful link between biological databases and microarray data analysis. Bioinformatics, 21, pp. 3439-3440.

Matt Dowle and Arun Srinivasan (2017). data.table: Extension of data.frame. R package version 1.10.4. https://CRAN.R-project.org/package=data.table.
}
\seealso{
Other genomic_ressource: 
\code{\link{Bioconductor2GO}()},
\code{\link{Custom2GO}()},
\code{\link{EntrezGene2GO}()},
\code{\link{Uniprot2GO}()},
\code{\link{annotate}()},
\code{\link{available_organisms}()},
\code{\link{genomic_ressource-class}},
\code{\link{taxonomy}()}
}
\concept{genomic_ressource}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/EntrezGene_orthologs.R
\name{EntrezGene_orthologs}
\alias{EntrezGene_orthologs}
\title{Download EntrezGene orthologs groups.}
\usage{
EntrezGene_orthologs()
}
\value{
a \code{\link[data.table]{data.table}}.
}
\description{
Download EntrezGene orthologs groups.
}
\details{
Internal function used by \code{\link{annotate}} in order to download orthologs_groups from NCBI Annotation pipeline
 stored in the \href{ftp://ftp.ncbi.nih.gov/gene/DATA/gene_orthologs.gz}{gene_group.gz} file.
}
\examples{
\dontrun{
# Organism taxid, Scientific name and common name
ViSEAGO::EntrezGene_orthologs()
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gene2GO.R
\docType{class}
\name{gene2GO-class}
\alias{gene2GO-class}
\title{gene2GO class object definition.}
\description{
This class is invoked by \code{\link{annotate}} method in order to store GO annotations for each category (MF, BP, CC).
}
\section{Slots}{

\describe{
\item{\code{db}}{database source in \code{character}.}

\item{\code{stamp}}{date of stamp  in \code{character}.}

\item{\code{organism}}{target species GO annotation  in \code{character}.}

\item{\code{MF}}{a \code{list} containing  GO terms for Molecular Function (MF) category for each gene element.}

\item{\code{BP}}{a \code{list} containing  GO terms for Biological Process (BP) category for each gene element.}

\item{\code{CC}}{a \code{list} containing  GO terms for Cellular Component (CC) category for each gene element.}
}}

\seealso{
Other GO_terms: 
\code{\link{GOcount}()},
\code{\link{GOterms_heatmap}()},
\code{\link{annotate}()},
\code{\link{create_topGOdata}()},
\code{\link{merge_enrich_terms}()},
\code{\link{runfgsea}()}
}
\concept{GO_terms}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fgsea.R
\docType{class}
\name{fgsea-class}
\alias{fgsea-class}
\title{fgsea class object definition.}
\description{
This class is invoked by \code{\link{runfgsea}} method in order to store results.
}
\section{Slots}{

\describe{
\item{\code{description}}{a \code{character} string with database source, date of stamp, and target species GO annotation.}

\item{\code{method}}{fgsea method used.}

\item{\code{params}}{a \code{list} containing used input parameters for perform \code{\link[fgsea]{fgseaSimple}} or \code{\link[fgsea]{fgseaMultilevel}}.}

\item{\code{input}}{a \code{list} containing input values.}

\item{\code{data}}{a \code{list} containing \code{data.table} fgsea procedure output.}
}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/EntrezGene2GO.R
\name{EntrezGene2GO}
\alias{EntrezGene2GO}
\title{Store available organisms GO annotations at EntrezGene.}
\usage{
EntrezGene2GO()
}
\value{
a  \code{\link{genomic_ressource-class}} object required by \code{\link{annotate}}.
}
\description{
Store the available species and current GO annotations from the
\href{https://ftp.ncbi.nih.gov/gene/DATA/gene2go.gz}{gene2go.gz} nfile avalable at NCBI \href{https://ftp.ncbi.nih.gov/gene/DATA}{EntrezGene ftp}.
}
\details{
This function downloads the \href{https://ftp.ncbi.nih.gov/gene/DATA/gene2go.gz}{gene2go.gz} file from
\href{https://ftp.ncbi.nih.gov/gene/DATA}{EntrezGene ftp} which contains available organisms (taxid) with the corresponding GO annotations.
}
\examples{
\dontrun{
# Download EntrezGene available organisms GO annotations
EntrezGene<-ViSEAGO::EntrezGene2GO()
}
}
\references{
Matt Dowle and Arun Srinivasan (2017). data.table: Extension of `data.frame`. R package version 1.10.4. https://CRAN.R-project.org/package=data.table.

Eric Sayers (2013). Entrez Programming Utilities Help.

#' Henrik Bengtsson (2016). R.utils: Various Programming Utilities. R package version 2.5.0. https://CRAN.R-project.org/package=R.utils.

Maglott, D, Ostell, J, Pruitt, KD, Tatusova, T (2011). Entrez Gene: gene-centered information at NCBI. Nucleic Acids Res., 39, Database issue:D52-7.
}
\seealso{
Other genomic_ressource: 
\code{\link{Bioconductor2GO}()},
\code{\link{Custom2GO}()},
\code{\link{Ensembl2GO}()},
\code{\link{Uniprot2GO}()},
\code{\link{annotate}()},
\code{\link{available_organisms}()},
\code{\link{genomic_ressource-class}},
\code{\link{taxonomy}()}
}
\concept{genomic_ressource}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Uniprot2GO.R
\name{Uniprot2GO}
\alias{Uniprot2GO}
\title{Check available organisms databases at Uniprot.}
\usage{
Uniprot2GO()
}
\value{
a  \code{\link{genomic_ressource-class}} object required by \code{\link{annotate}}.
}
\description{
Check the \href{http://www.ebi.ac.uk/GOA}{Uniprot-GOA} available organisms.
}
\details{
This function downloads the current_release_numbers file (ftp://ftp.ebi.ac.uk/pub/databases/GO/goa/current_release_numbers.txt) from
\href{http://www.ebi.ac.uk/GOA}{Uniprot-GOA} which contains available organisms.
}
\examples{
\dontrun{
# List Uniprot-GOA available organisms
Uniprot<-ViSEAGO::Uniprot2GO()
}
}
\references{
Matt Dowle and Arun Srinivasan (2017). data.table: Extension of `data.frame`. R package version 1.10.4. https://CRAN.R-project.org/package=data.table.

Huntley, RP, Sawford, T, Mutowo-Meullenet, P, Shypitsyna, A, Bonilla, C, Martin, MJ, O'Donovan, C (2015).
The GOA database: gene Ontology annotation updates for 2015. Nucleic Acids Res., 43, Database issue:D1057-63.
}
\seealso{
Other genomic_ressource: 
\code{\link{Bioconductor2GO}()},
\code{\link{Custom2GO}()},
\code{\link{Ensembl2GO}()},
\code{\link{EntrezGene2GO}()},
\code{\link{annotate}()},
\code{\link{available_organisms}()},
\code{\link{genomic_ressource-class}},
\code{\link{taxonomy}()}
}
\concept{genomic_ressource}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/enrich_GO_terms.R
\docType{class}
\name{enrich_GO_terms-class}
\alias{enrich_GO_terms-class}
\title{enrich_GO_terms class object definition.}
\description{
This class is invoked by \code{\link{merge_enrich_terms}} method in order to store the merged \code{data.table} and associated metadata.
}
\section{Slots}{

\describe{
\item{\code{same_genes_background}}{logical.
object(s) to combinate (see examples in \code{\link{merge_enrich_terms}}).}

\item{\code{ont}}{ontology used "MF", "BP", or "CC".}

\item{\code{method}}{enrichment test used "topGO", or "fgsea".}

\item{\code{summary}}{a \code{list} with topGO or fgsea object(s) summary informations.}

\item{\code{data}}{a merged \code{data.table}  of enriched GO terms (p<0.01) in at least once with GO descriptions and statistical values.}
}}

\seealso{
Other enrich_GO_terms: 
\code{\link{Upset}()},
\code{\link{overLapper}()},
\code{\link{show_heatmap}()},
\code{\link{show_table}()}
}
\concept{enrich_GO_terms}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/annotate.R
\name{annotate}
\alias{annotate}
\alias{annotate,character,genomic_ressource-method}
\title{Retrieve GO annotations for a specie from genomic ressource database.}
\usage{
annotate(id, object, ortholog = FALSE)

\S4method{annotate}{character,genomic_ressource}(id, object, ortholog = FALSE)
}
\arguments{
\item{id}{identifiant corresponding to the organism of interest.
This id name is referenced in the first column of the database
used (see \code{\link{available_organisms}}).}

\item{object}{a required \code{\link{genomic_ressource-class}} object created by
\code{\link{Bioconductor2GO}}, \code{\link{EntrezGene2GO}},
\code{\link{Ensembl2GO}}, or \code{\link{Uniprot2GO}} methods.}

\item{ortholog}{\code{logical} (default to FALSE). Only available for
vertebrates organisms and for object created by \code{\link{EntrezGene2GO}} method (see Details).}
}
\value{
\code{annotate} produces an object of \code{\link{gene2GO-class}} required by \code{\link{build_GO_SS}} method.
}
\description{
This method retrieves and stores GO annotations for the
organism of interest from one of genomic ressource database
(Bioconductor, EntrezGene, Ensembl, Uniprot).
}
\details{
This method uses a \code{\link{genomic_ressource-class}} object to retrieve
\href{http://www.geneontology.org/page/ontology-documentation}{GO} annotations for the organism of interest.
The stored annotations are structured in 3 slots corresponding to the 3 GO categories: MF (Molecular Function),
BP (Biological Process), and CC (Cellular Component). Each slot contains GO terms with
associated \href{http://www.geneontology.org/page/guide-go-evidence-codes}{evidence code}.

The \code{\link{genomic_ressource-class}} object is created by one of the four available methods:
\code{\link{Bioconductor2GO}}, \code{\link{EntrezGene2GO}},
\code{\link{Ensembl2GO}}, or \code{\link{Uniprot2GO}}.

In the case of vertebrates, setting \code{ortholog} argument to \code{TRUE} is required if you need to add GO terms with experimental
\href{http://geneontology.org/page/guide-go-evidence-codes}{evidence codes} from orthologs genes
when using \code{\link{EntrezGene2GO}} method. To display organisms supported by NCBI EntrezGene orthologs pipeline,
set the arguments \code{id=NULL} and \code{ortholog=TRUE}.
This approch is highly similar to the strategy developed by Uniprot-GOA consortium for the Electronic Annotation Method using
\href{http://www.ebi.ac.uk/GOA/compara_go_annotations}{Ensembl Compara}.
}
\examples{
\dontrun{
## load Mus musculus (mouse) GO annotations

# from Bioconductor
Bioconductor<-ViSEAGO::Bioconductor2GO()
myGENE2GO<-ViSEAGO::annotate(
    id="org.Mm.eg.db",
    object=Bioconductor
)

# from EntrezGene
EntrezGene<-ViSEAGO::EntrezGene2GO()
myGENE2GO<-ViSEAGO::annotate(
    id="10090",
    object=EntrezGene
)

# from EntrezGene
Ensembl<-ViSEAGO::Ensembl2GO()
myGENE2GO<-ViSEAGO::annotate(
    id="mmusculus_gene_ensembl",
    object=Ensembl
)

# from Uniprot
Uniprot<-ViSEAGO::Uniprot2GO()
myGENE2GO<-ViSEAGO::annotate(
    id="mouse",
    object=Uniprot
)

## from Custom GO annotation file
Custom<-ViSEAGO::Custom2GO(system.file("extdata/customfile.txt",package = "ViSEAGO"))
myGENE2GO<-ViSEAGO::annotate(
    id="myspecies1",
    object=Custom
)

## specific options for EntrezGene database

# Chicken GO annotations without adding orthologs
EntrezGene<-ViSEAGO::EntrezGene2GO()
myGENE2GO<-ViSEAGO::annotate(
    id="9031",
    object=EntrezGene
)

# Chicken GO annotation with the add of orthologs GO annotations
EntrezGene<-ViSEAGO::EntrezGene2GO()
myGENE2GO<-ViSEAGO::annotate(
    id="9031",
    object=EntrezGene,
    ortholog=TRUE
)

# display organisms supported by NCBI EntrezGene orthologs pipeline
EntrezGene<-ViSEAGO::EntrezGene2GO()
ViSEAGO::annotate(
    id="NULL",
    object=EntrezGene,
    ortholog=TRUE
)
}
}
\references{
Durinck S, Spellman P, Birney E and Huber W (2009). Mapping identifiers for the integration of genomic datasets with the R/Bioconductor
package biomaRt. Nature Protocols, 4, pp. 1184-1191.

Durinck S, Moreau Y, Kasprzyk A, Davis S, De Moor B, Brazma A and Huber W (2005).
BioMart and Bioconductor: a powerful link between biological databases and microarray data analysis. Bioinformatics, 21, pp. 3439-3440.

Fong, JH, Murphy, TD, Pruitt, KD (2013). Comparison of RefSeq protein-coding regions in human and vertebrate genomes. BMC Genomics, 14:654.

Henrik Bengtsson (2016). R.utils: Various Programming Utilities. R package version 2.5.0. https://CRAN.R-project.org/package=R.utils.

Herve Pages, Marc Carlson, Seth Falcon and Nianhua Li (2017). AnnotationDbi: Annotation Database Interface. R package version 1.38.0.

Matt Dowle and Arun Srinivasan (2017). data.table: Extension of data.frame. R package version 1.10.4. https://CRAN.R-project.org/package=data.table.
}
\seealso{
Other genomic_ressource: 
\code{\link{Bioconductor2GO}()},
\code{\link{Custom2GO}()},
\code{\link{Ensembl2GO}()},
\code{\link{EntrezGene2GO}()},
\code{\link{Uniprot2GO}()},
\code{\link{available_organisms}()},
\code{\link{genomic_ressource-class}},
\code{\link{taxonomy}()}

Other GO_terms: 
\code{\link{GOcount}()},
\code{\link{GOterms_heatmap}()},
\code{\link{create_topGOdata}()},
\code{\link{gene2GO-class}},
\code{\link{merge_enrich_terms}()},
\code{\link{runfgsea}()}
}
\concept{GO_terms}
\concept{genomic_ressource}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Upset.R
\name{Upset}
\alias{Upset}
\alias{Upset,ANY-method}
\title{Enriched GO terms intersections plot.}
\usage{
Upset(object, file = "./upset.xls")

\S4method{Upset}{ANY}(object, file = "./upset.xls")
}
\arguments{
\item{object}{an \code{\link{enrich_GO_terms-class}} or \code{\link{GO_clusters-class}} objects.}

\item{file}{output file name (default to "./upset.xls")}
}
\value{
print table and upset.
}
\description{
This method allows to visualize GO terms intersections between results of enrichment tests.
}
\details{
This function displays and print the intersections of enriched GO terms (p<0.01) between all results provided by \code{\link{enrich_GO_terms-class}}
or \code{\link{GO_clusters-class}} objects. The intersections are shown in an upset plot and printed in a table.
}
\examples{
# load example object
data(
    myGOs,
    package="ViSEAGO"
)

# print upset
ViSEAGO::Upset(myGOs)
}
\seealso{
Other enrich_GO_terms: 
\code{\link{enrich_GO_terms-class}},
\code{\link{overLapper}()},
\code{\link{show_heatmap}()},
\code{\link{show_table}()}

Other visualization: 
\code{\link{GOclusters_heatmap}()},
\code{\link{GOcount}()},
\code{\link{GOterms_heatmap}()},
\code{\link{available_organisms}()},
\code{\link{overLapper}()},
\code{\link{show_heatmap}()},
\code{\link{show_table}()}
}
\concept{enrich_GO_terms}
\concept{visualization}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/show_table.R
\name{show_table}
\alias{show_table}
\alias{show_table,ANY-method}
\title{Display an interactive or static table.}
\usage{
show_table(object, file = NULL)

\S4method{show_table}{ANY}(object, file = NULL)
}
\arguments{
\item{object}{an \code{\link{enrich_GO_terms-class}} object from \code{\link{merge_enrich_terms}},
or \code{\link{GO_clusters-class}} object from \code{\link{GOterms_heatmap}}.}

\item{file}{table output file name (default to NULL).}
}
\value{
display or print table
}
\description{
This method is used to display or print the table for \code{\link{enrich_GO_terms-class}} or \code{\link{GO_clusters-class}} objects.
}
\details{
This method displays an interactive table (if file=NULL) from \code{\link{enrich_GO_terms-class}} or \code{\link{GO_clusters-class}} objects.\cr
The table could be printed by setting file argument.
}
\examples{
# load example object
data(
    myGOs,
    package="ViSEAGO"
)

# display merge_enrich_terms output
ViSEAGO::show_table(myGOs)

# print merge_enrich_terms output
ViSEAGO::show_table(
    myGOs,
    "myGOs.txt"
)

\dontrun{
# compute GO terms Semantic Similarity distances
myGOs<-ViSEAGO::compute_SS_distances(
    distance="Wang"
)

# GOtermsHeatmap with default parameters
Wang_clusters_wardD2<-ViSEAGO::GOterms_heatmap(
    myGOs,
    showIC=TRUE,
    showGOlabels=TRUE,
    GO.tree=list(
        tree=list(
            distance="Wang",
            aggreg.method="ward.D2",
            rotate=NULL
        ),
        cut=list(
            dynamic=list(
                pamStage=TRUE,
                pamRespectsDendro=TRUE,
                deepSplit=2,
                minClusterSize =2
            )
        )
    ),
    samples.tree=NULL
)

# display table of GO_clusters-class object
ViSEAGO::show_table(Wang_clusters_wardD2)

# print table of GO_clusters-class object
ViSEAGO::show_table(
    Wang_clusters_wardD2,
    "Wang_clusters_wardD2.txt"
)
}
}
\references{
Yihui Xie (2016). DT: A Wrapper of the JavaScript Library 'DataTables'. R package version 0.2. https://CRAN.R-project.org/package=DT
}
\seealso{
Other enrich_GO_terms: 
\code{\link{Upset}()},
\code{\link{enrich_GO_terms-class}},
\code{\link{overLapper}()},
\code{\link{show_heatmap}()}

Other GO_clusters: 
\code{\link{GO_clusters-class}},
\code{\link{GOclusters_heatmap}()},
\code{\link{clusters_cor}()},
\code{\link{compare_clusters}()},
\code{\link{show_heatmap}()}

Other visualization: 
\code{\link{GOclusters_heatmap}()},
\code{\link{GOcount}()},
\code{\link{GOterms_heatmap}()},
\code{\link{Upset}()},
\code{\link{available_organisms}()},
\code{\link{overLapper}()},
\code{\link{show_heatmap}()}
}
\concept{GO_clusters}
\concept{enrich_GO_terms}
\concept{visualization}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/genomic_ressource.R
\docType{class}
\name{genomic_ressource-class}
\alias{genomic_ressource-class}
\title{genomic_ressource class object definition.}
\description{
This class stores the annotations and associated metadata obtained by \code{\link{Bioconductor2GO}},
\code{\link{EntrezGene2GO}}, \code{\link{Ensembl2GO}}, or \code{\link{Uniprot2GO}} .
}
\section{Slots}{

\describe{
\item{\code{db}}{name of database used (Bioconductor, EntrezGene, Ensembl, or Uniprot).}

\item{\code{stamp}}{date of stamp (for Bioconductor, EntrezGene, and Uniprot), or annotation version for Ensembl database.}

\item{\code{data}}{GO annotations from \code{\link{EntrezGene2GO}} method.}

\item{\code{organisms}}{informations about species/datasets availables.}

\item{\code{mart}}{Ensembl mart from \code{\link{Ensembl2GO}} method.}
}}

\seealso{
Other genomic_ressource: 
\code{\link{Bioconductor2GO}()},
\code{\link{Custom2GO}()},
\code{\link{Ensembl2GO}()},
\code{\link{EntrezGene2GO}()},
\code{\link{Uniprot2GO}()},
\code{\link{annotate}()},
\code{\link{available_organisms}()},
\code{\link{taxonomy}()}
}
\concept{genomic_ressource}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compare_clusters.R
\name{compare_clusters}
\alias{compare_clusters}
\alias{compare_clusters,list-method}
\title{Heatmap to compare partitions}
\usage{
compare_clusters(clusters)

\S4method{compare_clusters}{list}(clusters)
}
\arguments{
\item{clusters}{a \code{list} of named \code{\link{GO_clusters-class}} objects,
from \code{\link{GOterms_heatmap}} or \code{\link{GOclusters_heatmap}} methods.}
}
\value{
an interactive javascript heatmap.
}
\description{
Build an interactive heatmap of the common GO terms frequency between several partitions.
}
\details{
Build an interactive heatmap of common GO terms frequency between partitions from several
\code{\link{GO_clusters-class}} objects.
}
\examples{
# load example object
data(
    myGOs,
    package="ViSEAGO"
)

\dontrun{
# compute Semantic Similarity (SS)
myGOs<-ViSEAGO::compute_SS_distances(
    myGOs,
    distance=c("Resnik","Lin","Rel","Jiang","Wang")
)

# Resnik distance GO terms heatmap
Resnik_clusters_wardD2<-ViSEAGO::GOterms_heatmap(
    myGOs,
    showIC=TRUE,
    showGOlabels=TRUE,
    GO.tree=list(
        tree=list(
            distance="Resnik",
            aggreg.method="ward.D2"
        ),
        cut=list(
            dynamic=list(
                deepSplit=2,
                minClusterSize =2
            )
        )
    ),
    samples.tree=NULL
)

# Lin distance GO terms heatmap
Lin_clusters_wardD2<-ViSEAGO::GOterms_heatmap(
    myGOs,
    showIC=TRUE,
    showGOlabels=TRUE,
    GO.tree=list(
        tree=list(
            distance="Lin",
            aggreg.method="ward.D2"
        ),
        cut=list(
            dynamic=list(
                deepSplit=2,
                minClusterSize =2
            )
        )
    ),
    samples.tree=NULL
)

# Resnik distance GO terms heatmap
Rel_clusters_wardD2<-ViSEAGO::GOterms_heatmap(
    myGOs,
    showIC=TRUE,
    showGOlabels=TRUE,
    GO.tree=list(
        tree=list(
            distance="Rel",
            aggreg.method="ward.D2"
        ),
        cut=list(
            dynamic=list(
                deepSplit=2,
                minClusterSize =2
            )
        )
    ),
    samples.tree=NULL
)

# Resnik distance GO terms heatmap
Jiang_clusters_wardD2<-ViSEAGO::GOterms_heatmap(
    myGOs,
    showIC=TRUE,
    showGOlabels=TRUE,
    GO.tree=list(
        tree=list(
            distance="Jiang",
            aggreg.method="ward.D2"
        ),
        cut=list(
            dynamic=list(
                deepSplit=2,
                minClusterSize =2
            )
        )
    ),
    samples.tree=NULL
)

# Resnik distance GO terms heatmap
Wang_clusters_wardD2<-ViSEAGO::GOterms_heatmap(
    myGOs,
    showIC=TRUE,
    showGOlabels=TRUE,
    GO.tree=list(
        tree=list(
            distance="Wang",
            aggreg.method="ward.D2"
        ),
        cut=list(
            dynamic=list(
                deepSplit=2,
                minClusterSize =2
            )
        )
    ),
    samples.tree=NULL
)
}

# clusters to compare
clusters<-list(
    Resnik="Resnik_clusters_wardD2",
    Lin="Lin_clusters_wardD2",
    Rel="Rel_clusters_wardD2",
    Jiang="Jiang_clusters_wardD2",
    Wang="Wang_clusters_wardD2"
)

\dontrun{
# clusters content comparisons
clusters_comp<-ViSEAGO::compare_clusters(clusters)
}
}
\references{
Carson Sievert, Chris Parmer, Toby Hocking, Scott Chamberlain, Karthik Ram, Marianne Corvellec and Pedro Despouy (2017).
 plotly: Create Interactive Web Graphics via 'plotly.js'. R package version 4.6.0. https://CRAN.R-project.org/package=plotly
}
\seealso{
Other GO_clusters: 
\code{\link{GO_clusters-class}},
\code{\link{GOclusters_heatmap}()},
\code{\link{clusters_cor}()},
\code{\link{show_heatmap}()},
\code{\link{show_table}()}
}
\concept{GO_clusters}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/runfgsea.R
\name{runfgsea}
\alias{runfgsea}
\alias{runfgsea,ANY,gene2GO,character-method}
\title{perform multilevel preranked gene set enrichment analysis.}
\usage{
runfgsea(
  geneSel,
  gene2GO,
  ont,
  method = c("fgseaSimple", "fgseaMultilevel"),
  params = list(nperm = 10000, sampleSize = 101, minSize = 1, maxSize = Inf, eps = 0,
    scoreType = c("std", "pos", "neg"), nproc = 0, gseaParam = 1, BPPARAM = NULL, absEps
    = NULL)
)

\S4method{runfgsea}{ANY,gene2GO,character}(
  geneSel,
  gene2GO,
  ont,
  method = c("fgseaSimple", "fgseaMultilevel"),
  params = list(nperm = 10000, sampleSize = 101, minSize = 1, maxSize = Inf, eps = 0,
    scoreType = c("std", "pos", "neg"), nproc = 0, gseaParam = 1, BPPARAM = NULL, absEps
    = NULL)
)
}
\arguments{
\item{geneSel}{a 2 columns \code{data.table} with \bold{preranked} gene identifiants (in first column) based on the statistical values (second column).}

\item{gene2GO}{a \code{\link{gene2GO-class}} object created by \code{\link{annotate}} method.}

\item{ont}{the ontology used is "MF" (Molecuar Function), "BP" (Biological Process), or "CC" (Cellular Component).}

\item{method}{fgsea method to use with \code{\link[fgsea]{fgseaSimple}} or \code{\link[fgsea]{fgseaMultilevel}}.}

\item{params}{a \code{list} with \code{\link[fgsea]{fgseaSimple}} or \code{\link[fgsea]{fgseaMultilevel}} parameters.}
}
\value{
a \code{\link{fgsea-class}} object.
}
\description{
This method perform fast gene set enrichment analysis (GSEA) using \pkg{fgsea} package.
}
\details{
This method is a convenient wrapper using a given ontology category (\code{ont} argument)
in order to perform gene set enrichment analysis using \code{\link[fgsea]{fgseaSimple}} or
\code{\link[fgsea]{fgseaMultilevel}} algorithm from  \pkg{fgsea} package.

The complete GO annotation is required (\code{gene2GO} argument), and also a 2 columns \code{data.table} with \bold{preranked}
gene identifiants (in first column) based on statistical values (second column).

Defaults fgseaSimple parameters were used for perform test with \code{nperm} set to 10,000.\cr
Defaults fgseaMultilevel parameters were used for perform test 
except the \code{eps} arg that was set to 0 for better pvalues estimation.\cr
A gene frequency (\%) of leadingEdge/size is added to output \code{data.table}.
}
\examples{
# gene list
PregnantvsLactate<-data.table::fread(
    system.file(
        "extdata/data/input",
        "pregnantvslactate.complete.txt",
        package = "ViSEAGO"
    ),
    select = c("Id","padj")
)

# rank Id based on statistical value (padj here)
PregnantvsLactate<-data.table::setorder(PregnantvsLactate,padj)

\dontrun{
# connect to Bioconductor
Bioconductor<-ViSEAGO::Bioconductor2GO()

myGENE2GO<-ViSEAGO::annotate(
   "org.Mm.eg.db",
   Bioconductor
)

# run fgseaMultilevel
pregnantvslactate<-ViSEAGO::runfgsea(
    geneSel=PregnantvsLactate,
    gene2GO=myGENE2GO,
    ont="BP",
    method="fgseaMultilevel",
    params=list(
        minSize=5,
        scoreType="pos"
    )
)
}
}
\references{
Korotkevich G, Sukhov V, Sergushichev A (2019). "Fast gene set enrichment analysis." bioRxiv. doi: 10.1101/060012, http://biorxiv.org/content/early/2016/06/20/060012.
}
\seealso{
Other GO_terms: 
\code{\link{GOcount}()},
\code{\link{GOterms_heatmap}()},
\code{\link{annotate}()},
\code{\link{create_topGOdata}()},
\code{\link{gene2GO-class}},
\code{\link{merge_enrich_terms}()}
}
\concept{GO_terms}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GOterms_heatmap.R
\name{GOterms_heatmap}
\alias{GOterms_heatmap}
\alias{GOterms_heatmap,GO_SS-method}
\title{Build a clustering heatmap on GO terms.}
\usage{
GOterms_heatmap(
  myGOs,
  showIC = TRUE,
  showGOlabels = TRUE,
  heatmap_colors = c("#ffffff", "#99000D"),
  GO.tree = list(tree = list(distance = "Wang", aggreg.method = "ward.D2", rotate =
    NULL), cut = list(dynamic = list(pamStage = TRUE, pamRespectsDendro = TRUE, deepSplit
    = 2, minClusterSize = 2))),
  samples.tree = NULL
)

\S4method{GOterms_heatmap}{GO_SS}(
  myGOs,
  showIC = TRUE,
  showGOlabels = TRUE,
  heatmap_colors = c("#ffffff", "#99000D"),
  GO.tree = list(tree = list(distance = "Wang", aggreg.method = "ward.D2", rotate =
    NULL), cut = list(dynamic = list(pamStage = TRUE, pamRespectsDendro = TRUE, deepSplit
    = 2, minClusterSize = 2))),
  samples.tree = NULL
)
}
\arguments{
\item{myGOs}{a \code{\link{GO_SS-class}} object from \code{\link{compute_SS_distances}}.}

\item{showIC}{\code{logical} (default to TRUE) to display the GO terms Information Content (IC) side bar.}

\item{showGOlabels}{\code{logical} (default to TRUE) to display the GO terms ticks on y axis.}

\item{heatmap_colors}{pvalues color range with white to Sangria collors by default (c("#ffffff","#99000D")).}

\item{GO.tree}{a named \code{list} of parameters to build and cut the GO terms \code{dendrogram}.
 \describe{
     \item{tree (a named \code{list} with:)}{
         \describe{
             \item{distance ("Wang" by default)}{distance computed from the semantic similarity which could be
              IC-based ("Resnik", "Rel", "Lin", or "Jiang") or graph-based ("Wang").
             }
              \item{aggreg.method ("ward.D2" by default)}{aggregation method criteria from \code{\link[stats]{hclust}} ("ward.D",
              "ward.D2", "single", "complete", "average", "mcquitty", "median", or "centroid") to build a \code{dendrogram}.}
             \item{rotate}{sort the branches of the tree based on a vector - eithor of labels order or the labels in their new order}
         }
     }
     \item{cut (a named \code{list} with:)}{
         \describe{
             \item{static (default to NULL)}{a \code{numeric} value that is the height (between 0 and 1),
             or the number of clusters (value > 1) to cut the \code{dendrogram}.
             }
              \item{dynamic (a named \code{list} which only contains \code{\link[dynamicTreeCut]{cutreeDynamic}}
              options values below)
              }{
                \describe{
                    \item{pamStage (default to TRUE)}{second (PAM-like) stage will be performed.}
                    \item{pamRespectsDendro (default to TRUE)}{PAM stage will respect the dendrogram in the sense that objects
                     and small clusters will only be assigned to clusters that belong to the same branch that the objects or small
                     clusters being assigned belong to.}
                    \item{deepSplit (default to 2)}{provides a rough control over sensitivity for cluster splitting (range 0 to 4).
                      The higher the value (or if TRUE), the more and smaller clusters will be produced.}
                    \item{minClusterSize (default to 2)}{minimum cluster size.}
                }
             }
         }
     }
}}

\item{samples.tree}{a named \code{list} of parameters to build and cut the samples \code{dendrogram} (default to NULL).
 \describe{
     \item{tree (a named \code{list} with:)}{
         \describe{
             \item{distance ("pearson" by default)}{distance computed that could be correlation ("abs.pearson","pearson", "kendall", or "spearman"),
             or \code{dist} method (euclidean", "maximum", "manhattan", "canberra", "binary", or "minkowski).}
              \item{aggreg.method ("average" by default)}{same options than for \code{GO.tree} argument}
         }
     }
     \item{cut}{same options than for \code{GO.tree} argument.}
}}
}
\value{
a \code{\link{GO_clusters-class}} object.
}
\description{
This method computes a clustering heatmap based on GO terms semantic similarity.
}
\details{
This method computes a clustering heatmap based on GO terms semantic similarity (computed with \code{\link{compute_SS_distances}}).\cr
The dendrogram produced could be cutted in static or dynamic mode.\cr
 \enumerate{
  \item build dendrograms on  GO terms and optionally on samples.
  \item cut in static or dynamic mode and color the dendrogram branchs.
  \item build an interactive clustering heatmap based on \code{\link[heatmaply]{heatmaply}}.
 }
}
\examples{
# load data example
utils::data(
    myGOs,
    package="ViSEAGO"
)
\dontrun{
# compute GO terms Semantic Similarity distances
myGOs<-ViSEAGO::compute_SS_distances(
   myGOs,
   distance="Wang"
)

# GOtermsHeatmap with default parameters
Wang_clusters_wardD2<-ViSEAGO::GOterms_heatmap(
    myGOs,
    showIC=TRUE,
    showGOlabels=TRUE,
    GO.tree=list(
        tree=list(
            distance="Wang",
            aggreg.method="ward.D2",
            rotate=NULL
        ),
        cut=list(
            dynamic=list(
                pamStage=TRUE,
                pamRespectsDendro=TRUE,
                deepSplit=2,
                minClusterSize =2
            )
        )
    ),
    samples.tree=NULL
)
}
}
\references{
Matt Dowle and Arun Srinivasan (2017). data.table: Extension of `data.frame`. R package version 1.10.4. https://CRAN.R-project.org/package=data.table.

Tal Galili (2015). dendextend: an R package for visualizing, adjusting, and comparing trees of hierarchical clustering.
Bioinformatics. DOI:10.1093/bioinformatics/btv428.

Tal Galili (2017). heatmaply: Interactive Cluster Heat Maps Using 'plotly'.
R package version 0.9.1. https://CRAN.R-project.org/package=heatmaply.

Peter Langfelder, Bin Zhang and with contributions from Steve Horvath (2016). dynamicTreeCut: Methods for Detection of Clusters
in Hierarchical Clustering Dendrograms. R package version 1.63-1. https://CRAN.R-project.org/package=dynamicTreeCut.

Erich Neuwirth (2014). RColorBrewer: ColorBrewer Palettes. R package version 1.1-2. https://CRAN.R-project.org/package=RColorBrewer.

Carson Sievert, Chris Parmer, Toby Hocking, Scott Chamberlain, Karthik Ram, Marianne Corvellec and Pedro Despouy (2017).
plotly: Create Interactive Web Graphics via 'plotly.js'. R package version 4.6.0. https://CRAN.R-project.org/package=plotly.

Hadley Wickham (2016). scales: Scale Functions for Visualization. R package version 0.4.1. https://CRAN.R-project.org/package=scales.

H. Wickham. ggplot2: Elegant Graphics for Data Analysis. Springer-Verlag New York, 2009.
}
\seealso{
Other GO_terms: 
\code{\link{GOcount}()},
\code{\link{annotate}()},
\code{\link{create_topGOdata}()},
\code{\link{gene2GO-class}},
\code{\link{merge_enrich_terms}()},
\code{\link{runfgsea}()}

Other visualization: 
\code{\link{GOclusters_heatmap}()},
\code{\link{GOcount}()},
\code{\link{Upset}()},
\code{\link{available_organisms}()},
\code{\link{overLapper}()},
\code{\link{show_heatmap}()},
\code{\link{show_table}()}
}
\concept{GO_terms}
\concept{semantic_similiarity}
\concept{visualization}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/merge_enrich_terms.R
\name{merge_enrich_terms}
\alias{merge_enrich_terms}
\alias{merge_enrich_terms,list-method}
\title{Merge enriched GO terms.}
\usage{
merge_enrich_terms(Input, cutoff = 0.01, envir = .GlobalEnv)

\S4method{merge_enrich_terms}{list}(Input, cutoff = 0.01, envir = .GlobalEnv)
}
\arguments{
\item{Input}{a list containing named elements. Each element must contain the name of:
\itemize{
    \item{\pkg{topGO}: \code{\link[topGO]{topGOdata-class}} object created by \code{\link{create_topGOdata}} method and the associated \code{\link[topGO]{topGOresult-class}} object.}
    \item{\pkg{fgsea}: \code{\link{fgsea-class}} object created by \code{\link{runfgsea}} method.}
}}

\item{cutoff}{default pvalue cutoff (default to 0.01). Several cutoff can be use in the same order as list elements.}

\item{envir}{objects environment (default to .GlobalEnv).}
}
\value{
an \code{\link{enrich_GO_terms-class}} object.
}
\description{
combine results from GO enrichment tests (obtained with \pkg{topGO} package) or from \pkg{fgsea} (obtained with \code{\link{runfgsea}} method),
for a given ontology (MF, BP, or CC).
}
\details{
This method extracts for each result of GO enrichment test:
informations about GO term (identifiant, name, and description),
gene frequency (number of significant genes / Annotated genes), pvalue, -log10(pvalue), significant genes
identifiants (GeneID, or Ensembl ID, or uniprot accession), and gene symbols.
At the last, this method builds a merged data.table of enriched GO terms
at least once and provides all mentionned columns.
}
\examples{
## topGO terms enrichment

# load genes identifiants (GeneID,ENS...) universe/background (Expressed genes)
background_L<-scan(
    system.file(
        "extdata/data/input",
        "background_L.txt",
        package = "ViSEAGO"
    ),
    quiet=TRUE,
    what=""
)

# load Differentialy Expressed (DE) gene identifiants from files
PregnantvslactateDE<-scan(
    system.file(
        "extdata/data/input",
        "pregnantvslactateDE.txt",
        package = "ViSEAGO"
    ),
    quiet=TRUE,
    what=""
)

VirginvslactateDE<-scan(
    system.file(
        "extdata/data/input",
        "virginvslactateDE.txt",
        package = "ViSEAGO"
    ),
    quiet=TRUE,
    what=""
)

VirginvspregnantDE<-scan(
    system.file(
        "extdata/data/input",
        "virginvspregnantDE.txt",
        package="ViSEAGO"
    ),
    quiet=TRUE,
    what=""
)

\dontrun{
# connect to Bioconductor
Bioconductor<-ViSEAGO::Bioconductor2GO()

# load GO annotations from Bioconductor
myGENE2GO<-ViSEAGO::annotate(
    "org.Mm.eg.db",
    Bioconductor
)

# create topGOdata for BP for each list of DE genes
BP_Pregnantvslactate<-ViSEAGO::create_topGOdata(
    geneSel=PregnantvslactateDE,
    allGenes=background_L,
    gene2GO=myGENE2GO,
    ont="BP",
    nodeSize=5
)

BP_Virginvslactate<-ViSEAGO::create_topGOdata(
    geneSel=VirginvslactateDE,
    allGenes=background_L,
    gene2GO=myGENE2GO,
    ont="BP",
    nodeSize=5
)

BP_Virginvspregnant<-ViSEAGO::create_topGOdata(
    geneSel=VirginvspregnantDE,
    allGenes=background_L,
    gene2GO=myGENE2GO,
    ont="BP",
    nodeSize=5
)

# perform TopGO tests
elim_BP_Pregnantvslactate<-topGO::runTest(
    BP_L_pregnantvslactate,
    algorithm ="elim",
    statistic = "fisher"
)

elim_BP_Virginvslactate<-topGO::runTest(
    BP_L_virginvslactate,
    algorithm ="elim",
    statistic = "fisher"
)

elim_BP_Virginvspregnant<-topGO::runTest(
    BP_L_virginvspregnant,
    algorithm ="elim",
    statistic = "fisher"
)

# merge topGO results
BP_sResults<-ViSEAGO::merge_enrich_terms(
    Input=list(
        Pregnantvslactate=c("BP_Pregnantvslactate","elim_BP_Pregnantvslactate"),
        Virginvslactate=c("BP_Virginvslactate","elim_BP_Virginvslactate"),
        Virginvspregnant=c("BP_Virginvspregnant","elim_BP_Virginvspregnant")
    )
)
}

## fgsea analysis

# load gene identifiants and padj test results from Differential Analysis complete tables
PregnantvsLactate<-data.table::fread(
    system.file(
        "extdata/data/input",
        "pregnantvslactate.complete.txt",
        package = "ViSEAGO"
    ),
    select = c("Id","padj")
)

VirginvsLactate<-data.table::fread(
    system.file(
        "extdata/data/input",
        "virginvslactate.complete.txt",
        package = "ViSEAGO"
   ),
   select = c("Id","padj")
)

VirginvsPregnant<-data.table::fread(
    system.file(
       "extdata/data/input",
       "virginvspregnant.complete.txt",
        package = "ViSEAGO"
    ),
    select = c("Id","padj")
)

# rank Id based on statistical value (padj)
PregnantvsLactate<-data.table::setorder(PregnantvsLactate,padj)

VirginvsLactate<-data.table::setorder(VirginvsLactate,padj)

VirginvsPregnant<-data.table::setorder(VirginvsPregnant,padj)

\dontrun{
# connect to Bioconductor
Bioconductor<-ViSEAGO::Bioconductor2GO()

# load GO annotations from Bioconductor
myGENE2GO<-ViSEAGO::annotate(
    "org.Mm.eg.db",
    Bioconductor
)

# perform fgseaMultilevel tests
BP_PregnantvsLactate<-runfgsea(
    geneSel=PregnantvsLactate,
    gene2GO=myGENE2GO, 
    ont="BP",
    params = list(
        scoreType = "pos",
        minSize=5
    )
)

BP_VirginvsLactate<-runfgsea(
    geneSel=VirginvsLactate,
    gene2GO=myGENE2GO, 
    ont="BP",
    params = list(
        scoreType = "pos",
        minSize=5
    )
)

BP_VirginvsPregnant<-runfgsea(
    geneSel=VirginvsPregnant,
    gene2GO=myGENE2GO, 
    ont="BP",
    params = list(
        scoreType = "pos",
        minSize=5
    )
)

# merge fgsea results
BP_sResults<-merge_enrich_terms(
    cutoff=0.01,
    Input=list(
        PregnantvsLactate="BP_PregnantvsLactate",
        VirginvsLactate="BP_VirginvsLactate",
        VirginvsPregnant="BP_VirginvsPregnant"
    )
)
}
}
\references{
Matt Dowle and Arun Srinivasan (2017). data.table: Extension of data.frame. R package version 1.10.4. https://CRAN.R-project.org/package=data.table

Herve Pages, Marc Carlson, Seth Falcon and Nianhua Li (2017). AnnotationDbi: Annotation Database Interface. R package version 1.38.0.
}
\seealso{
Other GO_terms: 
\code{\link{GOcount}()},
\code{\link{GOterms_heatmap}()},
\code{\link{annotate}()},
\code{\link{create_topGOdata}()},
\code{\link{gene2GO-class}},
\code{\link{runfgsea}()}
}
\concept{GO_terms}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GOcount.R
\name{GOcount}
\alias{GOcount}
\alias{GOcount,ANY-method}
\title{Barplot for the count of GO terms.}
\usage{
GOcount(object, file = NULL)

\S4method{GOcount}{ANY}(object, file = NULL)
}
\arguments{
\item{object}{an \code{\link{enrich_GO_terms-class}} object from \code{\link{merge_enrich_terms}} method.}

\item{file}{the name of the output file (default to NULL for interactive screen display).}
}
\value{
a barplot.
}
\description{
This method displays in barplot the count of GO terms splitted in two categories (significant or not)
for each result of GO enrichment tests.
}
\details{
This method displays an interactive barplot, using \pkg{plotly} package, from a \code{\link{merge_enrich_terms}}
output object.\cr
A static image (in png) could be printed by setting \code{file} argument.
}
\examples{
# load object
utils::data(
 myGOs,
 package="ViSEAGO"
)

# barplot for the count of GO terms
ViSEAGO::GOcount( myGOs)
}
\references{
Carson Sievert, Chris Parmer, Toby Hocking, Scott Chamberlain, Karthik Ram, Marianne Corvellec and Pedro Despouy (2017).
plotly: Create InteractiveWeb Graphics via 'plotly.js'. R package version 4.6.0. https://CRAN.R-project.org/package=plotly.
}
\seealso{
Other GO_terms: 
\code{\link{GOterms_heatmap}()},
\code{\link{annotate}()},
\code{\link{create_topGOdata}()},
\code{\link{gene2GO-class}},
\code{\link{merge_enrich_terms}()},
\code{\link{runfgsea}()}

Other visualization: 
\code{\link{GOclusters_heatmap}()},
\code{\link{GOterms_heatmap}()},
\code{\link{Upset}()},
\code{\link{available_organisms}()},
\code{\link{overLapper}()},
\code{\link{show_heatmap}()},
\code{\link{show_table}()}
}
\concept{GO_terms}
\concept{visualization}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/available_organisms.R
\name{available_organisms}
\alias{available_organisms}
\alias{available_organisms,genomic_ressource-method}
\title{Display available organisms from a specified database.}
\usage{
available_organisms(object)

\S4method{available_organisms}{genomic_ressource}(object)
}
\arguments{
\item{object}{a  \code{\link{genomic_ressource-class}} object created by \code{\link{Bioconductor2GO}}, \code{\link{EntrezGene2GO}},
\code{\link{Ensembl2GO}},or \code{\link{Uniprot2GO}} methods.}
}
\value{
javascript datatable
}
\description{
Display an interactive table with available organisms from a genomic ressource database (Bioconductor, EntrezGene, Ensembl, Uniprot).
}
\details{
an interactive \code{datatable}.
}
\examples{
# display Bioconductor table
Bioconductor<-ViSEAGO::Bioconductor2GO()
ViSEAGO::available_organisms(Bioconductor)
\dontrun{

# display EntrezGene table
EntrezGene<-ViSEAGO::EntrezGene2GO()
ViSEAGO::available_organisms(EntrezGene)

# display Ensembl table
Ensembl<-ViSEAGO::Ensembl2GO()
ViSEAGO::available_organisms(Ensembl)

# display Uniprot table
Uniprot<-ViSEAGO::Uniprot2GO()
ViSEAGO::available_organisms(Uniprot)
}
}
\references{
Yihui Xie (2016). DT: A Wrapper of the JavaScript Library 'DataTables'. R package version 0.2. https://CRAN.R-project.org/package=DT
}
\seealso{
Other genomic_ressource: 
\code{\link{Bioconductor2GO}()},
\code{\link{Custom2GO}()},
\code{\link{Ensembl2GO}()},
\code{\link{EntrezGene2GO}()},
\code{\link{Uniprot2GO}()},
\code{\link{annotate}()},
\code{\link{genomic_ressource-class}},
\code{\link{taxonomy}()}

Other visualization: 
\code{\link{GOclusters_heatmap}()},
\code{\link{GOcount}()},
\code{\link{GOterms_heatmap}()},
\code{\link{Upset}()},
\code{\link{overLapper}()},
\code{\link{show_heatmap}()},
\code{\link{show_table}()}
}
\concept{genomic_ressource}
\concept{visualization}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compute_SS_distances.R
\name{compute_SS_distances}
\alias{compute_SS_distances}
\alias{compute_SS_distances,ANY,character-method}
\title{Compute distance between GO terms or GO clusters based on semantic similarity.}
\usage{
compute_SS_distances(object, distance)

\S4method{compute_SS_distances}{ANY,character}(object, distance)
}
\arguments{
\item{object}{a \code{\link{GO_SS-class}}, or  \code{\link{GO_clusters-class}} objects created by \code{\link{build_GO_SS}}
or \code{\link{GOterms_heatmap}} methods, respectively.}

\item{distance}{The available methods for calculating GO terms Semantic Similarity (SS) are
"Resnik", "Rel", "Lin", and "Jiang" which are based on Information Content (IC), and "Wang" which is based on graph topology.\cr
The available methods for calculating clusters of GO terms SS are "max", "avg","rcmax", and "BMA".}
}
\value{
a \code{\link{GO_SS-class}}, or a \code{\link{GO_clusters-class}} object (same class as input object).
}
\description{
This method computes distance between GO terms or GO clusters based on semantic similarity.
}
\details{
This method computes semantic similarity distances between all GO terms
provided by \code{\link{GO_SS-class}} object.\cr
This method also computes semantic similarity distances between all GO clusters
provided by \code{\link{GO_clusters-class}} object.\cr

Semantic Similarity computations are based on \code{\link[GOSemSim]{mgoSim}} method from the \pkg{GoSemSim} package.
}
\examples{
# load data example
data(
    myGOs,
    package="ViSEAGO"
)

\dontrun{
# compute GO terms Semantic Similarity distances
myGOs<-ViSEAGO::compute_SS_distances(
    myGOs,
    distance=c("Resnik","Lin","Rel","Jiang","Wang")
)

# GOtermsHeatmap with default parameters
Wang_clusters_wardD2<-ViSEAGO::GOterms_heatmap(
    myGOs,
    showIC=TRUE,
    showGOlabels=TRUE,
    GO.tree=list(
        tree=list(
            distance="Wang",
            aggreg.method="ward.D2",
            rotate=NULL
        ),
        cut=list(
            dynamic=list(
                pamStage=TRUE,
                pamRespectsDendro=TRUE,
                deepSplit=2,
                minClusterSize=2
            )
        )
    ),
    samples.tree=NULL
)

# compute clusters of GO terms Semantic Similarity distances
Wang_clusters_wardD2<-ViSEAGO::compute_SS_distances(
    Wang_clusters_wardD2,
    distance=c("max","avg","rcmax","BMA")
)
}
}
\references{
Marc Carlson (2017). GO.db: A set of annotation maps describing the entire Gene Ontology. R package version 3.4.1.

Guangchuang Yu, Fei Li, Yide Qin, Xiaochen Bo, Yibo Wu and Shengqi Wang. GOSemSim: an R package for measuring semantic similarity
among GO terms and gene products. Bioinformatics 2010 26(7):976-978

Herve Pages, Marc Carlson, Seth Falcon and Nianhua Li (2017). AnnotationDbi: Annotation Database Interface. R package version 1.38.0.
}
\seealso{
Other GO_semantic_similarity: 
\code{\link{GO_SS-class}},
\code{\link{build_GO_SS}()}
}
\concept{GO_semantic_similarity}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/create_topGOdata.R
\name{create_topGOdata}
\alias{create_topGOdata}
\alias{create_topGOdata,ANY,ANY,ANY,gene2GO,character,numeric-method}
\title{Create topGOdata object for enrichment test with topGO package.}
\usage{
create_topGOdata(geneSel, allGenes, geneList = NULL, gene2GO, ont, nodeSize)

\S4method{create_topGOdata}{ANY,ANY,ANY,gene2GO,character,numeric}(geneSel, allGenes, geneList = NULL, gene2GO, ont, nodeSize)
}
\arguments{
\item{geneSel}{genes of interest.}

\item{allGenes}{customized background genes.}

\item{geneList}{logical factor (1: genes of interest, 0: genes background,
and gene identifiants in \code{names}) (default value to NULL).}

\item{gene2GO}{a \code{\link{gene2GO-class}} object created by \code{\link{annotate}} method.}

\item{ont}{the ontology used is "MF" (Molecuar Function), "BP" (Biological Process), or "CC" (Cellular Component).}

\item{nodeSize}{the minimum number of genes for each GO term.}
}
\value{
a  \code{\link[topGO]{topGOdata-class}} object required by \code{runTest} from \pkg{topGO} package.
}
\description{
This method create a \code{\link[topGO]{topGOdata-class}} object required by \pkg{topGO} package in order
to perform GO enrichment test.
}
\details{
This method is a convenient wrapper building a \code{\link[topGO]{topGOdata-class}} object
using a given ontology category (\code{ont} argument) in order to perform GO enrichment test.
The complete GO annotation is required (\code{gene2GO} argument) and also
the list of genes of interest (\code{geneSel} argument) against the corresponding background (\code{allGenes} argument)
separately, or grouped together in a \code{factor} (\code{geneList} argument).
}
\examples{
 # load genes identifiants (GeneID,ENS...) background (Expressed genes)
 background<-scan(
  system.file(
   "extdata/data/input",
   "background_L.txt",
   package = "ViSEAGO"
  ),
  quiet=TRUE,
  what=""
 )

 # load Differentialy Expressed (DE) gene identifiants from files
 pregnantvslactateDE<-scan(
  system.file(
   "extdata/data/input",
   "pregnantvslactateDE.txt",
   package = "ViSEAGO"
 ),
  quiet=TRUE,
  what=""
 )

\dontrun{
# create topGOdata for BP for each list of DE genes
BP_L_pregnantvslactate<-ViSEAGO::create_topGOdata(
 geneSel=pregnantvslactateDE,
 allGenes=background,
 gene2GO=myGENE2GO,
 ont="BP",
 nodeSize=5
)
}
}
\references{
Alexa A, Rahnenfuhrer J, Lengauer T. Improved scoring of functional groups from gene expression data by
decorrelating GO graph structure. Bioinformatics 2006; 22:1600-1607.
}
\seealso{
Other GO_terms: 
\code{\link{GOcount}()},
\code{\link{GOterms_heatmap}()},
\code{\link{annotate}()},
\code{\link{gene2GO-class}},
\code{\link{merge_enrich_terms}()},
\code{\link{runfgsea}()}
}
\concept{GO_terms}
